@@ -9,7 +9,7 @@ ALPINE_ARCH="$4"
 IS_ROLLING_RELEASE="$5"
 IS_RELEASE="$6"
 MINIMAL_EXCLUDE_DEPS="$7"
-APK_RELEASE=1
+APK_RELEASE=0
 
 cd /tmp/liquidsoap-full/liquidsoap
 
@@ -17,6 +17,7 @@ APK_VERSION=$(opam show -f version ./liquidsoap.opam | cut -d'-' -f 1)
 COMMIT_SHORT=$(echo "${GITHUB_SHA}" | cut -c-7)
 
 export LIQUIDSOAP_BUILD_TARGET=posix
+export ABUILD_APK_INDEX_OPTS="--allow-untrusted"
 
 if [ -n "${IS_ROLLING_RELEASE}" ]; then
   APK_PACKAGE="liquidsoap-${COMMIT_SHORT}-${ALPINE_ARCH}"
@@ -36,7 +37,7 @@ sed -e "s#@APK_PACKAGE@#${APK_PACKAGE}#" liquidsoap/.github/alpine/APKBUILD.in |
   sed -e "s#@APK_RELEASE@#${APK_RELEASE}#" \
     > APKBUILD
 
-cp "liquidsoap/.github/alpine/liquidsoap.pre-install" "${APK_PACKAGE}.pre-install"
+cp "liquidsoap/.github/alpine/liquidsoap.post-install" "${APK_PACKAGE}.post-install"
 
 abuild-keygen -a -n
 abuild
@@ -60,7 +61,8 @@ rm -rf APKBUILD /home/opam/packages/tmp/"${ALPINE_ARCH}"
 
 echo "::group:: building ${APK_PACKAGE}-minimal.."
 
-eval "opam remove --no-depexts --force -y $MINIMAL_EXCLUDE_DEPS"
+# shellcheck disable=SC2086
+opam remove -y $MINIMAL_EXCLUDE_DEPS
 
 eval "$(opam config env)"
 
@@ -81,7 +83,7 @@ sed -e "s#@APK_PACKAGE@#${APK_PACKAGE}-minimal#" liquidsoap/.github/alpine/APKBU
   sed -e "s#@APK_RELEASE@#${APK_RELEASE}#" \
     > APKBUILD
 
-cp "liquidsoap/.github/alpine/liquidsoap.pre-install" "${APK_PACKAGE}-minimal.pre-install"
+cp "liquidsoap/.github/alpine/liquidsoap.post-install" "${APK_PACKAGE}-minimal.post-install"
 
 abuild-keygen -a -n
 abuild
