@@ -2,6 +2,8 @@
 
 set -e
 
+export PKG_CONFIG_PATH=/usr/src/mxe/usr/x86_64-w64-mingw32.static/lib/pkgconfig
+
 SYSTEM="$1"
 BRANCH="$2"
 CPU_CORES="$3"
@@ -42,20 +44,22 @@ export CC=""
 echo "::group::Installing deps"
 
 eval "$(opam config env)"
-opam repository set-url default https://github.com/ocaml/opam-repository.git
 opam repository set-url windows https://github.com/ocaml-cross/opam-cross-windows.git
 opam update windows
-# shellcheck disable=SC2046
-opam upgrade -y $(echo "$OPAM_DEPS" | sed -e 's#,# #g') ffmpeg-windows ffmpeg-avutil-windows
-opam remove -y pcre-windows
 
-# Debug
-opam reinstall -y cry-windows
+cd /tmp
+rm -rf ocaml-posix
+git clone https://github.com/savonet/ocaml-posix.git
+cd ocaml-posix
+opam pin -ny .
+opam install -y posix-socket.2.2.0 posix-base.2.2.0
+
+opam install -y srt-windows.0.3.3
 
 echo "::endgroup::"
 
 echo "::group::Install liquidsoap-windows"
-opam install -y liquidsoap-core-windows
+opam install -y liquidsoap-windows
 echo "::endgroup::"
 
 echo "::group::Save build config"
