@@ -23,3 +23,10 @@ let bin_dir () = get_site Sites.bin
 let bin_dir_descr = "(set by dune-site)"
 let camomile_dir () = Filename.dirname CamomileLib.Config.Default.datadir
 let camomile_dir_descr = "(set by dune-site)"
+let user_cache_override () = None
+let user_cache_override_descr = "$HOME/.cache/liquidsoap"
+
+let system_cache_override () =
+  match Sites.cache with [] -> None | d :: _ -> Some d
+
+let system_cache_override_descr = "(set by dune-site)"
