@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -98,7 +98,7 @@ let decode_audio_frame ~field ~mode generator =
     | `Frame frame ->
         let data, params =
           match Ffmpeg_copy_content.get_data frame with
-            | { Ffmpeg_content_base.data; params = Some (`Audio params) } ->
+            | { Content.Video.data; params = Some (`Audio params) } ->
                 (data, params)
             | _ -> assert false
         in
@@ -165,7 +165,7 @@ let decode_audio_frame ~field ~mode generator =
 
     function
     | `Frame frame ->
-        let { Ffmpeg_content_base.data; params } =
+        let { Content.Video.data; params } =
           Ffmpeg_raw_content.Audio.get_data frame
         in
         let data =
@@ -181,14 +181,14 @@ let decode_audio_frame ~field ~mode generator =
 
   let convert
         : 'a 'b.
-          get_data:(Content.data -> ('a, 'b) Ffmpeg_content_base.content) ->
+          get_data:(Content.data -> ('a, 'b) Content_video.Base.content) ->
           decoder:([ `Frame of Content.data | `Flush ] -> unit) ->
           [ `Frame of Frame.t | `Flush ] ->
           unit =
    fun ~get_data ~decoder -> function
     | `Frame frame ->
         let frame = Frame.get frame field in
-        let { Ffmpeg_content_base.data; _ } = get_data frame in
+        let { Content.Video.data; _ } = get_data frame in
         if data = [] then () else decoder (`Frame frame)
     | `Flush -> decoder `Flush
   in
@@ -253,8 +253,7 @@ let decode_video_frame ~field ~mode generator =
         Ffmpeg_utils.unpack_image ~width:internal_width ~height:internal_height
           (InternalScaler.convert scaler data)
       in
-      let data = Video.Canvas.single_image img in
-      let data = Content.Video.lift_data data in
+      let data = Content.Video.lift_image (Video.Canvas.Image.make img) in
       Generator.put generator field data
     in
 
@@ -286,9 +285,14 @@ let decode_video_frame ~field ~mode generator =
     let current_decoder = ref None in
 
     let mk_decoder ~params ~stream_idx ~time_base =
-      let codec_id = Avcodec.Video.get_params_id params in
+      let codec_id =
+        Avcodec.Video.get_params_id params.Ffmpeg_copy_content.params
+      in
       let codec = Avcodec.Video.find_decoder codec_id in
-      let decoder = Avcodec.Video.create_decoder ~params codec in
+      let decoder =
+        Avcodec.Video.create_decoder ~params:params.Ffmpeg_copy_content.params
+          codec
+      in
       current_decoder := Some decoder;
       current_stream_idx := Some stream_idx;
       current_params := Some params;
@@ -318,7 +322,7 @@ let decode_video_frame ~field ~mode generator =
     | `Frame frame ->
         let data, params =
           match Ffmpeg_copy_content.get_data frame with
-            | { Ffmpeg_content_base.data; params = Some (`Video params) } ->
+            | { Content.Video.data; params = Some (`Video params) } ->
                 (data, params)
             | _ -> assert false
         in
@@ -364,7 +368,7 @@ let decode_video_frame ~field ~mode generator =
     let last_params = ref None in
     function
     | `Frame frame ->
-        let { Ffmpeg_content_base.data; _ } =
+        let { Content.Video.data; _ } =
           Ffmpeg_raw_content.Video.get_data frame
         in
         let data =
@@ -385,14 +389,14 @@ let decode_video_frame ~field ~mode generator =
 
   let convert
         : 'a 'b.
-          get_data:(Content.data -> ('a, 'b) Ffmpeg_content_base.content) ->
+          get_data:(Content.data -> ('a, 'b) Content_video.Base.content) ->
           decoder:([ `Frame of Content.data | `Flush ] -> unit) ->
           [ `Frame of Frame.t | `Flush ] ->
           unit =
    fun ~get_data ~decoder -> function
     | `Frame frame ->
         let frame = Frame.get frame field in
-        let { Ffmpeg_content_base.data; _ } = get_data frame in
+        let { Content.Video.data; _ } = get_data frame in
         if data = [] then () else decoder (`Frame frame)
     | `Flush -> decoder `Flush
   in
@@ -437,11 +441,6 @@ let mk_decoder mode =
            Lang.to_default_option ~default:name Lang.to_string
              (List.assoc "id" p)
          in
-         let pos =
-           match Liquidsoap_lang.Lang_core.pos p with
-             | [] -> None
-             | p :: _ -> Some p
-         in
          let field, source = Lang.to_track (List.assoc "" p) in
 
          let mk_decode_frame generator =
@@ -460,7 +459,7 @@ let mk_decoder mode =
                    (Frame.get_all_metadata frame);
                  List.iter
                    (fun pos -> Generator.add_track_mark ~pos generator)
-                   (List.filter (fun x -> x < size) (Frame.breaks frame));
+                   (List.filter (fun x -> x < size) (Frame.track_marks frame));
                  decode_frame (`Frame frame)
              | `Flush -> decode_frame `Flush
            in
@@ -493,10 +492,10 @@ let mk_decoder mode =
              ~always_enabled:true ~write_frame:decode_frame
              ~name:(id ^ ".consumer") ~source:(Lang.source source) ()
          in
-         consumer#set_pos pos;
+         let stack = Liquidsoap_lang.Lang_core.pos p in
+         consumer#set_stack stack;
 
-         let input_frame_t = Typing.generalize ~level:(-1) input_frame_t in
-         let input_frame_t = Typing.instantiate ~level:(-1) input_frame_t in
+         let input_frame_t = Type.fresh input_frame_t in
          Typing.(
            consumer#frame_type
            <: Lang.frame_t (Lang.univ_t ())
@@ -505,8 +504,8 @@ let mk_decoder mode =
          ( field,
            new Producer_consumer.producer
            (* We are expecting real-rate with a couple of hickups.. *)
-             ?pos ~create_known_clock:false ~check_self_sync:false
-             ~consumers:[consumer] ~name:(id ^ ".producer") () )))
+             ~stack ~check_self_sync:false ~consumers:[consumer]
+             ~name:(id ^ ".producer") () )))
 
 let () =
   List.iter mk_decoder [`Audio_encoded; `Audio_raw; `Video_encoded; `Video_raw]
