@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -30,15 +30,12 @@ let () =
       ignore
         (Lang.add_builtin_base ~category:`Configuration
            ~descr:(Printf.sprintf "Liquidsoap's %s." kind)
-           ~base:configure name
-           Lang.(Ground (Ground.String str))
-           Lang.string_t))
+           ~base:configure name (`String str) Lang.string_t))
     [
       ("libdir", "library directory", Configure.liq_libs_dir ());
       ("bindir", "Internal script directory", Configure.bin_dir ());
       ("rundir", "PID file directory", Configure.rundir ());
       ("logdir", "logging directory", Configure.logdir ());
-      ("default_font", "default font file", Configure.default_font);
     ]
 
 (** Liquidsoap stuff *)
@@ -54,12 +51,6 @@ let conf_strip_types_types =
     ~p:(conf_runtime#plug "strip_types")
     ~d:true "Strip runtime types whenever possible to optimize memory usage."
 
-let () =
-  Lifecycle.after_script_parse ~name:"strip types and cleanup memory" (fun () ->
-      if conf_strip_types_types#get then (
-        Liquidsoap_lang.Term.trim_runtime_types ();
-        Gc.full_major ()))
-
 let _ =
   let kind = Lang.univ_t () in
   Lang.add_builtin ~category:`Liquidsoap ~base:encoder "content_type"
@@ -225,7 +216,8 @@ let _ =
     let a = Lang.to_string (Lang.assoc "" 2 p) in
     let s = match a with "" -> c | _ -> c ^ " " ^ a in
     let r = try Server.exec s with Not_found -> "Command not found!" in
-    Lang.list (List.map Lang.string (Pcre.split ~pat:"\r?\n" r))
+    Lang.list
+      (List.map Lang.string (Re.Pcre.split ~rex:(Re.Pcre.regexp "\r?\n") r))
   in
   Lang.add_builtin ~base:Modules.server "execute" ~category ~descr params
     return_t execute
