@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -62,15 +62,16 @@ type fps = Decoder_utils.fps = { num : int; den : int }
 type buffer = {
   generator : Generator.t;
   put_pcm : ?field:Frame.field -> samplerate:int -> Content.Audio.data -> unit;
-  put_yuva420p : ?field:Frame.field -> fps:fps -> Content.Video.data -> unit;
+  put_yuva420p : ?field:Frame.field -> fps:fps -> Video.Canvas.image -> unit;
 }
 
 type decoder = {
   decode : buffer -> unit;
   eof : buffer -> unit;
-  (* [seek x]: Skip [x] main ticks. Returns the number of ticks atcually
+  (* [seek x]: Skip [x] main ticks. Returns the number of ticks actually
      skipped. *)
   seek : int -> int;
+  close : unit -> unit;
 }
 
 type input = {
@@ -82,16 +83,11 @@ type input = {
   length : (unit -> int) option;
 }
 
-(** A decoder is a filling function and a closing function, called at least when
-    filling fails, i.e. the frame is partial. The closing function can be called
-    earlier e.g. if the user skips. In most cases, file decoders are wrapped
-    stream decoders. *)
 type file_decoder_ops = {
-  fill : Frame.t -> int;
-  (* Return remaining ticks. *)
+  fread : int -> Frame.t;
+  remaining : unit -> int;
   fseek : int -> int;
-  (* There is a record name clash here.. *)
-  close : unit -> unit;
+  fclose : unit -> unit;
 }
 
 type file_decoder =
@@ -106,7 +102,11 @@ type file_decoder =
     close function. *)
 type stream_decoder = input -> decoder
 
-type image_decoder = file -> Video.Image.t
+type image_decoder = {
+  image_decoder_priority : unit -> int;
+  check_image : file -> bool;
+  decode_image : file -> Video.Image.t;
+}
 
 (** Decoder description. *)
 type decoder_specs = {
@@ -159,7 +159,7 @@ let conf_image_file_decoders =
     ~p:(conf_decoder#plug "image_file_decoders")
     ~d:[] "Decoders and order used to decode image files."
 
-let image_file_decoders : (file -> Video.Image.t option) Plug.t =
+let image_file_decoders : image_decoder Plug.t =
   Plug.create
     ~register_hook:(fun name _ -> f conf_image_file_decoders name)
     ~doc:"Image file decoding methods." "image file decoding"
@@ -172,7 +172,10 @@ let get_image_file_decoders () =
           log#severe "Cannot find decoder %s" name;
           cur
   in
-  List.fold_left f [] conf_image_file_decoders#get
+  List.sort
+    (fun (_, { image_decoder_priority = p })
+         (_, { image_decoder_priority = p' }) -> Stdlib.compare (p' ()) (p ()))
+    (List.fold_left f [] conf_image_file_decoders#get)
 
 let conf_debug =
   Dtools.Conf.bool
@@ -210,6 +213,11 @@ let conf_priorities =
     ~p:(conf_decoder#plug "priorities")
     "Priorities used for choosing audio and video file decoders"
 
+let conf_image_priorities =
+  Dtools.Conf.void
+    ~p:(conf_priorities#plug "image")
+    "Priorities used for choosing image file decoders"
+
 let base_mime s = List.hd (String.split_on_char ';' s)
 
 let test_file ~(log : Log.t) ~extension ~mime ~mimes ~extensions fname =
@@ -239,7 +247,7 @@ let test_file ~(log : Log.t) ~extension ~mime ~mimes ~extensions fname =
   ext_ok || mime_ok
 
 let channel_layout audio =
-  Lazy.force Content.(Audio.(get_params audio).Content.channel_layout)
+  Lazy.force Content.(Audio.(get_params audio).Content.Audio.channel_layout)
 
 let can_decode_type decoded_type target_type =
   let map_convertible cur (field, target_field) =
@@ -337,33 +345,42 @@ let get_file_decoder ~metadata ~ctype filename =
             fun () -> (Option.get specs.file_decoder) ~metadata ~ctype filename
           )))
 
+(** Check if decoder can decode image. *)
+let check_image_file_decoder filename =
+  let decoders = get_image_file_decoders () in
+  log#info "Available image decoders: %s"
+    (String.concat ", "
+       (List.map
+          (fun (name, specs) ->
+            Printf.sprintf "%s (priority: %d)" name
+              (specs.image_decoder_priority ()))
+          decoders));
+  List.exists
+    (fun (name, { image_decoder_priority; check_image }) ->
+      log#info "Trying image decoder %S (priority: %d)" name
+        (image_decoder_priority ());
+      if check_image filename then (
+        log#important "Image decoder %S accepted %s" name
+          (Lang_string.quote_string filename);
+        true)
+      else false)
+    decoders
+
 (** Get a valid image decoder creator for [filename]. *)
 let get_image_file_decoder filename =
-  let ans = ref None in
-  try
-    List.iter
-      (fun (name, decoder) ->
+  match
+    List.find_map
+      (fun (name, { decode_image }) ->
         log#info "Trying method %S for %s..." name
           (Lang_string.quote_string filename);
-        match
-          try decoder filename
-          with e ->
-            log#info "Decoder %S failed on %s: %s!" name
-              (Lang_string.quote_string filename)
-              (Printexc.to_string e);
-            None
-        with
-          | Some img ->
-              log#important "Method %S accepted %s." name
-                (Lang_string.quote_string filename);
-              ans := Some img;
-              raise Stdlib.Exit
-          | None -> ())
-      (get_image_file_decoders ());
-    log#important "Unable to decode %s using image decoder(s)!"
-      (Lang_string.quote_string filename);
-    !ans
-  with Exit -> !ans
+        try Some (decode_image filename) with _ -> None)
+      (get_image_file_decoders ())
+  with
+    | None ->
+        log#important "Unable to decode %s using image decoder(s)!"
+          (Lang_string.quote_string filename);
+        raise Not_found
+    | Some d -> d
 
 let get_stream_decoder ~ctype mime =
   let decoders =
@@ -386,11 +403,12 @@ let get_stream_decoder ~ctype mime =
       (Frame.string_of_content_type ctype);
     None)
   else (
-    log#info "Available decoders:";
-    List.iter
-      (fun (name, specs) ->
-        log#info "%s (priority: %d)" name (specs.priority ()))
-      decoders;
+    log#info "Available stream decoders: %s"
+      (String.concat ", "
+         (List.map
+            (fun (name, specs) ->
+              Printf.sprintf "%s (priority: %d)" name (specs.priority ()))
+            decoders));
     let name, decoder = List.hd decoders in
     log#info "Selected decoder %s for mime-type %s with expected content %s"
       name mime
@@ -433,7 +451,7 @@ let mk_buffer ~ctype generator =
             Generator.put generator field data)
         else fun ~samplerate:_ _ -> ()
       in
-      Hashtbl.add audio_handlers field handler;
+      Hashtbl.replace audio_handlers field handler;
       handler
   in
 
@@ -457,19 +475,31 @@ let mk_buffer ~ctype generator =
           let out_freq =
             Decoder_utils.{ num = Lazy.force Frame.video_rate; den = 1 }
           in
-          fun ~fps (data : Content.Video.data) ->
-            let data = Array.map video_scale data in
-            let data = video_resample ~in_freq:fps ~out_freq data in
-            let len = Video.Canvas.length data in
-            let data =
-              Content.Video.lift_data
-                ~length:(Frame_settings.main_of_video len)
-                data
-            in
-            Generator.put generator field data)
+          let params =
+            {
+              Content.Video.width = Some Frame.video_width;
+              height = Some Frame.video_height;
+            }
+          in
+          let interval = Frame.main_of_video 1 in
+          fun ~fps img ->
+            match video_resample ~in_freq:fps ~out_freq img with
+              | [] -> ()
+              | data ->
+                  let data =
+                    List.mapi
+                      (fun i img -> (i * interval, video_scale img))
+                      data
+                  in
+                  let length = List.length data * interval in
+                  let buf =
+                    Content.Video.lift_data
+                      { Content.Video.params; length; data }
+                  in
+                  Generator.put generator field buf)
         else fun ~fps:_ _ -> ()
       in
-      Hashtbl.add video_handlers field handler;
+      Hashtbl.replace video_handlers field handler;
       handler
   in
 
@@ -477,31 +507,33 @@ let mk_buffer ~ctype generator =
     get_audio_handler ~field ~samplerate data
   in
 
-  let put_yuva420p ?(field = Frame.Fields.video) ~fps data =
-    get_video_handler ~field ~fps data
+  let put_yuva420p ?(field = Frame.Fields.video) ~fps img =
+    get_video_handler ~field ~fps img
   in
 
   { generator; put_pcm; put_yuva420p }
 
-let mk_decoder ~filename ~close ~remaining ~buffer decoder =
-  let prebuf = Frame.main_of_seconds 0.5 in
+let mk_decoder ~filename ~remaining ~buffer decoder =
   let decoding_done = ref false in
 
-  let remaining frame offset =
-    remaining ()
-    + Generator.length buffer.generator
-    + Frame.position frame - offset
+  let remaining () =
+    try remaining () + Generator.length buffer.generator
+    with e ->
+      log#info "Error while getting decoder's remaining time: %s"
+        (Printexc.to_string e);
+      decoding_done := true;
+      0
   in
 
-  let close () =
+  let fclose () =
     decoder.eof buffer;
-    close ()
+    decoder.close ()
   in
 
-  let fill frame =
+  let fread size =
     if not !decoding_done then (
       try
-        while Generator.length buffer.generator < prebuf do
+        while Generator.length buffer.generator < size do
           decoder.decode buffer
         done
       with e ->
@@ -513,15 +545,7 @@ let mk_decoder ~filename ~close ~remaining ~buffer decoder =
         decoding_done := true;
         if conf_debug#get then raise e);
 
-    let offset = Frame.position frame in
-    Generator.fill buffer.generator frame;
-
-    try remaining frame offset
-    with e ->
-      log#info "Error while getting decoder's remaining time: %s"
-        (Printexc.to_string e);
-      decoding_done := true;
-      0
+    Generator.slice buffer.generator size
   in
 
   let fseek len =
@@ -534,15 +558,19 @@ let mk_decoder ~filename ~close ~remaining ~buffer decoder =
       Generator.truncate buffer.generator len;
       len)
   in
-  { fill; fseek; close }
+  { fread; remaining; fseek; fclose }
 
-let file_decoder ~filename ~close ~remaining ~ctype decoder =
-  let generator = Generator.create ~log:(log#info "%s") ctype in
+let file_decoder ~filename ~remaining ~ctype decoder =
+  let generator = Generator.create ~log ctype in
   let buffer = mk_buffer ~ctype generator in
-  mk_decoder ~filename ~close ~remaining ~buffer decoder
+  mk_decoder ~filename ~remaining ~buffer decoder
+
+let openfile filename =
+  let extra_flags = if Sys.win32 then [Unix.O_SHARE_DELETE] else [] in
+  Unix.openfile filename ([Unix.O_RDONLY; Unix.O_CLOEXEC] @ extra_flags) 0
 
 let opaque_file_decoder ~filename ~ctype create_decoder =
-  let fd = Unix.openfile filename [Unix.O_RDONLY; Unix.O_CLOEXEC] 0 in
+  let fd = openfile filename in
 
   let file_size = (Unix.stat filename).Unix.st_size in
   let proc_bytes = ref 0 in
@@ -562,7 +590,7 @@ let opaque_file_decoder ~filename ~ctype create_decoder =
     { read; tell = Some tell; length = Some length; lseek = Some lseek }
   in
 
-  let generator = Generator.create ~log:(log#info "%s") ctype in
+  let generator = Generator.create ~log ctype in
   let buffer = mk_buffer ~ctype generator in
   let decoder = create_decoder input in
 
@@ -574,7 +602,7 @@ let opaque_file_decoder ~filename ~ctype create_decoder =
     out_ticks := !out_ticks + stop - start
   in
 
-  let decoder = { decoder with decode } in
+  let decoder = { decoder with decode; close = (fun () -> Unix.close fd) } in
 
   let remaining () =
     let in_bytes = tell () in
@@ -587,6 +615,4 @@ let opaque_file_decoder ~filename ~ctype create_decoder =
       int_of_float remaining_ticks)
   in
 
-  let close () = Unix.close fd in
-
-  mk_decoder ~filename ~close ~remaining ~buffer decoder
+  mk_decoder ~filename ~remaining ~buffer decoder
