@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 
  *****************************************************************************)
 
@@ -49,8 +49,8 @@ type fps = { num : int; den : int }
     - Implicit content drop *)
 type buffer = {
   generator : Generator.t;
-  put_pcm : ?field:Frame.field -> samplerate:int -> Content.Audio.data -> unit;
-  put_yuva420p : ?field:Frame.field -> fps:fps -> Content.Video.data -> unit;
+  put_pcm : ?field:Frame.field -> samplerate:int -> Audio.t -> unit;
+  put_yuva420p : ?field:Frame.field -> fps:fps -> Video.Canvas.image -> unit;
 }
 
 type decoder = {
@@ -59,16 +59,23 @@ type decoder = {
   (* [seek x]: Skip [x] main ticks.
    * Returns the number of ticks atcually skipped. *)
   seek : int -> int;
-}
-
-type file_decoder_ops = {
-  fill : Frame.t -> int;
-  fseek : int -> int;
   close : unit -> unit;
 }
 
+type file_decoder_ops = {
+  fread : int -> Frame.t;
+  remaining : unit -> int;
+  fseek : int -> int;
+  fclose : unit -> unit;
+}
+
 type stream_decoder = input -> decoder
-type image_decoder = file -> Video.Image.t
+
+type image_decoder = {
+  image_decoder_priority : unit -> int;
+  check_image : file -> bool;
+  decode_image : file -> Video.Image.t;
+}
 
 type file_decoder =
   metadata:Frame.metadata ->
@@ -103,6 +110,10 @@ val conf_decoder : Dtools.Conf.ut
 val conf_mime_types : Dtools.Conf.ut
 val conf_file_extensions : Dtools.Conf.ut
 val conf_priorities : Dtools.Conf.ut
+val conf_image_priorities : Dtools.Conf.ut
+
+(** Open file with readonly, cloexec and share delete on windows. *)
+val openfile : string -> Unix.file_descr
 
 (** Test file extension and mime if available *)
 val test_file :
@@ -127,8 +138,9 @@ val get_file_decoder :
 val get_stream_decoder :
   ctype:Frame.content_type -> string -> stream_decoder option
 
-val image_file_decoders : (file -> Video.Image.t option) Plug.t
-val get_image_file_decoder : file -> Video.Image.t option
+val image_file_decoders : image_decoder Plug.t
+val check_image_file_decoder : file -> bool
+val get_image_file_decoder : file -> Video.Image.t
 
 (* Initialize a decoding buffer *)
 val mk_buffer : ctype:Frame.content_type -> Generator.t -> buffer
@@ -136,7 +148,6 @@ val mk_buffer : ctype:Frame.content_type -> Generator.t -> buffer
 (* Create a file decoder when remaining time is known. *)
 val file_decoder :
   filename:string ->
-  close:(unit -> unit) ->
   remaining:(unit -> int) ->
   ctype:Frame.content_type ->
   decoder ->
