@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-   Liquidsoap, a programmable audio stream generator.
-   Copyright 2003-2023 Savonet team
+   Liquidsoap, a programmable stream generator.
+   Copyright 2003-2024 Savonet team
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -509,29 +509,31 @@ let priority =
     ~p:(Decoder.conf_priorities#plug "ffmpeg")
     "Priority for the ffmpeg decoder" ~d:10
 
-let parse_encoder_params s =
-  let lexbuf = Sedlexing.Utf8.from_string ("(" ^ s ^ ")") in
+let parse_encoder_params =
   let processor =
     MenhirLib.Convert.Simplified.traditional2revised
       Liquidsoap_lang.Parser.plain_encoder_params
   in
-  let tokenizer = Liquidsoap_lang.Preprocessor.mk_tokenizer ~pwd:"" lexbuf in
-  processor tokenizer
+  fun s ->
+    let lexbuf = Sedlexing.Utf8.from_string ("(" ^ s ^ ")") in
+    let tokenizer = Liquidsoap_lang.Preprocessor.mk_tokenizer lexbuf in
+    Liquidsoap_lang.Term_reducer.to_encoder_params (processor tokenizer)
 
 let parse_input_args args =
   try
     let args = parse_encoder_params args in
     List.fold_left
       (fun (args, format) -> function
-        | "f", `Term Term.{ term = Var format; _ }
-        | "format", `Term Term.{ term = Var format; _ } ->
+        | `Labelled ("f", Term.{ term = `Var format; _ })
+        | `Labelled ("format", Term.{ term = `Var format; _ }) ->
             (args, Av.Format.find_input_format format)
-        | k, `Term Term.{ term = Var v; _ } -> ((k, `String v) :: args, format)
-        | k, `Term Term.{ term = Ground (Ground.String s); _ } ->
+        | `Labelled (k, Term.{ term = `Var v; _ }) ->
+            ((k, `String v) :: args, format)
+        | `Labelled (k, Term.{ term = `String s; _ }) ->
             ((k, `String s) :: args, format)
-        | k, `Term Term.{ term = Ground (Ground.Int i); _ } ->
+        | `Labelled (k, Term.{ term = `Int i; _ }) ->
             ((k, `Int i) :: args, format)
-        | k, `Term Term.{ term = Ground (Ground.Float f); _ } ->
+        | `Labelled (k, Term.{ term = `Float f; _ }) ->
             ((k, `Float f) :: args, format)
         | _ -> assert false)
       ([], None) args
@@ -540,14 +542,14 @@ let parse_input_args args =
       "ffmpeg_decoder"
 
 let parse_file_decoder_args metadata =
-  match Hashtbl.find_opt metadata "ffmpeg_options" with
+  match Frame.Metadata.find_opt "ffmpeg_options" metadata with
     | Some args -> parse_input_args args
     | None -> ([], None)
 
-let duration ~metadata file =
+let dresolver ~metadata file =
   let args, format = parse_file_decoder_args metadata in
   let opts = Hashtbl.create 10 in
-  List.iter (fun (k, v) -> Hashtbl.add opts k v) args;
+  List.iter (fun (k, v) -> Hashtbl.replace opts k v) args;
   let container = Av.open_input ?format ~opts file in
   Fun.protect
     ~finally:(fun () -> Av.close container)
@@ -556,10 +558,16 @@ let duration ~metadata file =
       Option.map (fun d -> Int64.to_float d /. 1000.) duration)
 
 let () =
-  Plug.register Request.dresolvers "ffmepg" ~doc:"" (fun ~metadata fname ->
-      match duration ~metadata fname with
-        | None -> raise Not_found
-        | Some d -> d)
+  Plug.register Request.dresolvers "ffmpeg" ~doc:""
+    {
+      dpriority = (fun () -> priority#get);
+      file_extensions = (fun () -> file_extensions#get);
+      dresolver =
+        (fun ~metadata fname ->
+          match dresolver ~metadata fname with
+            | None -> raise Not_found
+            | Some d -> d);
+    }
 
 let tags_substitutions = [("track", "tracknumber")]
 
@@ -572,7 +580,7 @@ let get_tags ~metadata ~extension ~mime file =
     then raise Invalid_file;
     let args, format = parse_file_decoder_args metadata in
     let opts = Hashtbl.create 10 in
-    List.iter (fun (k, v) -> Hashtbl.add opts k v) args;
+    List.iter (fun (k, v) -> Hashtbl.replace opts k v) args;
     let container = Av.open_input ?format ~opts file in
     Fun.protect
       ~finally:(fun () -> Av.close container)
@@ -598,7 +606,17 @@ let get_tags ~metadata ~extension ~mime file =
              (Printexc.to_string e));
         raise Not_found
 
-let () = Plug.register Request.mresolvers "ffmpeg" ~doc:"" get_tags
+let metadata_decoder_priority =
+  Dtools.Conf.int
+    ~p:(Request.conf_metadata_decoder_priorities#plug "ffmpeg")
+    "Priority for the ffmpeg metadata decoder" ~d:1
+
+let () =
+  Plug.register Request.mresolvers "ffmpeg" ~doc:""
+    {
+      Request.priority = (fun () -> metadata_decoder_priority#get);
+      resolver = get_tags;
+    }
 
 (* Get the type of an input container. *)
 let get_type ~ctype ~format ~url container =
@@ -639,7 +657,7 @@ let get_type ~ctype ~format ~url container =
   in
   let video_streams, descriptions =
     List.fold_left
-      (fun (video_streams, descriptions) (_, _, params) ->
+      (fun (video_streams, descriptions) (_, stream, params) ->
         try
           let field = Frame.Fields.video_n (List.length video_streams) in
           let width = Avcodec.Video.get_width params in
@@ -660,7 +678,8 @@ let get_type ~ctype ~format ~url container =
               (Frame.Fields.string_of_field field)
               codec_name width height pixel_format
           in
-          (video_streams @ [(field, params)], descriptions @ [description])
+          ( video_streams @ [(field, Av.get_avg_frame_rate stream, params)],
+            descriptions @ [description] )
         with Avutil.Error _ as exn ->
           let bt = Printexc.get_raw_backtrace () in
           Utils.log_exception ~log
@@ -723,12 +742,14 @@ let get_type ~ctype ~format ~url container =
   in
   let content_type =
     List.fold_left
-      (fun content_type (field, params) ->
+      (fun content_type (field, avg_frame_rate, params) ->
         match (params, Frame.Fields.find_opt field ctype) with
-          | p, Some format when Ffmpeg_copy_content.is_format format ->
+          | params, Some format when Ffmpeg_copy_content.is_format format ->
               ignore
                 (Content.merge format
-                   (Ffmpeg_copy_content.lift_params (Some (`Video p))));
+                   (Ffmpeg_copy_content.lift_params
+                      (Some
+                         (`Video { Ffmpeg_copy_content.avg_frame_rate; params }))));
               Frame.Fields.add field format content_type
           | p, Some format when Ffmpeg_raw_content.Video.is_format format ->
               ignore
@@ -775,24 +796,26 @@ let mk_decoder ~streams ~target_position container =
     Int64.to_float pts *. float num /. float den
   in
   let decodable = ref [] in
-  let push (position, decode) =
+  let push (position, ts, decode) =
     decodable :=
-      (position, decode)
+      (position, ts, decode)
       :: List.filter
-           (fun (p, _) ->
+           (fun (p, _, _) ->
              Float.abs (p -. position)
              <= Ffmpeg_decoder_common.conf_max_interleave_duration#get)
            !decodable
   in
   let flush position =
-    let d = List.sort (fun (p, _) (p', _) -> Float.compare p p') !decodable in
+    let d =
+      List.sort (fun (_, p, _) (_, p', _) -> Int64.compare p p') !decodable
+    in
     let min_position =
       position -. Ffmpeg_decoder_common.conf_max_interleave_delta#get
     in
-    List.iter (fun (p, decode) -> if min_position <= p then decode ()) d;
+    List.iter (fun (p, _, decode) -> if min_position <= p then decode ()) d;
     decodable := []
   in
-  let check_pts ~decode stream pts =
+  let check_pts ~decode ~ts stream pts =
     match (pts, !target_position) with
       | Some pts, Some target_position ->
           if target_position <= position ~pts stream then decode ()
@@ -802,7 +825,7 @@ let mk_decoder ~streams ~target_position container =
           if Hashtbl.length streams_seen = Streams.cardinal streams then (
             flush position;
             decode ())
-          else push (position, decode)
+          else push (position, ts, decode)
       | None, _ ->
           log#important
             "Got packet or frame with no timestamp! Synchronization issues may \
@@ -848,6 +871,7 @@ let mk_decoder ~streams ~target_position container =
               match Streams.find_opt i streams with
                 | Some (`Audio_frame (s, decode)) ->
                     check_pts s
+                      ~ts:(Option.value ~default:0L (Avutil.Frame.pts frame))
                       ~decode:(fun () -> decode ~buffer (`Frame frame))
                       (Avutil.Frame.pts frame)
                 | _ -> f ())
@@ -855,6 +879,9 @@ let mk_decoder ~streams ~target_position container =
               match Streams.find_opt i streams with
                 | Some (`Audio_packet (s, decode)) ->
                     check_pts
+                      ~ts:
+                        (Option.value ~default:0L
+                           (Avcodec.Packet.get_dts packet))
                       ~decode:(fun () -> decode ~buffer packet)
                       s
                       (Avcodec.Packet.get_pts packet)
@@ -863,6 +890,7 @@ let mk_decoder ~streams ~target_position container =
               match Streams.find_opt i streams with
                 | Some (`Video_frame (s, decode)) ->
                     check_pts
+                      ~ts:(Option.value ~default:0L (Avutil.Frame.pts frame))
                       ~decode:(fun () -> decode ~buffer (`Frame frame))
                       s (Avutil.Frame.pts frame)
                 | _ -> f ())
@@ -870,6 +898,9 @@ let mk_decoder ~streams ~target_position container =
               match Streams.find_opt i streams with
                 | Some (`Video_packet (s, decode)) ->
                     check_pts
+                      ~ts:
+                        (Option.value ~default:0L
+                           (Avcodec.Packet.get_dts packet))
                       ~decode:(fun () -> decode ~buffer packet)
                       s
                       (Avcodec.Packet.get_pts packet)
@@ -878,6 +909,9 @@ let mk_decoder ~streams ~target_position container =
               match Streams.find_opt i streams with
                 | Some (`Data_packet (s, decode)) ->
                     check_pts
+                      ~ts:
+                        (Option.value ~default:0L
+                           (Avcodec.Packet.get_dts packet))
                       ~decode:(fun () -> decode ~buffer packet)
                       s
                       (Avcodec.Packet.get_pts packet)
@@ -907,7 +941,7 @@ let mk_streams ~ctype ~decode_first_metadata container =
         is_first := false;
         latest_metadata := Some m;
         Generator.add_metadata buffer.Decoder.generator
-          (Frame.metadata_of_list m));
+          (Frame.Metadata.from_list m));
       fn ~buffer data
   in
   let stream_idx = Ffmpeg_content_base.new_stream_idx () in
@@ -1041,7 +1075,7 @@ let mk_streams ~ctype ~decode_first_metadata container =
                       in
                       if metadata <> [] then
                         Generator.add_metadata buffer.Decoder.generator
-                          (Frame.metadata_of_list metadata) ))
+                          (Frame.Metadata.from_list metadata) ))
                 streams,
               pos + 1 )
           else (streams, pos + 1)
@@ -1059,34 +1093,36 @@ let mk_streams ~ctype ~decode_first_metadata container =
 
 let create_decoder ~ctype ~metadata fname =
   let args, format = parse_file_decoder_args metadata in
-  let duration = duration ~metadata fname in
-  let remaining = ref duration in
-  let m = Mutex.create () in
-  let set_remaining stream pts =
-    Tutils.mutexify m
-      (fun () ->
-        match (duration, pts) with
-          | None, _ | Some _, None -> ()
-          | Some d, Some pts -> (
-              let { Avutil.num; den } = Av.get_time_base stream in
-              let position =
-                Int64.to_float (Int64.mul (Int64.of_int num) pts) /. float den
-              in
-              match !remaining with
-                | None -> remaining := Some (d -. position)
-                | Some r -> remaining := Some (min (d -. position) r)))
-      ()
+  let file_duration = dresolver ~metadata fname in
+  let remaining = Atomic.make file_duration in
+  let set_remaining ~pts ~duration stream =
+    let pts =
+      Option.map
+        (fun pts -> Int64.add pts (Option.value ~default:0L duration))
+        pts
+    in
+    match (file_duration, pts) with
+      | None, _ | Some _, None -> ()
+      | Some d, Some pts -> (
+          let { Avutil.num; den } = Av.get_time_base stream in
+          let position =
+            Int64.to_float (Int64.mul (Int64.of_int num) pts) /. float den
+          in
+          match Atomic.get remaining with
+            | None -> Atomic.set remaining (Some (d -. position))
+            | Some r -> Atomic.set remaining (Some (min (d -. position) r)))
   in
-  let get_remaining =
-    Tutils.mutexify m (fun () ->
-        match !remaining with None -> -1 | Some r -> Frame.main_of_seconds r)
+  let get_remaining () =
+    match Atomic.get remaining with
+      | None -> -1
+      | Some r -> Frame.main_of_seconds r
   in
   let opts = Hashtbl.create 10 in
-  List.iter (fun (k, v) -> Hashtbl.add opts k v) args;
+  List.iter (fun (k, v) -> Hashtbl.replace opts k v) args;
   let ext = Filename.extension fname in
   if List.exists (fun s -> ext = "." ^ s) image_file_extensions#get then (
-    Hashtbl.add opts "loop" (`Int 1);
-    Hashtbl.add opts "framerate" (`Int (Lazy.force Frame.video_rate)));
+    Hashtbl.replace opts "loop" (`Int 1);
+    Hashtbl.replace opts "framerate" (`Int (Lazy.force Frame.video_rate)));
   let container = Av.open_input ?format ~opts fname in
   let streams = mk_streams ~ctype ~decode_first_metadata:false container in
   let streams =
@@ -1094,28 +1130,36 @@ let create_decoder ~ctype ~metadata fname =
       (function
         | `Audio_packet (stream, decoder) ->
             let decoder ~buffer packet =
-              set_remaining stream (Avcodec.Packet.get_pts packet);
+              set_remaining stream
+                ~pts:(Avcodec.Packet.get_pts packet)
+                ~duration:(Avcodec.Packet.get_duration packet);
               decoder ~buffer packet
             in
             `Audio_packet (stream, decoder)
         | `Audio_frame (stream, decoder) ->
             let decoder ~buffer frame =
               (match frame with
-                | `Frame frame -> set_remaining stream (Avutil.Frame.pts frame)
+                | `Frame frame ->
+                    set_remaining stream ~pts:(Avutil.Frame.pts frame)
+                      ~duration:(Avutil.Frame.duration frame)
                 | _ -> ());
               decoder ~buffer frame
             in
             `Audio_frame (stream, decoder)
         | `Video_packet (stream, decoder) ->
             let decoder ~buffer packet =
-              set_remaining stream (Avcodec.Packet.get_pts packet);
+              set_remaining stream
+                ~pts:(Avcodec.Packet.get_pts packet)
+                ~duration:(Avcodec.Packet.get_duration packet);
               decoder ~buffer packet
             in
             `Video_packet (stream, decoder)
         | `Video_frame (stream, decoder) ->
             let decoder ~buffer frame =
               (match frame with
-                | `Frame frame -> set_remaining stream (Avutil.Frame.pts frame)
+                | `Frame frame ->
+                    set_remaining stream ~pts:(Avutil.Frame.pts frame)
+                      ~duration:(Avutil.Frame.duration frame)
                 | _ -> ());
               decoder ~buffer frame
             in
@@ -1128,7 +1172,7 @@ let create_decoder ~ctype ~metadata fname =
   ( {
       Decoder.seek =
         (fun ticks ->
-          match duration with
+          match file_duration with
             | None -> -1
             | Some d -> (
                 let target =
@@ -1139,13 +1183,13 @@ let create_decoder ~ctype ~metadata fname =
                   | _ -> ticks));
       decode = mk_decoder ~streams ~target_position container;
       eof = mk_eof streams;
+      close;
     },
-    close,
     get_remaining )
 
 let create_file_decoder ~metadata ~ctype filename =
-  let decoder, close, remaining = create_decoder ~ctype ~metadata filename in
-  Decoder.file_decoder ~filename ~close ~remaining ~ctype decoder
+  let decoder, remaining = create_decoder ~ctype ~metadata filename in
+  Decoder.file_decoder ~filename ~remaining ~ctype decoder
 
 let create_stream_decoder ~ctype mime input =
   let seek_input =
@@ -1160,10 +1204,10 @@ let create_stream_decoder ~ctype mime input =
       | _ -> (mime, ([], None))
   in
   let opts = Hashtbl.create 10 in
-  List.iter (fun (k, v) -> Hashtbl.add opts k v) args;
+  List.iter (fun (k, v) -> Hashtbl.replace opts k v) args;
   if List.exists (fun s -> mime = s) image_mime_types#get then (
-    Hashtbl.add opts "loop" (`Int 1);
-    Hashtbl.add opts "framerate" (`Int (Lazy.force Frame.video_rate)));
+    Hashtbl.replace opts "loop" (`Int 1);
+    Hashtbl.replace opts "framerate" (`Int (Lazy.force Frame.video_rate)));
   let container =
     Av.open_input_stream ?seek:seek_input ~opts ?format input.Decoder.read
   in
@@ -1175,10 +1219,12 @@ let create_stream_decoder ~ctype mime input =
       "ffmpeg_decoder";
   let streams = mk_streams ~ctype ~decode_first_metadata:true container in
   let target_position = ref None in
+  let close () = Av.close container in
   {
     Decoder.seek = seek ~target_position ~container;
     decode = mk_decoder ~streams ~target_position container;
     eof = mk_eof streams;
+    close;
   }
 
 let get_file_type ~metadata ~ctype filename =
@@ -1194,7 +1240,7 @@ let get_file_type ~metadata ~ctype filename =
     | _ ->
         let args, format = parse_file_decoder_args metadata in
         let opts = Hashtbl.create 10 in
-        List.iter (fun (k, v) -> Hashtbl.add opts k v) args;
+        List.iter (fun (k, v) -> Hashtbl.replace opts k v) args;
         let container = Av.open_input ?format ~opts filename in
         Fun.protect
           ~finally:(fun () -> Av.close container)
