@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,9 +26,8 @@ let log = Ffmpeg_utils.log
 
 type encoder = {
   mk_stream : Frame.t -> unit;
-  encode : Frame.t -> int -> int -> unit;
+  encode : Frame.t -> unit;
   flush : unit -> unit;
-  can_split : unit -> bool;
   codec_attr : unit -> string option;
   bitrate : unit -> int option;
   video_size : unit -> (int * int) option;
@@ -43,12 +42,13 @@ type handler = {
     sample_position:int ->
     (string * string) list ->
     string option;
-  mutable started : bool;
+  started : bool Atomic.t;
 }
 
 type stream_data = {
   idx : Int64.t;
   mutable last_start : Int64.t;
+  mutable position : Int64.t;
   mutable ready_count : int;
   mutable ready : bool;
 }
@@ -77,7 +77,8 @@ let mk_stream_store total_count =
   let store = Stream.create 1 in
   let add ~last_start ~ready idx =
     let data =
-      Stream.merge store { idx; ready_count = 0; last_start; ready = false }
+      Stream.merge store
+        { idx; ready_count = 0; last_start; position = 0L; ready = false }
     in
     if ready then data.ready_count <- data.ready_count + 1;
     if data.ready_count = total_count then data.ready <- true;
@@ -98,11 +99,10 @@ let mk_format ffmpeg =
     | Some short_name, _ -> Av.Format.guess_output_format ~short_name ()
     | _ -> None
 
-let encode ~encoder frame start len =
-  if not encoder.started then
+let encode ~encoder frame =
+  if not (Atomic.exchange encoder.started true) then
     Frame.Fields.iter (fun _ { mk_stream } -> mk_stream frame) encoder.streams;
-  encoder.started <- true;
-  Frame.Fields.iter (fun _ { encode } -> encode frame start len) encoder.streams
+  Frame.Fields.iter (fun _ { encode } -> encode frame) encoder.streams
 
 (* Convert ffmpeg-specific options. *)
 let convert_options opts =
@@ -115,10 +115,11 @@ let convert_options opts =
     | `String fmt -> `Int Avutil.Sample_format.(get_id (find fmt))
     | _ -> assert false);
   convert "channel_layout" (function
-    | `String layout -> `Int64 Avutil.Channel_layout.(get_id (find layout))
+    | `String layout ->
+        `String Avutil.Channel_layout.(get_description (find layout))
     | _ -> assert false)
 
-let encoder ~pos ~mk_streams ffmpeg meta =
+let encoder ~pos ~on_keyframe ~keyframes ~mk_streams ffmpeg meta =
   let buf = Strings.Mutable.empty () in
   let make () =
     let options = Hashtbl.copy ffmpeg.Ffmpeg_format.opts in
@@ -128,6 +129,12 @@ let encoder ~pos ~mk_streams ffmpeg meta =
       len
     in
     let format = mk_format ffmpeg in
+    let interleaved =
+      match ffmpeg.interleaved with
+        | `Default -> 0 < List.length ffmpeg.streams
+        | `True -> true
+        | `False -> false
+    in
     let output =
       match ffmpeg.Ffmpeg_format.output with
         | `Stream ->
@@ -138,8 +145,9 @@ let encoder ~pos ~mk_streams ffmpeg meta =
                     Lang_encoder.raise_error ~pos
                       (Printf.sprintf
                          "No ffmpeg format could be found for format=%S" fmt));
-            Av.open_output_stream ~opts:options write (Option.get format)
-        | `Url url -> Av.open_output ?format ~opts:options url
+            Av.open_output_stream ~interleaved ~opts:options write
+              (Option.get format)
+        | `Url url -> Av.open_output ?format ~interleaved ~opts:options url
     in
     let streams = mk_streams output in
     if Hashtbl.length options > 0 then
@@ -151,12 +159,12 @@ let encoder ~pos ~mk_streams ffmpeg meta =
       output;
       streams;
       insert_id3 = (fun ~frame_position:_ ~sample_position:_ _ -> None);
-      started = false;
+      started = Atomic.make false;
     }
   in
-  let encoder = ref (make ()) in
+  let encoder = Atomic.make (make ()) in
   let codec_attrs () =
-    let encoder = !encoder in
+    let encoder = Atomic.get encoder in
     match
       Frame.Fields.fold
         (fun _ c cur ->
@@ -167,14 +175,17 @@ let encoder ~pos ~mk_streams ffmpeg meta =
       | l -> Some (String.concat "," l)
   in
   let bitrate () =
-    let encoder = !encoder in
-    Some
-      (Frame.Fields.fold
-         (fun _ c cur -> cur + Option.value ~default:0 (c.bitrate ()))
-         encoder.streams 0)
+    let encoder = Atomic.get encoder in
+    let ( + ) v v' =
+      match (v, v') with
+        | None, None -> None
+        | None, Some v | Some v, None -> Some v
+        | Some v, Some v' -> Some (v + v')
+    in
+    Frame.Fields.fold (fun _ c cur -> cur + c.bitrate ()) encoder.streams None
   in
   let video_size () =
-    let encoder = !encoder in
+    let encoder = Atomic.get encoder in
     match
       Frame.Fields.fold
         (fun _ stream cur ->
@@ -186,9 +197,11 @@ let encoder ~pos ~mk_streams ffmpeg meta =
       | (width, height) :: [] -> Some (width, height)
       | _ -> None
   in
-  let sent = Frame.Fields.map (fun _ -> ref false) !encoder.streams in
+  let sent =
+    Frame.Fields.map (fun _ -> ref false) (Atomic.get encoder).streams
+  in
   let init ?id3_enabled ?id3_version () =
-    let encoder = !encoder in
+    let encoder = Atomic.get encoder in
     match Option.map Av.Format.get_output_name encoder.format with
       | Some "mpegts" ->
           if id3_enabled <> Some false then (
@@ -199,7 +212,7 @@ let encoder ~pos ~mk_streams ffmpeg meta =
             in
             encoder.insert_id3 <-
               (fun ~frame_position ~sample_position m ->
-                if encoder.started then (
+                if Atomic.get encoder.started then (
                   let tag = Utils.id3v2_of_metadata ~version:id3_version m in
                   let packet = Avcodec.Packet.create tag in
                   let position =
@@ -230,20 +243,28 @@ let encoder ~pos ~mk_streams ffmpeg meta =
       | None -> Lang_encoder.raise_error ~pos "Format is required!"
   in
   let insert_id3 ~frame_position ~sample_position m =
-    let encoder = !encoder in
+    let encoder = Atomic.get encoder in
     encoder.insert_id3 ~frame_position ~sample_position m
   in
-  let init_encode frame start len =
-    let encoder = !encoder in
+  let init_encode frame =
+    let encoder = Atomic.get encoder in
     match ffmpeg.Ffmpeg_format.format with
       | Some "mp4" ->
-          encode ~encoder frame start len;
+          encode ~encoder frame;
+          let frame =
+            Frame.Fields.filter
+              (fun _ c ->
+                not
+                  (Content_timed.Track_marks.is_data c
+                  || Content_timed.Metadata.is_data c))
+              frame
+          in
           Frame.Fields.iter
             (fun field c ->
-              let sent = Frame.Fields.find field sent in
-              let d = Content.sub c start len in
-              sent := !sent || not (Content.is_empty d))
-            (Generator.peek_media frame);
+              match Frame.Fields.find_opt field sent with
+                | None -> ()
+                | Some sent -> sent := !sent || not (Content.is_empty c))
+            frame;
           if Frame.Fields.exists (fun _ c -> not !c) sent then
             raise Encoder.Not_enough_data;
           Av.flush encoder.output;
@@ -252,49 +273,61 @@ let encoder ~pos ~mk_streams ffmpeg meta =
       | Some "webm" ->
           Av.flush encoder.output;
           let init = Strings.Mutable.flush buf in
-          encode ~encoder frame start len;
+          encode ~encoder frame;
           (Some init, Strings.Mutable.flush buf)
       | _ ->
-          encode ~encoder frame start len;
+          encode ~encoder frame;
           (None, Strings.Mutable.flush buf)
   in
-  let split_encode frame start len =
-    let encoder = !encoder in
-    Av.flush encoder.output;
-    let flushed = Strings.Mutable.flush buf in
-    encode ~encoder frame start len;
+  let split_encode frame =
+    let encoder = Atomic.get encoder in
+    let can_split () =
+      List.for_all (fun (_, keyframe) -> Atomic.get keyframe) keyframes
+    in
+    let flushed =
+      if can_split () then Atomic.make (Some (Strings.Mutable.flush buf))
+      else (
+        let flushed = Atomic.make None in
+        Atomic.set on_keyframe (fun () ->
+            match (can_split (), Atomic.get flushed) with
+              | true, None ->
+                  Atomic.set flushed (Some (Strings.Mutable.flush buf))
+              | _ -> ());
+        flushed)
+    in
+    Fun.protect
+      (fun () -> encode ~encoder frame)
+      ~finally:(fun () -> Atomic.set on_keyframe (fun () -> ()));
     let encoded = Strings.Mutable.flush buf in
-    if Frame.Fields.exists (fun _ c -> not (c.can_split ())) encoder.streams
-    then `Nope (Strings.append flushed encoded)
-    else `Ok (flushed, encoded)
+    match Atomic.get flushed with
+      | Some flushed ->
+          List.iter (fun (_, keyframe) -> Atomic.set keyframe false) keyframes;
+          `Ok (flushed, encoded)
+      | None -> `Nope encoded
   in
-  let encode frame start len =
-    encode ~encoder:!encoder frame start len;
+  let encode frame =
+    encode ~encoder:(Atomic.get encoder) frame;
     Strings.Mutable.flush buf
   in
   let flush () =
-    Frame.Fields.iter (fun _ { flush } -> flush ()) !encoder.streams
+    Frame.Fields.iter (fun _ { flush } -> flush ()) (Atomic.get encoder).streams
   in
   let insert_metadata m =
-    let m =
-      Hashtbl.fold
-        (fun lbl v l -> (lbl, v) :: l)
-        (Export_metadata.to_metadata m)
-        []
-    in
+    let m = Frame.Metadata.to_list (Frame.Metadata.Export.to_metadata m) in
     match (ffmpeg.Ffmpeg_format.output, ffmpeg.Ffmpeg_format.format) with
-      | _ when not !encoder.started -> Av.set_output_metadata !encoder.output m
+      | _ when not (Atomic.get (Atomic.get encoder).started) ->
+          Av.set_output_metadata (Atomic.get encoder).output m
       | `Stream, Some "ogg" ->
           flush ();
-          Av.close !encoder.output;
-          encoder := make ();
-          Av.set_output_metadata !encoder.output m
+          Av.close (Atomic.get encoder).output;
+          Atomic.set encoder (make ());
+          Av.set_output_metadata (Atomic.get encoder).output m
       | _ -> ()
   in
   insert_metadata meta;
   let stop () =
     flush ();
-    (try Av.close !encoder.output with _ -> ());
+    (try Av.close (Atomic.get encoder).output with _ -> ());
     Strings.Mutable.flush buf
   in
   let hls =
