@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -41,13 +41,12 @@ let encoder ~pos shine =
       (Frame.Fields.make ~audio:(Content.Audio.format_of_channels channels) ())
   in
   let encoded = Strings.Mutable.empty () in
-  let encode frame start len =
+  let encode frame =
     let b = AFrame.pcm frame in
-    let start = Frame.audio_of_main start in
-    let len = Frame.audio_of_main len in
+    let len = AFrame.position frame in
     let b, start, len =
       Audio_converter.Samplerate.resample samplerate_converter
-        (dst_freq /. src_freq) b start len
+        (dst_freq /. src_freq) b 0 len
     in
     let start = Frame.main_of_audio start in
     let len = Frame.main_of_audio len in
@@ -56,8 +55,7 @@ let encoder ~pos shine =
     while Generator.length buf > samples do
       let pcm =
         Content.Audio.get_data
-          (Frame.Fields.find Frame.Fields.audio
-             (Generator.get ~length:samples buf))
+          (Frame.Fields.find Frame.Fields.audio (Generator.slice buf samples))
       in
       Strings.Mutable.add encoded (Shine.encode_buffer enc pcm)
     done;
