@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
  *****************************************************************************)
 
 open Value
-open Ground
 
 type decode_type = [ `Raw | `Internal ]
 type content_type = [ `Audio | `Video ]
@@ -30,27 +29,24 @@ type encoder_params =
   decode_type * content_type * (string * Liquidsoap_lang.Value.t) list
 
 type mode =
-  [ `Copy of Liquidsoap_lang.Value.t option | `Encode of encoder_params ]
+  [ `Drop | `Copy of Liquidsoap_lang.Value.t option | `Encode of encoder_params ]
 
 type parsed_encoder = Frame.field * mode
 
 let channels_of_channel_layout args =
   match List.assoc "channel_layout" args with
     (* 5.1 as float. *)
-    | `Term { Term.term = Term.Ground (Float layout) } ->
+    | { Term.term = `Float layout } ->
         let layout = Printf.sprintf "%.1f" layout in
         Avutil.Channel_layout.(get_nb_channels (find layout))
-    | `Term { Term.term = Term.Ground (String layout) } ->
+    | { Term.term = `String layout } ->
         Avutil.Channel_layout.(get_nb_channels (find layout))
-    | `Term ({ t = { Type.pos } } as tm) ->
+    | { t = { Type.pos } } as tm ->
         Lang_encoder.raise_error ~pos
           (Printf.sprintf
              "Invalid value %s for channel_layout parameter. Only static \
               numbers are allowed."
              (Term.to_string tm))
-    | _ ->
-        Lang_encoder.raise_error ~pos:None
-          "Invalid value for channel_layout parameter."
 
 let channels args =
   try channels_of_channel_layout args
@@ -61,45 +57,42 @@ let channels args =
         with Not_found -> ("ac", List.assoc "ac" args)
       in
       match channels with
-        | `Term { Term.term = Term.Ground (Int n) } -> n
-        | `Term ({ t = { Type.pos } } as tm) ->
+        | { Term.term = `Int n } -> n
+        | { t = { Type.pos } } as tm ->
             Lang_encoder.raise_error ~pos
               (Printf.sprintf
                  "Invalid value %s for %s parameter. Only static numbers are \
                   allowed."
                  name (Term.to_string tm))
-        | _ ->
-            Lang_encoder.raise_error ~pos:None
-              (Printf.sprintf "Invalid value for %s parameter." name)
     with Not_found -> 2)
 
 let to_int t =
-  match t.value with
-    | Ground (Int i) -> i
-    | Ground (String s) -> int_of_string s
-    | Ground (Float f) -> int_of_float f
-    | _ -> Lang_encoder.raise_error ~pos:t.pos "integer expected"
+  match t with
+    | Value.Int { value = i } -> i
+    | Value.String { value = s } -> int_of_string s
+    | Value.Float { value = f } -> int_of_float f
+    | _ -> Lang_encoder.raise_error ~pos:(Value.pos t) "integer expected"
 
 let to_string t =
-  match t.value with
-    | Ground (Int i) -> Printf.sprintf "%i" i
-    | Ground (String s) -> s
-    | Ground (Float f) -> Printf.sprintf "%f" f
-    | _ -> Lang_encoder.raise_error ~pos:t.pos "string expected"
+  match t with
+    | Value.Int { value = i } -> Printf.sprintf "%i" i
+    | Value.String { value = s } -> s
+    | Value.Float { value = f } -> Printf.sprintf "%f" f
+    | _ -> Lang_encoder.raise_error ~pos:(Value.pos t) "string expected"
 
 let to_float t =
-  match t.value with
-    | Ground (Int i) -> float i
-    | Ground (String s) -> float_of_string s
-    | Ground (Float f) -> f
-    | _ -> Lang_encoder.raise_error ~pos:t.pos "float expected"
+  match t with
+    | Value.Int { value = i } -> float i
+    | Value.String { value = s } -> float_of_string s
+    | Value.Float { value = f } -> f
+    | _ -> Lang_encoder.raise_error ~pos:(Value.pos t) "float expected"
 
 let to_copy_opt t =
-  match t.value with
-    | Ground (String "wait_for_keyframe") -> `Wait_for_keyframe
-    | Ground (String "ignore_keyframe") -> `Ignore_keyframe
+  match t with
+    | Value.String { value = "wait_for_keyframe" } -> `Wait_for_keyframe
+    | Value.String { value = "ignore_keyframe" } -> `Ignore_keyframe
     | _ ->
-        Lang_encoder.raise_error ~pos:t.pos
+        Lang_encoder.raise_error ~pos:(Value.pos t)
           ("Invalid value for copy encoder parameter: " ^ Value.to_string t)
 
 let has_content ~to_static_string name p =
@@ -124,8 +117,8 @@ let stream_media_type ~to_pos ~to_static_string name args =
   match (name, args) with
     | _ when has_content ~to_static_string "audio_content" args -> `Audio
     | _ when has_content ~to_static_string "video_content" args -> `Video
-    | _ when Pcre.pmatch ~pat:"audio" name -> `Audio
-    | _ when Pcre.pmatch ~pat:"video" name -> `Video
+    | _ when Re.Pcre.pmatch ~rex:(Re.Pcre.regexp "audio") name -> `Audio
+    | _ when Re.Pcre.pmatch ~rex:(Re.Pcre.regexp "video") name -> `Video
     | _ -> (
         match List.assoc_opt "codec" args with
           | Some t -> (
@@ -143,7 +136,11 @@ let stream_media_type ~to_pos ~to_static_string name args =
 let copy_param = function
   | [] -> None
   | [("", t)] -> Some t
-  | [(l, v)] | _ :: (l, v) :: _ -> Lang_encoder.raise_generic_error (l, `Value v)
+  | [(l, v)] | _ :: (l, v) :: _ ->
+      Lang_encoder.raise_generic_error
+        (match l with
+          | "" -> `Anonymous (Lang.to_string v)
+          | l -> `Labelled (l, v))
 
 let parse_encoder_name name =
   let field, mode =
@@ -156,19 +153,21 @@ let parse_encoder_name name =
     match mode with
       | "copy" -> `Copy
       | "raw" -> `Raw
+      | "drop" -> `Drop
       | "" -> `Internal
       | _ -> failwith "invalid content field"
   in
   (field, mode)
 
 let to_static_string_value = function
-  | Value.{ value = Ground (Ground.String s) } -> Some s
+  | Value.String { value = s } -> Some s
   | _ -> None
 
 let parse_encoder_params ~to_pos (name, p) : parsed_encoder =
   let field, mode = parse_encoder_name name in
   let mode =
     match mode with
+      | `Drop -> `Drop
       | `Copy -> `Copy (copy_param p)
       | `Raw ->
           `Encode
@@ -187,73 +186,79 @@ let parse_encoder_params ~to_pos (name, p) : parsed_encoder =
   (field, mode)
 
 let to_static_string_term = function
-  | `Term Term.{ term = Ground (Ground.String s) } -> Some s
+  | Term.{ term = `String s } -> Some s
   | _ -> None
 
-let term_pos = function
-  | `Term { Term.t = { Type.pos } } -> pos
-  | _ -> raise Exit
+let term_pos { Term.t = { Type.pos } } = pos
 
 let type_of_encoder =
   List.fold_left
     (fun content_type p ->
       match p with
-        | "", `Encoder (name, args) ->
+        | `Encoder (name, args) ->
+            let args =
+              List.filter_map
+                (function
+                  | `Anonymous s -> Some ("", Term.make (`String s))
+                  | `Labelled (l, v) -> Some (l, v)
+                  | `Encoder _ -> None)
+                args
+            in
             let field, mode = parse_encoder_name name in
             let format =
               match mode with
+                | `Drop -> Type.var ~constraints:[Format_type.track] ()
                 | `Copy ->
-                    Content.(default_format (kind_of_string "ffmpeg.copy"))
-                | `Raw -> (
-                    match
-                      stream_media_type ~to_pos:term_pos
-                        ~to_static_string:to_static_string_term name args
-                    with
-                      | `Audio ->
-                          Content.(
-                            default_format (kind_of_string "ffmpeg.audio.raw"))
-                      | `Video ->
-                          Content.(
-                            default_format (kind_of_string "ffmpeg.video.raw")))
-                | `Internal -> (
-                    match
-                      stream_media_type ~to_pos:term_pos
-                        ~to_static_string:to_static_string_term name args
-                    with
-                      | `Audio ->
-                          let channels = channels args in
-                          let pcm_kind =
-                            List.fold_left
-                              (fun pcm_kind -> function
-                                | ( "",
-                                    `Term
-                                      { Term.term = Term.Ground (String "pcm") }
-                                  ) ->
-                                    Content.Audio.kind
-                                | ( "",
-                                    `Term
-                                      {
-                                        Term.term =
-                                          Term.Ground (String "pcm_s16");
-                                      } ) ->
-                                    Content_pcm_s16.kind
-                                | ( "",
-                                    `Term
-                                      {
-                                        Term.term =
-                                          Term.Ground (String "pcm_f32");
-                                      } ) ->
-                                    Content_pcm_f32.kind
-                                | _ -> pcm_kind)
-                              Content.Audio.kind args
-                          in
-                          Frame_base.format_of_channels ~pcm_kind channels
-                      | `Video -> Content.(default_format Video.kind))
+                    Type.make
+                      (Format_type.descr
+                         (`Format
+                           (Content.default_format Ffmpeg_copy_content.kind)))
+                | `Raw ->
+                    Type.make
+                      (Format_type.descr
+                         (`Format
+                           (match
+                              stream_media_type ~to_pos:term_pos
+                                ~to_static_string:to_static_string_term name
+                                args
+                            with
+                             | `Audio ->
+                                 Content.default_format
+                                   Ffmpeg_raw_content.Audio.kind
+                             | `Video ->
+                                 Content.default_format
+                                   Ffmpeg_raw_content.Video.kind)))
+                | `Internal ->
+                    Type.make
+                      (Format_type.descr
+                         (`Format
+                           (match
+                              stream_media_type ~to_pos:term_pos
+                                ~to_static_string:to_static_string_term name
+                                args
+                            with
+                             | `Audio ->
+                                 let channels = channels args in
+                                 let pcm_kind =
+                                   List.fold_left
+                                     (fun pcm_kind -> function
+                                       | "", { Term.term = `String "pcm" } ->
+                                           Content.Audio.kind
+                                       | "", { Term.term = `String "pcm_s16" }
+                                         ->
+                                           Content_pcm_s16.kind
+                                       | "", { Term.term = `String "pcm_f32" }
+                                         ->
+                                           Content_pcm_f32.kind
+                                       | _ -> pcm_kind)
+                                     Content.Audio.kind args
+                                 in
+                                 Frame_base.format_of_channels ~pcm_kind
+                                   channels
+                             | `Video -> Content.(default_format Video.kind))))
             in
             let field = Frame.Fields.register field in
-            Frame.Fields.add field
-              (Type.make (Format_type.descr (`Format format)))
-              content_type
+            Frame.Fields.add field format content_type
         | _ -> content_type)
     Frame.Fields.empty
 
@@ -281,6 +286,7 @@ let ffmpeg_gen params =
       Ffmpeg_format.format = None;
       output = `Stream;
       streams = [];
+      interleaved = `Default;
       opts = Hashtbl.create 0;
     }
   in
@@ -308,14 +314,14 @@ let ffmpeg_gen params =
 
   let parse_opts opts = function
     | "q", t | "qscale", t -> set_global_quality (to_float t) opts
-    | "", { value = Ground (String "audio_content"); _ }
-    | "", { value = Ground (String "video_content"); _ }
+    | "", String { value = "audio_content"; _ }
+    | "", String { value = "video_content"; _ }
     | "codec", _ ->
         ()
-    | k, { value = Ground (String s); _ } -> Hashtbl.add opts k (`String s)
-    | k, { value = Ground (Int i); _ } -> Hashtbl.add opts k (`Int i)
-    | k, { value = Ground (Float fl); _ } -> Hashtbl.add opts k (`Float fl)
-    | _, t -> Lang_encoder.raise_error ~pos:t.pos "unexpected option"
+    | k, String { value = s; _ } -> Hashtbl.replace opts k (`String s)
+    | k, Value.Int { value = i; _ } -> Hashtbl.replace opts k (`Int i)
+    | k, Float { value = fl; _ } -> Hashtbl.replace opts k (`Float fl)
+    | _, t -> Lang_encoder.raise_error ~pos:(Value.pos t) "unexpected option"
   in
 
   let rec parse_audio_args ~opts options = function
@@ -337,8 +343,8 @@ let ffmpeg_gen params =
     | (("channel_layout", t) as arg) :: args ->
         (* Handle 5.1 as float *)
         let layout =
-          match t.value with
-            | Ground (Float f) -> Printf.sprintf "%.1f" f
+          match t with
+            | Value.Float { value = f } -> Printf.sprintf "%.1f" f
             | _ -> to_string t
         in
         parse_opts opts arg;
@@ -391,7 +397,7 @@ let ffmpeg_gen params =
         parse_video_args ~opts
           { options with Ffmpeg_format.height = Lazy.from_val (to_int t) }
           args
-    | ("pixel_format", { value = Ground (String "guess"); _ }) :: args ->
+    | ("pixel_format", String { value = "guess" }) :: args ->
         parse_video_args ~opts
           { options with Ffmpeg_format.pixel_format = None }
           args
@@ -399,27 +405,27 @@ let ffmpeg_gen params =
         parse_video_args ~opts
           { options with Ffmpeg_format.pixel_format = Some (to_string t) }
           args
-    | ("hwaccel", { value = Ground (String "auto"); _ }) :: args ->
+    | ("hwaccel", String { value = "auto" }) :: args ->
         parse_video_args ~opts
           { options with Ffmpeg_format.hwaccel = `Auto }
           args
-    | ("hwaccel", { value = Ground (String "none"); _ }) :: args ->
+    | ("hwaccel", String { value = "none"; _ }) :: args ->
         parse_video_args ~opts
           { options with Ffmpeg_format.hwaccel = `None }
           args
-    | ("hwaccel", { value = Ground (String "internal"); _ }) :: args ->
+    | ("hwaccel", String { value = "internal"; _ }) :: args ->
         parse_video_args ~opts
           { options with Ffmpeg_format.hwaccel = `Internal }
           args
-    | ("hwaccel", { value = Ground (String "device"); _ }) :: args ->
+    | ("hwaccel", String { value = "device"; _ }) :: args ->
         parse_video_args ~opts
           { options with Ffmpeg_format.hwaccel = `Device }
           args
-    | ("hwaccel", { value = Ground (String "frame"); _ }) :: args ->
+    | ("hwaccel", String { value = "frame"; _ }) :: args ->
         parse_video_args ~opts
           { options with Ffmpeg_format.hwaccel = `Frame }
           args
-    | ("hwaccel_device", { value = Ground (String "none"); _ }) :: args ->
+    | ("hwaccel_device", String { value = "none"; _ }) :: args ->
         parse_video_args ~opts
           { options with Ffmpeg_format.hwaccel_device = None }
           args
@@ -427,8 +433,7 @@ let ffmpeg_gen params =
         parse_video_args ~opts
           { options with Ffmpeg_format.hwaccel_device = Some (to_string t) }
           args
-    | ("hwaccel_pixel_format", { value = Ground (String "guess"); _ }) :: args
-      ->
+    | ("hwaccel_pixel_format", String { value = "guess" }) :: args ->
         parse_video_args ~opts
           { options with Ffmpeg_format.hwaccel_pixel_format = None }
           args
@@ -458,6 +463,11 @@ let ffmpeg_gen params =
 
   List.fold_left
     (fun f -> function
+      | `Encoder (field, `Drop) ->
+          {
+            f with
+            Ffmpeg_format.streams = f.Ffmpeg_format.streams @ [(field, `Drop)];
+          }
       | `Encoder (field, `Copy None) ->
           {
             f with
@@ -478,35 +488,46 @@ let ffmpeg_gen params =
               f.Ffmpeg_format.streams
               @ [(field, `Encode { Ffmpeg_format.mode; options; codec; opts })];
           }
-      | `Option ("format", { value = Ground (String "none"); _ }) ->
+      | `Option ("format", String { value = "none"; _ }) ->
           { f with Ffmpeg_format.format = None }
-      | `Option ("format", { value = Ground (String fmt); _ }) ->
+      | `Option ("format", String { value = fmt }) ->
           { f with Ffmpeg_format.format = Some fmt }
-      | `Option (k, { value = Ground (String s); _ }) ->
-          Hashtbl.add f.Ffmpeg_format.opts k (`String s);
+      | `Option ("interleaved", Value.Bool { value = b; _ }) ->
+          { f with Ffmpeg_format.interleaved = (if b then `True else `False) }
+      | `Option ("interleaved", String { value = "default" }) ->
+          { f with Ffmpeg_format.interleaved = `Default }
+      | `Option (k, String { value = s; _ }) ->
+          Hashtbl.replace f.Ffmpeg_format.opts k (`String s);
           f
-      | `Option (k, { value = Ground (Int i); _ }) ->
-          Hashtbl.add f.Ffmpeg_format.opts k (`Int i);
+      | `Option (k, Value.Int { value = i; _ }) ->
+          Hashtbl.replace f.Ffmpeg_format.opts k (`Int i);
           f
-      | `Option (k, { value = Ground (Float i); _ }) ->
-          Hashtbl.add f.Ffmpeg_format.opts k (`Float i);
+      | `Option (k, Float { value = i }) ->
+          Hashtbl.replace f.Ffmpeg_format.opts k (`Float i);
           f
-      | `Option (l, v) -> Lang_encoder.raise_generic_error (l, `Value v))
+      | `Option (l, v) ->
+          Lang_encoder.raise_generic_error
+            (match l with
+              | "" -> `Anonymous (Lang.to_string v)
+              | l -> `Labelled (l, v)))
     defaults params
 
 let make params =
   let params =
     List.map
       (function
-        | _, `Encoder (name, args) ->
+        | `Encoder (name, args) ->
             let args =
               List.filter_map
-                (function l, `Value v -> Some (l, v) | _, `Encoder _ -> None)
+                (function
+                  | `Anonymous s -> Some ("", Lang.string s)
+                  | `Labelled (l, v) -> Some (l, v)
+                  | `Encoder _ -> None)
                 args
             in
-            `Encoder
-              (parse_encoder_params ~to_pos:(fun t -> t.pos) (name, args))
-        | l, `Value v -> `Option (l, v))
+            `Encoder (parse_encoder_params ~to_pos:Value.pos (name, args))
+        | `Anonymous s -> `Option ("", Lang.string s)
+        | `Labelled (l, v) -> `Option (l, v))
       params
   in
   Encoder.Ffmpeg (ffmpeg_gen params)
