@@ -25,6 +25,85 @@
 exception Done
 exception Not_connected
 
+type prefer_address = [ `System_default | `Ipv4 | `Ipv6 ]
+type socket_mode = [ `Connect | `Listen | `Incoming | `Close ]
+
+let conf_srt =
+  Dtools.Conf.void ~p:(Configure.conf#plug "srt") "SRT configuration"
+
+let conf_prefer_address =
+  Dtools.Conf.string
+    ~p:(conf_srt#plug "prefer_address")
+    ~d:"system"
+    "Set preference for resolving addresses. One of: `\"system\"`, `\"ipv4\"` \
+     or `\"ipv6\"`."
+
+let conf_log =
+  Dtools.Conf.bool ~p:(conf_srt#plug "log") ~d:true
+    "Route srt logs through liquidsoap's logs"
+
+let conf_verbosity =
+  Dtools.Conf.string
+    ~p:(conf_log#plug "verbosity")
+    "Verbosity" ~d:"warning"
+    ~comments:
+      [
+        "Set SRT log level, one of: \"critical\", \"error\", ";
+        "\"warning\", \"notice\" or \"debug\"";
+      ]
+
+let conf_level = Dtools.Conf.int ~p:(conf_log#plug "level") ~d:4 "Level"
+let conf_poll = Dtools.Conf.void ~p:(conf_srt#plug "poll") "Poll configuration"
+
+let conf_timeout =
+  Dtools.Conf.float ~p:(conf_poll#plug "timeout") ~d:0.1
+    "Timeout for polling loop, in seconda."
+
+let conf_enforced_encryption =
+  Dtools.Conf.bool
+    ~p:(conf_srt#plug "enforced_encryption")
+    ~d:true
+    "Enforce consistent encryption settings on both end of any connection."
+
+let string_of_address = function
+  | Unix.ADDR_UNIX _ -> assert false
+  | Unix.ADDR_INET (addr, port) ->
+      Printf.sprintf "%s:%d" (Unix.string_of_inet_addr addr) port
+
+let getaddrinfo ~(log : Log.t) ~prefer_address address port =
+  let open Ctypes in
+  let open Posix_socket in
+  let hints = allocate_n Addrinfo.t ~count:1 in
+  hints |-> Addrinfo.ai_flags <-@ ni_numerichost;
+  hints |-> Addrinfo.ai_family <-@ af_unspec;
+  hints |-> Addrinfo.ai_socktype <-@ sock_stream;
+  match getaddrinfo ~hints ~port:(`Int port) address with
+    | [] ->
+        Runtime_error.raise ~pos:[]
+          ~message:
+            (Printf.sprintf "getaddrinfo could not resolve address: %s:%i"
+               address port)
+          "srt"
+    | first_address :: _ as resolved_addresses ->
+        let rec filter_address inet_type = function
+          | [] -> first_address
+          | sockaddr :: _ when !@(sockaddr |-> Sockaddr.sa_family) = inet_type
+            ->
+              sockaddr
+          | _ :: resolved_addresses ->
+              filter_address inet_type resolved_addresses
+        in
+        let sockaddr =
+          match prefer_address with
+            | `System_default -> first_address
+            | `Ipv4 -> filter_address af_inet resolved_addresses
+            | `Ipv6 -> filter_address af_inet6 resolved_addresses
+        in
+        if log#active 5 then
+          log#f 5 "Address %s:%n resolved to: %s" address port
+            (string_of_address (to_unix_sockaddr sockaddr));
+        sockaddr
+
 module SyncSource = Clock.MkSyncSource (struct
   type t = unit
 
@@ -56,7 +135,7 @@ let common_options ~mode =
              (false, "hs_version", Lang.int_t);
              (false, "peeraddr", Lang.string_t);
              (false, "streamid", Lang.nullable_t Lang.string_t);
-             (false, "", Builtins_srt.Socket_value.t);
+             (false, "", Builtins_srt.Socket_value.base_t);
            ]
            Lang.bool_t),
       Some Lang.null,
@@ -106,6 +185,29 @@ let common_options ~mode =
       Lang.string_t,
       Some (Lang.string "0.0.0.0"),
       Some "Address to bind on the local machine. Used only in listener mode" );
+    ( "prefer_address",
+      Lang.nullable_t Lang.string_t,
+      Some Lang.null,
+      Some
+        "Preferred address type when resolving hostnames. One of: \
+         `\"system\"`, `\"ipv4\"` or `\"ipv6\"`. Defaults to global \
+         `srt.prefer_connection` settings when `null`." );
+    ( "on_socket",
+      Lang.nullable_t
+        (Lang.fun_t
+           [
+             (false, "mode", Lang.string_t);
+             (false, "", Builtins_srt.Socket_value.base_t);
+           ]
+           Lang.unit_t),
+      Some Lang.null,
+      Some
+        "Callback executed when a new SRT socket is created to set additional \
+         options, add monitoring, etc. `mode` should be one of: `\"connect\"` \
+         (socket created before connecting to a remote address), `\"listen\"` \
+         (socket created before binding for receiving new incoming \
+         connections), `\"incoming\"` (socket received as incoming connection) \
+         or `\"close\"` (socket is about to closed)." );
     ( "polling_delay",
       Lang.float_t,
       Some (Lang.float 2.),
@@ -147,7 +249,7 @@ let meth () =
       ( [],
         Lang.fun_t []
           (Lang.list_t
-             (Lang.product_t Lang.string_t Builtins_srt.Socket_value.t)) ),
+             (Lang.product_t Lang.string_t Builtins_srt.Socket_value.base_t)) ),
       "List of `(connected_address, connected_socket)`",
       fun s ->
         Lang.val_fun [] (fun _ ->
@@ -156,7 +258,7 @@ let meth () =
                  (fun (origin, s) ->
                    Lang.product
                      (Lang.string (Utils.name_of_sockaddr origin))
-                     (Builtins_srt.Socket_value.to_value s))
+                     (Builtins_srt.Socket_value.to_base_value s))
                  s#get_sockets)) );
   ]
 
@@ -164,7 +266,9 @@ type common_options = {
   mode : [ `Listener | `Caller ];
   hostname : string;
   port : int;
-  bind_address : Unix.sockaddr;
+  bind_address : string;
+  prefer_address : [ `System_default | `Ipv4 | `Ipv6 ];
+  on_socket : mode:socket_mode -> Srt.socket -> unit;
   listen_callback : Srt.listen_callback option;
   streamid : string option;
   pbkeylen : int option;
@@ -182,13 +286,38 @@ type common_options = {
 
 let parse_common_options p =
   let bind_address = Lang.to_string (List.assoc "bind_address" p) in
-  let bind_address =
-    try Unix.inet_addr_of_string bind_address
-    with exn ->
-      raise
-        (Error.Invalid_value
-           ( List.assoc "bind_address" p,
-             Printf.sprintf "Invalid address: %s" (Printexc.to_string exn) ))
+  let prefer_address =
+    let v = List.assoc "prefer_address" p in
+    match
+      Option.value ~default:conf_prefer_address#get
+        (Lang.to_valued_option Lang.to_string v)
+    with
+      | "system" -> `System_default
+      | "ipv4" -> `Ipv4
+      | "ipv6" -> `Ipv6
+      | _ ->
+          raise
+            (Error.Invalid_value
+               (v, "Valid values are: `\"system\"`, `\"ipv4\"` or `\"ipv6\"`."))
+  in
+  let on_socket =
+    match Lang.to_option (List.assoc "on_socket" p) with
+      | None -> fun ~mode:_ _ -> ()
+      | Some fn ->
+          fun ~mode s ->
+            let mode =
+              match mode with
+                | `Connect -> "connect"
+                | `Listen -> "listen"
+                | `Incoming -> "incoming"
+                | `Close -> "close"
+            in
+            ignore
+              (Lang.apply fn
+                 [
+                   ("mode", Lang.string mode);
+                   ("", Builtins_srt.Socket_value.to_base_value s);
+                 ])
   in
   let passphrase_v = List.assoc "passphrase" p in
   let passphrase = Lang.to_valued_option Lang.to_string passphrase_v in
@@ -222,12 +351,10 @@ let parse_common_options p =
                  match streamid with
                    | None -> Lang.null
                    | Some s -> Lang.string s );
-               ("", Builtins_srt.Socket_value.to_value socket);
+               ("", Builtins_srt.Socket_value.to_base_value socket);
              ]))
       (Lang.to_option fn)
   in
-  let port = Lang.to_int (List.assoc "port" p) in
-  let bind_address = Unix.ADDR_INET (bind_address, port) in
   let on_connect = List.assoc "on_connect" p in
   let on_disconnect = List.assoc "on_disconnect" p in
   let polling_delay = Lang.to_float (List.assoc "polling_delay" p) in
@@ -251,6 +378,8 @@ let parse_common_options p =
     hostname = Lang.to_string (List.assoc "host" p);
     port = Lang.to_int (List.assoc "port" p);
     bind_address;
+    prefer_address;
+    on_socket;
     listen_callback;
     pbkeylen;
     enforced_encryption;
@@ -266,36 +395,6 @@ let parse_common_options p =
     on_disconnect = ref (fun () -> ignore (Lang.apply on_disconnect []));
   }
 
-let conf_srt =
-  Dtools.Conf.void ~p:(Configure.conf#plug "srt") "SRT configuration"
-
-let conf_log =
-  Dtools.Conf.bool ~p:(conf_srt#plug "log") ~d:true
-    "Route srt logs through liquidsoap's logs"
-
-let conf_verbosity =
-  Dtools.Conf.string
-    ~p:(conf_log#plug "verbosity")
-    "Verbosity" ~d:"warning"
-    ~comments:
-      [
-        "Set SRT log level, one of: \"critical\", \"error\", ";
-        "\"warning\", \"notice\" or \"debug\"";
-      ]
-
-let conf_level = Dtools.Conf.int ~p:(conf_log#plug "level") ~d:4 "Level"
-let conf_poll = Dtools.Conf.void ~p:(conf_srt#plug "poll") "Poll configuration"
-
-let conf_timeout =
-  Dtools.Conf.float ~p:(conf_poll#plug "timeout") ~d:0.1
-    "Timeout for polling loop, in seconda."
-
-let conf_enforced_encryption =
-  Dtools.Conf.bool
-    ~p:(conf_srt#plug "enforced_encryption")
-    ~d:true
-    "Enforce consistent encryption settings on both end of any connection."
-
 let log = Log.make ["srt"]
 
 let log_handler { Srt.Log.message } =
@@ -368,6 +467,7 @@ module Poll = struct
     Duppy.Async.wake_up task
 
   let remove_socket socket =
+    Hashtbl.remove t.handlers socket;
     if List.mem socket (Srt.Poll.sockets t.p) then
       Srt.Poll.remove_usock t.p socket
 end
@@ -400,15 +500,19 @@ let string_of_address = function
   | Unix.ADDR_INET (addr, port) ->
       Printf.sprintf "%s:%d" (Unix.string_of_inet_addr addr) port
 
-let mk_socket ~payload_size ~messageapi () =
+let mk_socket ~mode ~on_socket ~payload_size ~messageapi () =
   let s = Srt.create_socket () in
   Srt.setsockflag s Srt.payloadsize payload_size;
   Srt.setsockflag s Srt.transtype `Live;
   Srt.setsockflag s Srt.messageapi messageapi;
   Srt.setsockflag s Srt.enforced_encryption conf_enforced_encryption#get;
+  on_socket ~mode s;
   s
 
-let close_socket s = Srt.close s
+let close_socket ~on_socket s =
+  on_socket ~mode:`Close s;
+  Srt.close s
+
 let shutdown = Atomic.make false
 
 let () =
@@ -467,8 +571,9 @@ let () =
       ToDisconnect.iter (fun s -> s#disconnect) to_disconnect)
 
 class virtual caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
-  ~polling_delay ~payload_size ~messageapi ~hostname ~port ~connection_timeout
-  ~read_timeout ~write_timeout ~on_connect ~on_disconnect =
+  ~polling_delay ~payload_size ~messageapi ~on_socket ~hostname ~port
+  ~prefer_address ~connection_timeout ~read_timeout ~write_timeout ~on_connect
+  ~on_disconnect =
   object (self)
     method virtual id : string
     method virtual should_stop : bool
@@ -485,38 +590,44 @@ class virtual caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
 
     method private connect_fn () =
       try
-        let ipaddr = (Unix.gethostbyname hostname).Unix.h_addr_list.(0) in
-        let sockaddr = Unix.ADDR_INET (ipaddr, port) in
+        let sockaddr =
+          getaddrinfo ~log:self#log ~prefer_address hostname port
+        in
         self#log#important "Connecting to srt://%s:%d.." hostname port;
         (match Atomic.exchange socket None with
           | None -> ()
-          | Some (_, s) -> close_socket s);
-        let s = mk_socket ~payload_size ~messageapi () in
+          | Some (_, s) -> close_socket ~on_socket s);
+        let s =
+          mk_socket ~mode:`Connect ~on_socket ~payload_size ~messageapi ()
+        in
         try
           Srt.setsockflag s Srt.sndsyn true;
           Srt.setsockflag s Srt.rcvsyn true;
-          ignore
-            (Option.map (fun id -> Srt.(setsockflag s streamid id)) streamid);
-          ignore
-            (Option.map
-               (fun b -> Srt.(setsockflag s enforced_encryption b))
-               enforced_encryption);
-          ignore
-            (Option.map (fun len -> Srt.(setsockflag s pbkeylen len)) pbkeylen);
-          ignore
-            (Option.map (fun p -> Srt.(setsockflag s passphrase p)) passphrase);
-          ignore
-            (Option.map
-               (fun v -> Srt.(setsockflag s conntimeo v))
-               connection_timeout);
-          ignore
-            (Option.map (fun v -> Srt.(setsockflag s sndtimeo v)) write_timeout);
-          ignore
-            (Option.map (fun v -> Srt.(setsockflag s rcvtimeo v)) read_timeout);
-          Srt.connect s sockaddr;
-          Atomic.set socket (Some (sockaddr, s));
+          Utils.optional_apply
+            (fun id -> Srt.(setsockflag s streamid id))
+            streamid;
+          Utils.optional_apply
+            (fun b -> Srt.(setsockflag s enforced_encryption b))
+            enforced_encryption;
+          Utils.optional_apply
+            (fun len -> Srt.(setsockflag s pbkeylen len))
+            pbkeylen;
+          Utils.optional_apply
+            (fun p -> Srt.(setsockflag s passphrase p))
+            passphrase;
+          Utils.optional_apply
+            (fun v -> Srt.(setsockflag s conntimeo v))
+            connection_timeout;
+          Utils.optional_apply
+            (fun v -> Srt.(setsockflag s sndtimeo v))
+            write_timeout;
+          Utils.optional_apply
+            (fun v -> Srt.(setsockflag s rcvtimeo v))
+            read_timeout;
+          Srt.connect_posix_socket s sockaddr;
           self#log#important "Client connected!";
           !on_connect ();
+          Atomic.set socket (Some (Posix_socket.to_unix_sockaddr sockaddr, s));
           -1.
         with exn ->
           let bt = Printexc.get_raw_backtrace () in
@@ -542,7 +653,7 @@ class virtual caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
       (match Atomic.exchange socket None with
         | None -> ()
         | Some (_, socket) ->
-            close_socket socket;
+            close_socket ~on_socket socket;
             !on_disconnect ());
       Atomic.set task_should_stop true;
       match connect_task with
@@ -553,8 +664,8 @@ class virtual caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
   end
 
 class virtual listener ~enforced_encryption ~pbkeylen ~passphrase ~max_clients
-  ~listen_callback ~payload_size ~messageapi ~bind_address ~read_timeout
-  ~write_timeout ~on_connect ~on_disconnect () =
+  ~listen_callback ~payload_size ~messageapi ~bind_address ~port ~prefer_address
+  ~on_socket ~read_timeout ~write_timeout ~on_connect ~on_disconnect () =
   object (self)
     val mutable client_sockets = []
     method virtual id : string
@@ -572,76 +683,87 @@ class virtual listener ~enforced_encryption ~pbkeylen ~passphrase ~max_clients
     method private listening_socket =
       match Atomic.get listening_socket with
         | Some s -> s
-        | None ->
-            let s = mk_socket ~payload_size ~messageapi () in
-            Srt.bind s bind_address;
-            let max_clients_callback =
-              Option.map
-                (fun n _ _ _ _ ->
-                  self#mutexify (fun () -> List.length client_sockets < n) ())
-                max_clients
+        | None -> (
+            let s =
+              mk_socket ~mode:`Listen ~on_socket ~payload_size ~messageapi ()
             in
-            let listen_callback =
-              List.fold_left
-                (fun cur v ->
-                  match (cur, v) with
-                    | None, _ -> v
-                    | Some _, None -> cur
-                    | Some cur, Some fn ->
-                        Some
-                          (fun s hs_version peeraddr streamid ->
-                            cur s hs_version peeraddr streamid
-                            && fn s hs_version peeraddr streamid))
-                None
-                [max_clients_callback; listen_callback]
-            in
-            ignore
-              (Option.map (fun fn -> Srt.listen_callback s fn) listen_callback);
-            ignore
-              (Option.map
-                 (fun b -> Srt.(setsockflag s enforced_encryption b))
-                 enforced_encryption);
-            ignore
-              (Option.map
-                 (fun len -> Srt.(setsockflag s pbkeylen len))
-                 pbkeylen);
-            ignore
-              (Option.map
-                 (fun p -> Srt.(setsockflag s passphrase p))
-                 passphrase);
-            Srt.listen s (Option.value ~default:1 max_clients);
-            self#log#info "Setting up socket to listen at %s"
-              (string_of_address bind_address);
-            Atomic.set listening_socket (Some s);
-            s
+            try
+              Srt.bind_posix_socket s
+                (getaddrinfo ~log:self#log ~prefer_address bind_address port);
+              let max_clients_callback =
+                Option.map
+                  (fun n _ _ _ _ ->
+                    self#mutexify (fun () -> List.length client_sockets < n) ())
+                  max_clients
+              in
+              let listen_callback =
+                List.fold_left
+                  (fun cur v ->
+                    match (cur, v) with
+                      | None, _ -> v
+                      | Some _, None -> cur
+                      | Some cur, Some fn ->
+                          Some
+                            (fun s hs_version peeraddr streamid ->
+                              cur s hs_version peeraddr streamid
+                              && fn s hs_version peeraddr streamid))
+                  None
+                  [max_clients_callback; listen_callback]
+              in
+              Utils.optional_apply
+                (fun fn -> Srt.listen_callback s fn)
+                listen_callback;
+              Utils.optional_apply
+                (fun b -> Srt.(setsockflag s enforced_encryption b))
+                enforced_encryption;
+              Utils.optional_apply
+                (fun len -> Srt.(setsockflag s pbkeylen len))
+                pbkeylen;
+              Utils.optional_apply
+                (fun p -> Srt.(setsockflag s passphrase p))
+                passphrase;
+              Srt.listen s (Option.value ~default:1 max_clients);
+              self#log#info "Setting up socket to listen at %s:%i" bind_address
+                port;
+              Atomic.set listening_socket (Some s);
+              s
+            with exn ->
+              let bt = Printexc.get_raw_backtrace () in
+              Srt.close s;
+              Printexc.raise_with_backtrace exn bt)
 
     method private connect =
       let rec accept_connection s =
         try
           let client, origin = Srt.accept s in
-          Poll.add_socket ~mode:`Read s accept_connection;
-          (try self#log#info "New connection from %s" (string_of_address origin)
-           with exn ->
-             self#log#important "Error while fetching connection source: %s"
-               (Printexc.to_string exn));
-          Srt.(setsockflag client sndsyn true);
-          Srt.(setsockflag client rcvsyn true);
-          ignore
-            (Option.map
-               (fun v -> Srt.(setsockflag client sndtimeo v))
-               write_timeout);
-          ignore
-            (Option.map
-               (fun v -> Srt.(setsockflag client rcvtimeo v))
-               read_timeout);
-          if self#should_stop then (
-            close_socket client;
-            raise Done);
-          self#mutexify
-            (fun () ->
-              client_sockets <- (origin, client) :: client_sockets;
-              !on_connect ())
-            ()
+          try
+            Poll.add_socket ~mode:`Read s accept_connection;
+            (try
+               self#log#info "New connection from %s" (string_of_address origin)
+             with exn ->
+               self#log#important "Error while fetching connection source: %s"
+                 (Printexc.to_string exn));
+            Srt.(setsockflag client sndsyn true);
+            Srt.(setsockflag client rcvsyn true);
+            Utils.optional_apply
+              (fun v -> Srt.(setsockflag client sndtimeo v))
+              write_timeout;
+            Utils.optional_apply
+              (fun v -> Srt.(setsockflag client rcvtimeo v))
+              read_timeout;
+            on_socket ~mode:`Incoming client;
+            if self#should_stop then (
+              close_socket ~on_socket client;
+              raise Done);
+            self#mutexify
+              (fun () ->
+                client_sockets <- (origin, client) :: client_sockets;
+                !on_connect ())
+              ()
+          with exn ->
+            let bt = Printexc.get_raw_backtrace () in
+            Srt.close client;
+            Printexc.raise_with_backtrace exn bt
         with exn ->
           self#log#debug "Failed to connect: %s" (Printexc.to_string exn)
       in
@@ -655,12 +777,12 @@ class virtual listener ~enforced_encryption ~pbkeylen ~passphrase ~max_clients
       let should_stop = self#should_stop in
       self#mutexify
         (fun () ->
-          List.iter (fun (_, s) -> close_socket s) client_sockets;
+          List.iter (fun (_, s) -> close_socket ~on_socket s) client_sockets;
           client_sockets <- [];
           (match (should_stop, Atomic.get listening_socket) with
             | true, Some s ->
                 Poll.remove_socket s;
-                close_socket s;
+                close_socket ~on_socket s;
                 Atomic.set listening_socket None
             | _ -> ());
           !on_disconnect ())
@@ -785,9 +907,9 @@ class virtual input_base ~max ~self_sync ~on_connect ~on_disconnect
   end
 
 class input_listener ~enforced_encryption ~pbkeylen ~passphrase ~listen_callback
-  ~bind_address ~max ~payload_size ~self_sync ~on_connect ~on_disconnect
-  ~read_timeout ~write_timeout ~messageapi ~dump ~on_start ~on_stop ~autostart
-  format =
+  ~bind_address ~port ~prefer_address ~on_socket ~max ~payload_size ~self_sync
+  ~on_connect ~on_disconnect ~read_timeout ~write_timeout ~messageapi ~dump
+  ~on_start ~on_stop ~autostart format =
   object (self)
     inherit
       input_base
@@ -797,8 +919,9 @@ class input_listener ~enforced_encryption ~pbkeylen ~passphrase ~listen_callback
     inherit
       listener
         ~enforced_encryption ~pbkeylen ~passphrase ~listen_callback
-          ~max_clients:(Some 1) ~bind_address ~payload_size ~read_timeout
-          ~write_timeout ~messageapi ~on_connect ~on_disconnect ()
+          ~max_clients:(Some 1) ~bind_address ~port ~prefer_address ~on_socket
+          ~payload_size ~read_timeout ~write_timeout ~messageapi ~on_connect
+          ~on_disconnect ()
 
     method private get_socket =
       match self#get_sockets with
@@ -808,9 +931,9 @@ class input_listener ~enforced_encryption ~pbkeylen ~passphrase ~listen_callback
   end
 
 class input_caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
-  ~polling_delay ~hostname ~port ~max ~payload_size ~self_sync ~on_connect
-  ~on_disconnect ~read_timeout ~write_timeout ~connection_timeout ~messageapi
-  ~dump ~on_start ~on_stop ~autostart format =
+  ~polling_delay ~hostname ~port ~prefer_address ~max ~payload_size ~self_sync
+  ~on_connect ~on_disconnect ~read_timeout ~write_timeout ~connection_timeout
+  ~messageapi ~dump ~on_socket ~on_start ~on_stop ~autostart format =
   object (self)
     inherit
       input_base
@@ -820,8 +943,9 @@ class input_caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
     inherit
       caller
         ~enforced_encryption ~pbkeylen ~passphrase ~streamid ~polling_delay
-          ~hostname ~port ~payload_size ~read_timeout ~write_timeout
-          ~connection_timeout ~messageapi ~on_connect ~on_disconnect
+          ~hostname ~port ~prefer_address ~payload_size ~read_timeout
+          ~write_timeout ~connection_timeout ~messageapi ~on_connect
+          ~on_disconnect ~on_socket
 
     method get_sockets =
       match self#get_socket with s -> [s] | exception Not_found -> []
@@ -864,11 +988,13 @@ let _ =
         mode;
         hostname;
         port;
+        prefer_address;
         streamid;
         enforced_encryption;
         pbkeylen;
         passphrase;
         bind_address;
+        on_socket;
         listen_callback;
         polling_delay;
         read_timeout;
@@ -902,18 +1028,19 @@ let _ =
         | `Listener ->
             (new input_listener
                ~enforced_encryption ~pbkeylen ~passphrase ~listen_callback
-               ~bind_address ~read_timeout ~write_timeout ~payload_size
-               ~self_sync ~on_connect ~on_disconnect ~messageapi ~max ~dump
-               ~on_start ~on_stop ~autostart format
+               ~bind_address ~port ~prefer_address ~on_socket ~read_timeout
+               ~write_timeout ~payload_size ~self_sync ~on_connect
+               ~on_disconnect ~messageapi ~max ~dump ~on_start ~on_stop
+               ~autostart format
               :> < Start_stop.active_source
                  ; get_sockets : (Unix.sockaddr * Srt.socket) list >)
         | `Caller ->
             (new input_caller
                ~enforced_encryption ~pbkeylen ~passphrase ~streamid
-               ~polling_delay ~hostname ~port ~payload_size ~self_sync
-               ~on_connect ~read_timeout ~write_timeout ~connection_timeout
-               ~on_disconnect ~messageapi ~max ~dump ~on_start ~on_stop
-               ~autostart format
+               ~polling_delay ~hostname ~port ~prefer_address ~payload_size
+               ~self_sync ~on_connect ~read_timeout ~write_timeout
+               ~connection_timeout ~on_disconnect ~messageapi ~max ~dump
+               ~on_start ~on_socket ~on_stop ~autostart format
               :> < Start_stop.active_source
                  ; get_sockets : (Unix.sockaddr * Srt.socket) list >))
 
@@ -1010,8 +1137,9 @@ class virtual output_base ~payload_size ~messageapi ~on_start ~on_stop
 
 class output_caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
   ~polling_delay ~payload_size ~messageapi ~on_start ~on_stop ~infallible
-  ~register_telnet ~autostart ~on_connect ~on_disconnect ~port ~hostname
-  ~read_timeout ~write_timeout ~connection_timeout ~encoder_factory source =
+  ~register_telnet ~autostart ~on_socket ~on_connect ~on_disconnect
+  ~prefer_address ~port ~hostname ~read_timeout ~write_timeout
+  ~connection_timeout ~encoder_factory source =
   object (self)
     inherit
       output_base
@@ -1021,8 +1149,9 @@ class output_caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
     inherit
       caller
         ~enforced_encryption ~pbkeylen ~passphrase ~streamid ~polling_delay
-          ~hostname ~port ~payload_size ~read_timeout ~write_timeout
-          ~connection_timeout ~messageapi ~on_connect ~on_disconnect
+          ~hostname ~port ~prefer_address ~payload_size ~read_timeout
+          ~write_timeout ~connection_timeout ~messageapi ~on_connect
+          ~on_disconnect ~on_socket
 
     method private get_sockets =
       try [self#get_socket] with Not_connected -> []
@@ -1039,7 +1168,8 @@ class output_caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
 class output_listener ~enforced_encryption ~pbkeylen ~passphrase
   ~listen_callback ~max_clients ~payload_size ~messageapi ~on_start ~on_stop
   ~infallible ~register_telnet ~autostart ~on_connect ~on_disconnect
-  ~bind_address ~read_timeout ~write_timeout ~encoder_factory source =
+  ~bind_address ~port ~prefer_address ~on_socket ~read_timeout ~write_timeout
+  ~encoder_factory source =
   object (self)
     inherit
       output_base
@@ -1048,9 +1178,10 @@ class output_listener ~enforced_encryption ~pbkeylen ~passphrase
 
     inherit
       listener
-        ~bind_address ~payload_size ~read_timeout ~write_timeout ~messageapi
-          ~on_connect ~on_disconnect ~enforced_encryption ~pbkeylen ~passphrase
-          ~listen_callback ~max_clients ()
+        ~bind_address ~port ~prefer_address ~on_socket ~payload_size
+          ~read_timeout ~write_timeout ~messageapi ~on_connect ~on_disconnect
+          ~enforced_encryption ~pbkeylen ~passphrase ~listen_callback
+          ~max_clients ()
 
     method private client_error socket exn bt =
       Utils.log_exception ~log:self#log
@@ -1059,7 +1190,7 @@ class output_listener ~enforced_encryption ~pbkeylen ~passphrase
            (Printexc.to_string exn));
       self#mutexify
         (fun () ->
-          close_socket socket;
+          close_socket ~on_socket socket;
           client_sockets <-
             List.filter (fun (_, s) -> s <> socket) client_sockets)
         ()
@@ -1090,11 +1221,13 @@ let _ =
         mode;
         hostname;
         port;
+        prefer_address;
         streamid;
         enforced_encryption;
         pbkeylen;
         passphrase;
         bind_address;
+        on_socket;
         listen_callback;
         polling_delay;
         read_timeout;
@@ -1135,18 +1268,18 @@ let _ =
         | `Caller ->
             (new output_caller
                ~enforced_encryption ~pbkeylen ~passphrase ~streamid
-               ~polling_delay ~hostname ~port ~payload_size ~autostart ~on_start
-               ~on_stop ~read_timeout ~write_timeout ~connection_timeout
-               ~infallible ~register_telnet ~messageapi ~encoder_factory
-               ~on_connect ~on_disconnect source
+               ~polling_delay ~hostname ~port ~prefer_address ~payload_size
+               ~autostart ~on_start ~on_stop ~read_timeout ~write_timeout
+               ~connection_timeout ~infallible ~register_telnet ~messageapi
+               ~encoder_factory ~on_socket ~on_connect ~on_disconnect source
               :> < Output.output
                  ; get_sockets : (Unix.sockaddr * Srt.socket) list >)
         | `Listener ->
             (new output_listener
-               ~enforced_encryption ~pbkeylen ~passphrase ~bind_address
-               ~read_timeout ~write_timeout ~payload_size ~autostart ~on_start
-               ~on_stop ~infallible ~register_telnet ~messageapi
-               ~encoder_factory ~on_connect ~on_disconnect ~listen_callback
-               ~max_clients source
+               ~enforced_encryption ~pbkeylen ~passphrase ~bind_address ~port
+               ~prefer_address ~on_socket ~read_timeout ~write_timeout
+               ~payload_size ~autostart ~on_start ~on_stop ~infallible
+               ~register_telnet ~messageapi ~encoder_factory ~on_connect
+               ~on_disconnect ~listen_callback ~max_clients source
               :> < Output.output
                  ; get_sockets : (Unix.sockaddr * Srt.socket) list >))
