@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -41,11 +41,11 @@ module Tutils = struct
     let lock = Mutex.create () in
     let should_stop = ref false in
     let has_stopped = ref false in
-    let kill = mutexify lock (fun () -> should_stop := true) in
+    let kill = Mutex_utils.mutexify lock (fun () -> should_stop := true) in
     let wait () = wait cond lock (fun () -> !has_stopped) in
-    let should_stop = mutexify lock (fun () -> !should_stop) in
+    let should_stop = Mutex_utils.mutexify lock (fun () -> !should_stop) in
     let has_stopped =
-      mutexify lock (fun () ->
+      Mutex_utils.mutexify lock (fun () ->
           has_stopped := true;
           Condition.signal cond)
     in
@@ -57,7 +57,7 @@ class output ~on_start ~on_stop ~register_telnet ~infallible ~autostart
   ~hostname ~port ~encoder_factory source =
   object (self)
     inherit
-      Output.encoded
+      [Strings.t] Output.encoded
         ~output_kind:"udp" ~on_start ~on_stop ~register_telnet ~infallible
           ~autostart ~export_cover_metadata:false
         ~name:(Printf.sprintf "udp://%s:%d" hostname port)
@@ -84,7 +84,7 @@ class output ~on_start ~on_stop ~register_telnet ~infallible ~autostart
                 f (pos + len))
             in
             f 0);
-      encoder <- Some (encoder_factory self#id Export_metadata.empty_metadata)
+      encoder <- Some (encoder_factory self#id Frame.Metadata.Export.empty)
 
     method! private reset =
       self#start;
@@ -94,8 +94,7 @@ class output ~on_start ~on_stop ~register_telnet ~infallible ~autostart
       socket_send <- None;
       encoder <- None
 
-    method private encode frame ofs len =
-      (Option.get encoder).Encoder.encode frame ofs len
+    method private encode frame = (Option.get encoder).Encoder.encode frame
 
     method private insert_metadata m =
       (Option.get encoder).Encoder.insert_metadata m
@@ -108,18 +107,21 @@ class output ~on_start ~on_stop ~register_telnet ~infallible ~autostart
 class input ~hostname ~port ~get_stream_decoder ~bufferize =
   let max_length = Some (2 * Frame.main_of_seconds bufferize) in
   object (self)
-    inherit Generated.source ~empty_on_abort:false ~bufferize ()
+    inherit Generated.source ~empty_on_abort:false ~bufferize () as generated
 
     inherit!
       Start_stop.active_source
-        ~name:"input.udp" ~clock_safe:false ~fallible:true ~on_start:ignore
-          ~on_stop:ignore ~autostart:true () as super
+        ~name:"input.udp" ~fallible:true ~on_start:ignore ~on_stop:ignore
+          ~autostart:true () as super
 
     val mutable kill_feeding = None
     val mutable wait_feeding = None
     val mutable decoder_factory = None
     method private decoder_factory = Option.get decoder_factory
 
+    method! private can_generate_frame =
+      super#started && generated#can_generate_frame
+
     method private start =
       begin
         let decoder args =
@@ -137,9 +139,9 @@ class input ~hostname ~port ~get_stream_decoder ~bufferize =
       kill_feeding <- Some kill;
       wait_feeding <- Some wait
 
-    method! wake_up act =
-      super#wake_up act;
-      Generator.set_max_length self#buffer max_length
+    initializer
+      self#on_wake_up (fun () ->
+          Generator.set_max_length self#buffer max_length)
 
     method private stop =
       (Option.get kill_feeding) ();
@@ -168,14 +170,15 @@ class input ~hostname ~port ~get_stream_decoder ~bufferize =
         n
       in
       let input = { Decoder.read; tell = None; length = None; lseek = None } in
+      let decoder, buffer = self#decoder_factory input in
       try
         (* Feeding loop. *)
-        let decoder, buffer = self#decoder_factory input in
         while true do
           if should_stop () then failwith "stop";
           decoder.Decoder.decode buffer
         done
       with e ->
+        decoder.Decoder.close ();
         Generator.add_track_mark self#buffer;
 
         (* Closing the socket is slightly overkill but
@@ -223,7 +226,7 @@ let _ =
       let hostname = Lang.to_string (List.assoc "host" p) in
       let fmt =
         let fmt = Lang.assoc "" 1 p in
-        try (Encoder.get_factory (Lang.to_format fmt)) ~pos:fmt.Value.pos
+        try (Encoder.get_factory (Lang.to_format fmt)) ~pos:(Value.pos fmt)
         with Not_found ->
           raise
             (Error.Invalid_value
