@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 
  *****************************************************************************)
 
@@ -29,46 +29,17 @@ type t = Liquidsoap_lang.Type.t
 
 type module_name = Liquidsoap_lang.Lang.module_name
 type scheme = Liquidsoap_lang.Type.scheme
-type regexp
+type regexp = Liquidsoap_lang.Lang.regexp
 
 (** {2 Values} *)
 
-(** A typed value. *)
-module Ground : sig
-  type t = Liquidsoap_lang.Term.Ground.t = ..
-  type t += Bool of bool | Int of int | String of string | Float of float
+module Custom = Value.Custom
+module Methods = Term.Methods
+module Flags = Liquidsoap_lang.Flags
 
-  type content = Liquidsoap_lang.Term.Ground.content = {
-    descr : t -> string;
-    to_json : pos:Liquidsoap_lang.Pos.t list -> t -> Json.t;
-    compare : t -> t -> int;
-    typ : (module Liquidsoap_lang.Type.Ground.Custom);
-  }
-
-  val register : (t -> bool) -> content -> unit
-  val to_string : t -> string
-end
-
-type value = Liquidsoap_lang.Value.t = {
-  pos : Liquidsoap_lang.Pos.Option.t;
-  value : in_value;
-  methods : value Liquidsoap_lang.Value.Methods.t;
-  id : int;
-}
-
-and env = (string * value) list
-and lazy_env = (string * value Lazy.t) list
-
-and in_value = Liquidsoap_lang.Value.in_value =
-  | Ground of Ground.t
-  | List of value list
-  | Tuple of value list
-  | Null
-  | Fun of
-      (string * string * value option) list * lazy_env * Liquidsoap_lang.Term.t
-  (* A function with given arguments (argument label, argument variable, default
-     value), closure and value. *)
-  | FFI of (string * string * value option) list * (env -> value)
+type in_value = Liquidsoap_lang.Value.in_value
+type env = Liquidsoap_lang.Value.env
+type value = Liquidsoap_lang.Value.t
 
 val demeth : value -> value
 val split_meths : value -> (string * value) list * value
@@ -82,18 +53,16 @@ val iter_sources :
 
 (** {2 Computation} *)
 
-val apply_fun : (?pos:Liquidsoap_lang.Pos.t -> value -> env -> value) ref
-
 (** Multiapply a value to arguments. The argument [t] is the type of the result
    of the application. *)
-val apply : value -> env -> value
+val apply : ?pos:Liquidsoap_lang.Pos.t list -> value -> env -> value
 
 (** {3 Helpers for registering protocols} *)
 
 val add_protocol :
   syntax:string ->
   doc:string ->
-  static:bool ->
+  static:(string -> bool) ->
   string ->
   Request.resolver ->
   unit
@@ -134,7 +103,7 @@ val add_builtin_base :
   ?flags:Doc.Value.flag list ->
   ?base:module_name ->
   string ->
-  in_value ->
+  Liquidsoap_lang.Value.in_value ->
   t ->
   module_name
 
@@ -206,7 +175,7 @@ val to_ref : value -> (unit -> value) * (value -> unit)
 val to_valued_ref :
   (value -> 'a) -> ('a -> value) -> value -> (unit -> 'a) * ('a -> unit)
 
-val to_http_transport : value -> Http.transport
+val to_http_transport : value -> Liq_http.transport
 
 (** [assoc x n l] returns the [n]-th [y] such that [(x,y)] is in the list [l].
   * This is useful for retrieving arguments of a function. *)
@@ -265,7 +234,7 @@ val metadata_t : t
 (** A getter on an arbitrary type. *)
 val getter_t : t -> t
 
-(** Abstract http transport with all methods *)
+(** Custom http transport with all methods *)
 val http_transport_t : t
 
 (** Same with no methods. *)
@@ -273,6 +242,8 @@ val http_transport_base_t : t
 
 val unit : value
 val int : int -> value
+val octal_int : int -> value
+val hex_int : int -> value
 val bool : bool -> value
 val float : float -> value
 val string : string -> value
@@ -287,8 +258,8 @@ val tuple : value list -> value
 val meth : value -> (string * value) list -> value
 val record : (string * value) list -> value
 val reference : (unit -> value) -> (value -> unit) -> value
-val http_transport : Http.transport -> value
-val base_http_transport : Http.transport -> value
+val http_transport : Liq_http.transport -> value
+val base_http_transport : Liq_http.transport -> value
 
 (** Build a function from an OCaml function. Items in the prototype indicate
     the label and optional values. Second string value is used when renaming
@@ -335,11 +306,28 @@ val descr_of_regexp : regexp -> string
 (** Return a string description of a regexp value i.e. r/^foo\/bla$/g *)
 val string_of_regexp : regexp -> string
 
-module Regexp : sig
-  include Liquidsoap_lang.Regexp.T with type t := regexp
+type stdlib = [ `Disabled | `If_present | `Force | `Override of string ]
 
-  type sub = Liquidsoap_lang.Regexp.sub = {
-    matches : string option list;
-    groups : (string * string) list;
-  }
-end
+(** Type a term, possibly returning the cached term instead. *)
+val type_term :
+  ?name:string ->
+  ?cache:bool ->
+  ?trim:bool ->
+  ?deprecated:bool ->
+  ?ty:t ->
+  stdlib:stdlib ->
+  parsed_term:Liquidsoap_lang.Parsed_term.t ->
+  Term.t ->
+  Term.t
+
+(** Evaluate a term. *)
+val eval :
+  ?toplevel:bool ->
+  ?typecheck:bool ->
+  ?cache:bool ->
+  ?deprecated:bool ->
+  ?ty:t ->
+  ?name:string ->
+  stdlib:stdlib ->
+  string ->
+  value
