@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,13 +20,102 @@
 
  *****************************************************************************)
 
-open Liquidsoap_lang.Lang
+module Lang = Liquidsoap_lang.Lang
+module Flags = Liquidsoap_lang.Flags
+open Lang
 
-module Alive_values_map = Liquidsoap_lang.Active_value.Make (struct
-  type t = Value.t
+module ClockValue = struct
+  include Value.MkCustom (struct
+    type content = Clock.t
 
-  let id v = v.Value.id
-end)
+    let name = "clock"
+    let to_string = Clock.descr
+
+    let to_json ~pos _ =
+      Lang.raise_error ~message:"Clocks cannot be represented as json" ~pos
+        "json"
+
+    let compare = Stdlib.compare
+  end)
+
+  let base_t = t
+  let to_base_value = to_value
+
+  let methods =
+    [
+      ( "id",
+        Lang.fun_t [] Lang.string_t,
+        "The clock's id",
+        fun c -> Lang.val_fun [] (fun _ -> Lang.string (Clock.id c)) );
+      ( "sync",
+        Lang.fun_t [] Lang.string_t,
+        "The clock's current sync mode. One of: `\"stopped\"`, `\"stopping\"`, \
+         `\"auto\"`, `\"CPU\"`, `\"unsynced\"` or `\"passive\"`.",
+        fun c ->
+          Lang.val_fun [] (fun _ ->
+              Lang.string Clock.(string_of_sync_mode (sync c))) );
+      ( "start",
+        Lang.fun_t [(true, "force", Lang.bool_t)] Lang.unit_t,
+        "Start the clock.",
+        fun c ->
+          Lang.val_fun
+            [("force", "force", Some (Lang.bool true))]
+            (fun p ->
+              let pos = Lang.pos p in
+              let force = Lang.to_bool (List.assoc "force" p) in
+              try
+                Clock.start ~force c;
+                Lang.unit
+              with Clock.Invalid_state ->
+                Runtime_error.raise
+                  ~message:
+                    (Printf.sprintf "Invalid clock state: %s"
+                       Clock.(string_of_sync_mode (sync c)))
+                  ~pos "clock") );
+      ( "stop",
+        Lang.fun_t [] Lang.unit_t,
+        "Stop the clock. Does nothing if the clock is stopping or stopped.",
+        fun c ->
+          Lang.val_fun [] (fun _ ->
+              Clock.stop c;
+              Lang.unit) );
+      ( "self_sync",
+        Lang.fun_t [] Lang.bool_t,
+        "`true` if the clock is in control of its latency.",
+        fun c -> Lang.val_fun [] (fun _ -> Lang.bool (Clock.self_sync c)) );
+      ( "unify",
+        Lang.fun_t [(false, "", base_t)] Lang.unit_t,
+        "Unify the clock with another one. One of the two clocks should be in \
+         `\"stopped\"` sync mode.",
+        fun c ->
+          Lang.val_fun
+            [("", "", None)]
+            (fun p ->
+              let pos = match Lang.pos p with p :: _ -> Some p | [] -> None in
+              let c' = of_value (List.assoc "" p) in
+              Clock.unify ~pos c c';
+              Lang.unit) );
+      ( "tick",
+        Lang.fun_t [] Lang.unit_t,
+        "Animate the clock and run one tick",
+        fun c ->
+          Lang.val_fun [] (fun _ ->
+              Clock.tick c;
+              Lang.unit) );
+      ( "ticks",
+        Lang.fun_t [] Lang.int_t,
+        "The total number of times the clock has ticked.",
+        fun c -> Lang.val_fun [] (fun _ -> Lang.int (Clock.ticks c)) );
+    ]
+
+  let t =
+    Lang.method_t base_t
+      (List.map (fun (lbl, typ, descr, _) -> (lbl, ([], typ), descr)) methods)
+
+  let to_value c =
+    Lang.meth (to_base_value c)
+      (List.map (fun (lbl, _, _, v) -> (lbl, v c)) methods)
+end
 
 let log = Log.make ["lang"]
 let metadata_t = list_t (product_t string_t string_t)
@@ -38,26 +127,23 @@ let to_metadata_list t =
   in
   List.map pop (to_list t)
 
-let to_metadata t =
-  let t = to_metadata_list t in
-  let metas = Hashtbl.create 10 in
-  List.iter (fun (a, b) -> Hashtbl.add metas a b) t;
-  metas
+let to_metadata t = Frame.Metadata.from_list (to_metadata_list t)
 
 let metadata_list m =
   list (List.map (fun (k, v) -> product (string k) (string v)) m)
 
-let metadata m =
-  list (Hashtbl.fold (fun k v l -> product (string k) (string v) :: l) m [])
-
+let metadata m = metadata_list (Frame.Metadata.to_list m)
 let metadata_track_t = Format_type.metadata
 let track_marks_t = Format_type.track_marks
 
-module Source_val = Liquidsoap_lang.Lang_core.MkAbstract (struct
+module Source_val = Liquidsoap_lang.Lang_core.MkCustom (struct
   type content = Source.source
 
   let name = "source"
-  let descr s = Printf.sprintf "<source#%s>" s#id
+
+  let to_string s =
+    Printf.sprintf "<source(id=%s, frame_type=%s>" s#id
+      (Type.to_string s#frame_type)
 
   let to_json ~pos _ =
     Runtime_error.raise ~pos
@@ -78,7 +164,16 @@ let source_methods =
       "Indicate if a source is ready to stream. This does not mean that the \
        source is currently streaming, just that its resources are all properly \
        initialized.",
-      fun (s : Source.source) -> val_fun [] (fun _ -> bool (s#is_ready ())) );
+      fun s -> val_fun [] (fun _ -> bool s#is_ready) );
+    ( "reset_last_metadata_on_track",
+      ([], ref_t bool_t),
+      "If `true`, the source's `last_metadata` is reset on each new track. If \
+       a metadata is present along with the track mark, then it becomes the \
+       new `last_metadata`, otherwise, `last_metadata becomes `null`.",
+      fun s ->
+        reference
+          (fun () -> bool s#reset_last_metadata_on_track)
+          (fun b -> s#set_reset_last_metadata_on_track (to_bool b)) );
     ( "buffered",
       ([], fun_t [] (list_t (product_t string_t float_t))),
       "Length of buffered data.",
@@ -101,6 +196,37 @@ let source_methods =
       fun s ->
         val_fun [] (fun _ ->
             match s#last_metadata with None -> null | Some m -> metadata m) );
+    ( "register_command",
+      ( [],
+        fun_t
+          [
+            (true, "usage", Lang.nullable_t Lang.string_t);
+            (false, "description", Lang.string_t);
+            (false, "", Lang.string_t);
+            (false, "", Lang.fun_t [(false, "", Lang.string_t)] Lang.string_t);
+          ]
+          unit_t ),
+      "Register a server command for this source. Command is registered under \
+       the source's id namespace when it gets up and de-registered when it \
+       gets down.",
+      fun s ->
+        val_fun
+          [
+            ("usage", "usage", Some Lang.null);
+            ("description", "description", None);
+            ("", "", None);
+            ("", "", None);
+          ]
+          (fun p ->
+            let usage =
+              Lang.to_valued_option Lang.to_string (List.assoc "usage" p)
+            in
+            let descr = Lang.to_string (List.assoc "description" p) in
+            let command = Lang.to_string (Lang.assoc "" 1 p) in
+            let f = Lang.assoc "" 2 p in
+            let f x = Lang.to_string (Lang.apply f [("", Lang.string x)]) in
+            s#register_command ?usage ~descr command f;
+            unit) );
     ( "on_metadata",
       ([], fun_t [(false, "", fun_t [(false, "", metadata_t)] unit_t)] unit_t),
       "Call a given handler on metadata packets.",
@@ -169,7 +295,7 @@ let source_methods =
     ( "self_sync",
       ([], fun_t [] bool_t),
       "Is the source currently controlling its own real-time loop.",
-      fun s -> val_fun [] (fun _ -> bool (snd s#self_sync)) );
+      fun s -> val_fun [] (fun _ -> bool (snd s#self_sync <> None)) );
     ( "log",
       ( [],
         record_t
@@ -212,7 +338,9 @@ let source_methods =
       "`true` if the source is active, i.e. it is continuously animated by its \
        own clock whenever it is ready. Typically, `true` for outputs and \
        sources such as `input.http`.",
-      fun s -> val_fun [] (fun _ -> bool s#is_active) );
+      fun s ->
+        val_fun [] (fun _ ->
+            bool (match s#source_type with `Passive -> false | _ -> true)) );
     ( "seek",
       ([], fun_t [(false, "", float_t)] float_t),
       "Seek forward, in seconds (returns the amount of time effectively \
@@ -235,32 +363,36 @@ let source_methods =
     ( "fallible",
       ([], bool_t),
       "Indicate if a source may fail, i.e. may not be ready to stream.",
-      fun s -> bool (s#stype = `Fallible) );
+      fun s -> bool s#fallible );
+    ( "clock",
+      ([], ClockValue.base_t),
+      "The source's clock",
+      fun s -> ClockValue.to_base_value s#clock );
     ( "time",
       ([], fun_t [] float_t),
       "Get a source's time, based on its assigned clock.",
       fun s ->
         val_fun [] (fun _ ->
-            let ticks =
-              if Source.Clock_variables.is_known s#clock then
-                (Source.Clock_variables.get s#clock)#get_tick
-              else 0
-            in
+            let ticks = Clock.ticks s#clock in
             let frame_position =
               Lazy.force Frame.duration *. float_of_int ticks
             in
             let in_frame_position =
-              Frame.seconds_of_main (Frame.position s#memo)
+              if s#is_ready then Frame.(seconds_of_main (position s#get_frame))
+              else 0.
             in
             float (frame_position +. in_frame_position)) );
   ]
 
+let make_t ?pos = Type.make ?pos:(Option.map Pos.of_lexing_pos pos)
+
 let source_methods_t t =
   method_t t (List.map (fun (name, t, doc, _) -> (name, t, doc)) source_methods)
 
-let source_t ?(methods = false) frame_t =
+let source_t ?(pos : Liquidsoap_lang.Term_base.parsed_pos option)
+    ?(methods = false) frame_t =
   let t =
-    Type.make
+    make_t ?pos
       (Type.Constr
          (* The type has to be invariant because we don't want the sup mechanism to be used here, see #2806. *)
          { Type.constructor = "source"; params = [(`Invariant, frame_t)] })
@@ -277,17 +409,6 @@ let source_tracks_t frame_t =
     ([], Format_type.track_marks)
     (Type.meth "metadata" ([], Format_type.metadata) frame_t)
 
-let source_tracks s =
-  meth unit
-    (( Frame.Fields.string_of_field Frame.Fields.metadata,
-       Track.to_value (Frame.Fields.metadata, s) )
-    :: ( Frame.Fields.string_of_field Frame.Fields.track_marks,
-         Track.to_value (Frame.Fields.track_marks, s) )
-    :: List.map
-         (fun (field, _) ->
-           (Frame.Fields.string_of_field field, Track.to_value (field, s)))
-         (Frame.Fields.bindings s#content_type))
-
 let source_methods ~base s =
   meth base (List.map (fun (name, _, _, fn) -> (name, fn s)) source_methods)
 
@@ -301,17 +422,23 @@ let to_track = Track.of_value
     the currently defined source as argument). *)
 type 'a operator_method = string * scheme * string * ('a -> value)
 
-let checked_values = Alive_values_map.create 10
+let has_value_flag v flag =
+  match v with
+    | Value.Float _ | Value.String _ | Value.Bool _ | Value.Null _ -> true
+    | v -> Value.has_flag v flag
+
+let add_value_flag v flag =
+  match v with
+    | Value.Float _ | Value.String _ | Value.Bool _ | Value.Null _ -> ()
+    | v -> Value.add_flag v flag
 
 (** Ensure that the frame contents of all the sources occurring in the value agree with [t]. *)
 let check_content v t =
   let check t t' = Typing.(t <: t') in
   let rec check_value v t =
-    if not (Alive_values_map.mem checked_values v) then (
-      (* We need to avoid checking the same value multiple times, otherwise we
-         get an exponential blowup, see #1247. *)
-      Alive_values_map.add checked_values v;
-      match (v.Value.value, (Type.deref t).Type.descr) with
+    if not (has_value_flag v Flags.checked_value) then (
+      add_value_flag v Flags.checked_value;
+      match (v, (Type.deref t).Type.descr) with
         | _, Type.Var _ -> ()
         | _ when Source_val.is_value v ->
             let source_t = source_t (Source_val.of_value v)#frame_type in
@@ -334,28 +461,81 @@ let check_content v t =
             let frame_t = Frame_type.make unit_t content_t in
             let encoder_t = Lang_encoder.L.format_t frame_t in
             check encoder_t t
-        | Value.Ground _, _ -> ()
-        | Value.List l, Type.List { Type.t } ->
+        | Value.Int _, _
+        | Value.Float _, _
+        | Value.String _, _
+        | Value.Bool _, _
+        | Value.Custom _, _ ->
+            ()
+        | Value.List { value = l }, Type.List { Type.t } ->
             List.iter (fun v -> check_value v t) l
-        | Value.Tuple l, Type.Tuple t -> List.iter2 check_value l t
-        | Value.Null, _ -> ()
+        | Value.Tuple { value = l }, Type.Tuple t -> List.iter2 check_value l t
+        | Value.Null _, _ -> ()
+        | _, Type.Nullable t -> check_value v t
         (* Value can have more methods than the type requires so check from the type here. *)
         | _, Type.Meth _ ->
             let meths, v = Value.split_meths v in
             let meths_t, t = Type.split_meths t in
             List.iter
-              (fun { Type.meth; optional; scheme = s } ->
-                let t = Typing.instantiate ~level:(-1) s in
+              (fun { Type.meth; optional; scheme = generalized, t } ->
+                let names = List.map (fun v -> v.Type.name) generalized in
+                let handler =
+                  Type.Fresh.init
+                    ~selector:(fun v -> List.mem v.Type.name names)
+                    ()
+                in
+                let t = Type.Fresh.make handler t in
                 try check_value (List.assoc meth meths) t
                 with Not_found when optional -> ())
               meths_t;
             check_value v t
-        (* We don't check functions, assuming anything creating a source is a
-           FFI registered via add_operator so the check will happen there. *)
-        | Fun _, _ | FFI _, _ -> ()
+        | Value.Fun { fun_args = []; fun_body = ret }, Type.Getter t ->
+            Typing.(ret.Term.t <: t)
+        | Value.FFI ({ ffi_args = []; ffi_fn } as ffi), Type.Getter t ->
+            ffi.ffi_fn <-
+              (fun env ->
+                let v = ffi_fn env in
+                check_value v t;
+                v)
+        | ( Value.Fun { fun_args = args; fun_body = ret },
+            Type.Arrow (args_t, ret_t) ) ->
+            List.iter
+              (fun typ ->
+                match typ with
+                  | true, lbl_t, typ ->
+                      List.iter
+                        (fun arg ->
+                          match arg with
+                            | lbl, _, Some v when lbl = lbl_t ->
+                                check_value v typ
+                            | _ -> ())
+                        args
+                  | _ -> ())
+              args_t;
+            Typing.(ret.Term.t <: ret_t)
+        | Value.FFI ({ ffi_args; ffi_fn } as ffi), Type.Arrow (args_t, ret_t) ->
+            List.iter
+              (fun typ ->
+                match typ with
+                  | true, lbl_t, typ ->
+                      List.iter
+                        (fun arg ->
+                          match arg with
+                            | lbl, _, Some v when lbl = lbl_t ->
+                                check_value v typ
+                            | _ -> ())
+                        ffi_args
+                  | _ -> ())
+              args_t;
+            ffi.ffi_fn <-
+              (fun env ->
+                let v = ffi_fn env in
+                check_value v ret_t;
+                v)
         | _ ->
             failwith
-              ("Unhandled value in check_content: " ^ Value.to_string v ^ "."))
+              (Printf.sprintf "Unhandled value in check_content: %s, type: %s."
+                 (Value.to_string v) (Type.to_string t)))
   in
   check_value v t
 
@@ -375,26 +555,51 @@ let check_content v t =
 let _meth = meth
 
 let check_arguments ~env ~return_t arguments =
-  (* Create a fresh instantiation of the return type and the type of arguments. *)
-  let return_t, arguments =
-    let s =
-      (* TODO: level -1 generalization is abusive, but it should be a good enough approximation for now *)
-      Typing.generalize ~level:(-1)
-        (Type.make
-           (Type.Tuple (return_t :: List.map (fun (_, t, _, _) -> t) arguments)))
-    in
-    let t = Typing.instantiate ~level:(-1) s in
-    match t.Type.descr with
-      | Type.Tuple (return_t :: arguments_t) ->
-          ( return_t,
-            List.map2
-              (fun (name, _, _, _) typ -> (name, typ))
-              arguments arguments_t )
-      | _ -> assert false
+  let handler = Type.Fresh.init () in
+  let return_t = Type.Fresh.make handler return_t in
+  let arguments =
+    List.map (fun (lbl, t, _, _) -> (lbl, Type.Fresh.make handler t)) arguments
   in
   let arguments =
     List.stable_sort (fun (l, _) (l', _) -> Stdlib.compare l l') arguments
   in
+  (* Generalize all terms inside the arguments *)
+  let map =
+    let open Liquidsoap_lang.Value in
+    let rec map v =
+      let v =
+        match v with
+          | Int _ | Float _ | String _ | Bool _ | Custom _ | Null _ -> v
+          | List ({ value = l } as v) -> List { v with value = List.map map l }
+          | Tuple ({ value = l } as v) ->
+              Tuple { v with value = List.map map l }
+          | Fun ({ fun_args = args; fun_body = ret } as fun_v) ->
+              Fun
+                {
+                  fun_v with
+                  fun_args =
+                    List.map (fun (l, l', v) -> (l, l', Option.map map v)) args;
+                  fun_body = Term.fresh ~handler ret;
+                }
+          | FFI ffi ->
+              FFI
+                {
+                  ffi with
+                  ffi_args =
+                    List.map
+                      (fun (l, l', v) -> (l, l', Option.map map v))
+                      ffi.ffi_args;
+                  ffi_fn =
+                    (fun env ->
+                      let v = ffi.ffi_fn env in
+                      map v);
+                }
+      in
+      map_methods v (Methods.map map)
+    in
+    map
+  in
+  let env = List.map (fun (lbl, v) -> (lbl, map v)) env in
   (* Negotiate content for all sources and formats in the arguments. *)
   let () =
     let env =
@@ -410,7 +615,7 @@ let check_arguments ~env ~return_t arguments =
         check_content v typ)
       arguments env
   in
-  return_t
+  (return_t, env)
 
 let add_operator ~(category : Doc.Value.source) ~descr ?(flags = [])
     ?(meth = ([] : 'a operator_method list)) ?base name arguments ~return_t f =
@@ -429,14 +634,9 @@ let add_operator ~(category : Doc.Value.source) ~descr ?(flags = [])
     :: List.stable_sort compare arguments
   in
   let f env =
-    let pos =
-      match Liquidsoap_lang.Lang_core.pos env with
-        | [] -> None
-        | p :: _ -> Some p
-    in
-    let return_t = check_arguments ~return_t ~env arguments in
+    let return_t, env = check_arguments ~return_t ~env arguments in
     let src : < Source.source ; .. > = f env in
-    src#set_pos pos;
+    src#set_stack (Liquidsoap_lang.Lang_core.pos env);
     Typing.(src#frame_type <: return_t);
     ignore
       (Option.map
@@ -469,14 +669,9 @@ let add_track_operator ~(category : Doc.Value.source) ~descr ?(flags = [])
     :: arguments
   in
   let f env =
-    let pos =
-      match Liquidsoap_lang.Lang_core.pos env with
-        | [] -> None
-        | p :: _ -> Some p
-    in
-    let return_t = check_arguments ~return_t ~env arguments in
+    let return_t, env = check_arguments ~return_t ~env arguments in
     let field, (src : < Source.source ; .. >) = f env in
-    src#set_pos pos;
+    src#set_stack (Liquidsoap_lang.Lang_core.pos env);
     (if field <> Frame.Fields.track_marks && field <> Frame.Fields.metadata then
        Typing.(
          src#frame_type
@@ -496,104 +691,67 @@ let add_track_operator ~(category : Doc.Value.source) ~descr ?(flags = [])
   let category = `Track category in
   add_builtin ~category ~descr ~flags ?base name arguments return_t f
 
-let itered_values = Alive_values_map.create 10
-
 let iter_sources ?(on_imprecise = fun () -> ()) f v =
   let rec iter_term env v =
     let iter_base_term env v =
       match v.Term.term with
-        | Term.Ground _ | Term.Encoder _ -> ()
-        | Term.List l -> List.iter (iter_term env) l
-        | Term.Tuple l -> List.iter (iter_term env) l
-        | Term.Null -> ()
-        | Term.Cast (a, _) -> iter_term env a
-        | Term.Invoke { Term.invoked = a } -> iter_term env a
-        | Term.Open (a, b) ->
+        | `Cache_env _ | `Int _ | `Float _ | `Bool _ | `String _ | `Custom _
+        | `Encoder _ ->
+            ()
+        | `List l -> List.iter (iter_term env) l
+        | `Tuple l -> List.iter (iter_term env) l
+        | `Null -> ()
+        | `Hide (a, _) -> iter_term env a
+        | `Cast { Term.cast = a } -> iter_term env a
+        | `Invoke { Term.invoked = a } -> iter_term env a
+        | `Open (a, b) ->
             iter_term env a;
             iter_term env b
-        | Term.Let { Term.def = a; body = b; _ } | Term.Seq (a, b) ->
+        | `Let { Term.def = a; body = b; _ } | `Seq (a, b) ->
             iter_term env a;
             iter_term env b
-        | Term.Var v -> (
+        | `Var v -> (
             try
               (* If it's locally bound it won't be in [env]. *)
               (* TODO since inner-bound variables don't mask outer ones in [env],
                *   we are actually checking values that may be out of reach. *)
               let v = List.assoc v env in
-              if Lazy.is_val v then (
-                let v = Lazy.force v in
-                iter_value v)
-              else ()
+              iter_value v
             with Not_found -> ())
-        | Term.App (a, l) ->
+        | `App (a, l) ->
             iter_term env a;
             List.iter (fun (_, v) -> iter_term env v) l
-        | Term.Fun (_, proto, body) | Term.RFun (_, _, proto, body) ->
+        | `Fun { Term.arguments; body } | `RFun (_, { Term.arguments; body }) ->
             iter_term env body;
             List.iter
-              (fun (_, _, _, v) ->
-                match v with Some v -> iter_term env v | None -> ())
-              proto
+              (function
+                | { Term.default = Some v } -> iter_term env v | _ -> ())
+              arguments
     in
     Term.Methods.iter
       (fun _ meth_term -> iter_term env meth_term)
       v.Term.methods;
     iter_base_term env v
   and iter_value v =
-    if not (Alive_values_map.mem itered_values v) then (
-      (* We need to avoid checking the same value multiple times, otherwise we
-         get an exponential blowup, see #1247. *)
-      Alive_values_map.add itered_values v;
-      Value.Methods.iter (fun _ v -> iter_value v) v.Value.methods;
-      match v.value with
+    if not (has_value_flag v Flags.itered_value) then (
+      add_value_flag v Flags.itered_value;
+      Value.Methods.iter (fun _ v -> iter_value v) (Value.methods v);
+      match v with
         | _ when Source_val.is_value v -> f (Source_val.of_value v)
-        | Ground _ -> ()
-        | List l -> List.iter iter_value l
-        | Tuple l -> List.iter iter_value l
-        | Null -> ()
-        | Fun (proto, env, body) ->
+        | Value.Int _ | Value.String _ | Value.Float _ | Value.Bool _
+        | Value.Custom _ ->
+            ()
+        | Value.List { value = l } -> List.iter iter_value l
+        | Value.Tuple { value = l } -> List.iter iter_value l
+        | Value.Null _ -> ()
+        | Value.Fun { fun_args = proto; fun_env = env; fun_body = body } ->
             (* The following is necessarily imprecise: we might see sources that
                will be unused in the execution of the function. *)
             iter_term env body;
             List.iter (function _, _, Some v -> iter_value v | _ -> ()) proto
-        | FFI (proto, _) ->
+        | Value.FFI { ffi_args = proto; _ } ->
             on_imprecise ();
-            List.iter (function _, _, Some v -> iter_value v | _ -> ()) proto
-      (*
-        | Ref r ->
-            if List.memq r !static_analysis_failed then ()
-            else (
-              (* Do not walk inside references, otherwise the list of "contained"
-                 sources may change from one time to the next, which makes it
-                 impossible to avoid ill-balanced activations. Not walking inside
-                 references does not break things more than they are already:
-                 detecting sharing in presence of references to sources cannot be
-                 done statically anyway. We display a fat log message to warn
-                 about this risky situation. *)
-              let may_have_source =
-                let rec aux v =
-                  match v.value with
-                    | _ when Source_val.is_value v -> true
-                    | Ground _ | Null -> false
-                    | List l -> List.exists aux l
-                    | Tuple l -> List.exists aux l
-                    | Ref r -> aux (Atomic.get r)
-                    | Fun _ | FFI _ -> true
-                    | Meth (_, v, t) -> aux v || aux t
-                in
-                aux v
-              in
-              static_analysis_failed := r :: !static_analysis_failed;
-              if may_have_source then (
-                match on_reference with
-                  | Some f -> f ()
-                  | None ->
-                      log#severe
-                        "WARNING! Found a reference, potentially containing \
-                         sources, inside a dynamic source-producing function. \
-                         Static analysis cannot be performed: make sure you \
-                         are not sharing sources contained in references!"))
-               *))
+            List.iter (function _, _, Some v -> iter_value v | _ -> ()) proto)
   in
   iter_value v
 
