@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ class defer ~delay ~overhead ~field source =
   let delay = Frame.main_of_seconds delay in
   object (self)
     inherit Source.operator ~name:"defer" [source]
-    method stype = `Fallible
+    method fallible = true
     method remaining = source#remaining
     method abort_track = source#abort_track
     method seek_source = source#seek_source
@@ -55,7 +55,6 @@ class defer ~delay ~overhead ~field source =
     val mutable deferred = true
     val mutable generator = None
     val mutable data = None
-    val mutable tmp_frame = None
 
     method private data =
       match data with
@@ -79,26 +78,16 @@ class defer ~delay ~overhead ~field source =
             generator <- Some gen;
             gen
 
-    method private tmp_frame =
-      match tmp_frame with
-        | Some tmp_frame -> tmp_frame
-        | None ->
-            let frame = Frame.create source#content_type in
-            tmp_frame <- Some frame;
-            frame
-
     method private buffer_data =
       let { offset; position } = state in
       let data = self#data.(position) in
       let chunk_len = Content_pcm_base.length data in
       let data_rem = chunk_len - offset in
 
-      let tmp_frame = self#tmp_frame in
-      Frame.clear tmp_frame;
+      let tmp_frame =
+        if source#is_ready then source#get_frame else self#empty_frame
+      in
 
-      while Frame.is_partial tmp_frame && source#is_ready ~frame:tmp_frame () do
-        source#get tmp_frame
-      done;
       let gen_len = Frame.position tmp_frame in
 
       let gen = self#generator in
@@ -113,7 +102,7 @@ class defer ~delay ~overhead ~field source =
         (fun pos ->
           if pos < gen_len then
             Generator.add_track_mark ~pos:(pos + buffered) gen)
-        (Frame.breaks tmp_frame);
+        (Frame.track_marks tmp_frame);
 
       let frame_content = Frame.get tmp_frame field in
       let frame_pcm = Content_pcm_s16.get_data frame_content in
@@ -135,11 +124,9 @@ class defer ~delay ~overhead ~field source =
     val mutable should_queue = false
 
     method private queue_output =
-      let clock = Source.Clock_variables.get self#clock in
-      clock#on_output (fun () ->
-          if source#is_ready () then self#buffer_data;
-          if should_queue then
-            clock#on_before_output (fun () -> self#queue_output))
+      Clock.on_tick self#clock (fun () ->
+          if source#is_ready then self#buffer_data;
+          if should_queue then self#queue_output)
 
     initializer
       self#on_wake_up (fun () ->
@@ -147,10 +134,11 @@ class defer ~delay ~overhead ~field source =
           self#queue_output);
       self#on_sleep (fun () -> should_queue <- false)
 
-    method private _is_ready ?frame:_ _ =
+    method private can_generate_frame =
       (not deferred) && Generator.length self#generator > 0
 
-    method private get_frame buf = Generator.fill self#generator buf
+    method private generate_frame =
+      Generator.slice self#generator (Lazy.force Frame.size)
   end
 
 let _ =
