@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -62,8 +62,9 @@ class iir (source : source) filter_family filter_type order freq1 freq2 qfactor
     (* I/O shift registries *)
     val mutable xv = [||]
     val mutable yv = [||]
+    initializer self#on_wake_up (fun () -> self#initialize)
 
-    initializer
+    method private initialize =
       self#log#info "Initializing...";
       self#log#info "Alpha 1: %.013f (warped: %.013f)" raw_alpha1 warped_alpha1;
       self#log#info "Alpha 2: %.013f (warped: %.013f)" raw_alpha2 warped_alpha2;
@@ -411,18 +412,17 @@ class iir (source : source) filter_family filter_type order freq1 freq2 qfactor
       self#log#info "Initialization done."
 
     (* Digital filter based on mkfilter/mkshape/gencode by A.J. Fisher *)
-    method stype = source#stype
+    method fallible = source#fallible
     method remaining = source#remaining
     method seek_source = source#seek_source
     method self_sync = source#self_sync
-    method private _is_ready = source#is_ready
+    method private can_generate_frame = source#is_ready
     method abort_track = source#abort_track
     val mutable v_offs = 0
 
-    method private get_frame buf =
-      let offset = AFrame.position buf in
-      source#get buf;
-      let b = AFrame.pcm buf in
+    method private generate_frame =
+      let c = source#get_mutable_content Frame.Fields.audio in
+      let b = Content.Audio.get_data c in
       let v_len = Array.length xv.(0) in
       let coeffs_len = Array.length xcoeffs in
       let fold_left2 init v coeffs l =
@@ -437,7 +437,7 @@ class iir (source : source) filter_family filter_type order freq1 freq2 qfactor
         let xvc = xv.(c) in
         let yvc = yv.(c) in
         let bc = b.(c) in
-        for i = offset to AFrame.position buf - 1 do
+        for i = 0 to source#frame_audio_position - 1 do
           v_offs <- (v_offs + 1) mod v_len;
           xvc.((coeffs_len - 1 + v_offs) mod v_len) <- bc.(i) /. gain;
           let insert =
@@ -447,7 +447,8 @@ class iir (source : source) filter_family filter_type order freq1 freq2 qfactor
           yvc.((coeffs_len - 1 + v_offs) mod v_len) <- insert;
           bc.(i) <- insert
         done
-      done
+      done;
+      source#set_frame_data Frame.Fields.audio Content.Audio.lift_data b
   end
 
 let filter_iir = Lang.add_module ~base:Filter.filter "iir"
