@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 open Mm
 open Source
 open Lilv
+module Cache = Liquidsoap_lang.Cache
 
 let lv2 = Lang.add_module "lv2"
 let log = Log.make ["Lilv LV2"]
@@ -36,10 +37,10 @@ let lilv_enabled =
 class virtual base source =
   object
     inherit operator ~name:"lilv" [source]
-    method stype = source#stype
+    method fallible = source#fallible
     method remaining = source#remaining
     method seek_source = source#seek_source
-    method private _is_ready = source#is_ready
+    method private can_generate_frame = source#is_ready
     method self_sync = source#self_sync
     method abort_track = source#abort_track
   end
@@ -48,8 +49,8 @@ class virtual base_nosource =
   object (self)
     inherit source ~name:"lilv" ()
     method seek_source = (self :> Source.source)
-    method stype = `Infallible
-    method private _is_ready ?frame:_ _ = true
+    method fallible = false
+    method private can_generate_frame = true
     val mutable must_fail = false
     method abort_track = must_fail <- true
     method remaining = -1
@@ -63,32 +64,31 @@ let constant_data len x =
 (** A mono LV2 plugin: a plugin is created for each channel. *)
 class lilv_mono (source : source) plugin input output params =
   object (self)
-    inherit base source as super
+    inherit base source
     val mutable inst = None
 
-    method! wake_up a =
-      super#wake_up a;
-      let i =
-        Array.init
-          (Content.Audio.channels_of_format
-             (Option.get
-                (Frame.Fields.find_opt Frame.Fields.audio self#content_type)))
-          (fun _ ->
-            Plugin.instantiate plugin
-              (float_of_int (Lazy.force Frame.audio_rate)))
-      in
-      Array.iter Plugin.Instance.activate i;
-      inst <- Some i
+    initializer
+      self#on_wake_up (fun () ->
+          let i =
+            Array.init
+              (Content.Audio.channels_of_format
+                 (Option.get
+                    (Frame.Fields.find_opt Frame.Fields.audio self#content_type)))
+              (fun _ ->
+                Plugin.instantiate plugin
+                  (float_of_int (Lazy.force Frame.audio_rate)))
+          in
+          Array.iter Plugin.Instance.activate i;
+          inst <- Some i)
 
-    method private get_frame buf =
-      let offset = AFrame.position buf in
-      source#get buf;
-      let b = AFrame.pcm buf in
+    method private generate_frame =
+      let b =
+        Content.Audio.get_data (source#get_mutable_content Frame.Fields.audio)
+      in
+      let len = source#frame_audio_position in
       let chans = Array.length b in
-      let position = AFrame.position buf in
-      let len = position - offset in
       let inst = Option.get inst in
-      let ba = Audio.to_ba b offset len in
+      let ba = Audio.to_ba b 0 len in
       for c = 0 to chans - 1 do
         Plugin.Instance.connect_port_float inst.(c) input ba.(c);
         Plugin.Instance.connect_port_float inst.(c) output ba.(c);
@@ -98,8 +98,9 @@ class lilv_mono (source : source) plugin input output params =
               (constant_data len (v ())))
           params;
         Plugin.Instance.run inst.(c) len;
-        Audio.copy_from_ba ba b offset len
-      done
+        Audio.copy_from_ba ba b 0 len
+      done;
+      source#set_frame_data Frame.Fields.audio Content.Audio.lift_data b
   end
 
 class lilv (source : source) plugin inputs outputs params =
@@ -111,12 +112,11 @@ class lilv (source : source) plugin inputs outputs params =
 
     initializer Plugin.Instance.activate inst
 
-    method private get_frame buf =
-      let offset = AFrame.position buf in
-      source#get buf;
-      let b = AFrame.pcm buf in
-      let position = AFrame.position buf in
-      let len = position - offset in
+    method private generate_frame =
+      let b =
+        Content.Audio.get_data (source#get_mutable_content Frame.Fields.audio)
+      in
+      let len = source#frame_audio_position in
       List.iter
         (fun (p, v) ->
           let data =
@@ -125,7 +125,7 @@ class lilv (source : source) plugin inputs outputs params =
           Bigarray.Array1.fill data (v ());
           Plugin.Instance.connect_port_float inst p data)
         params;
-      let ba = Audio.to_ba b offset len in
+      let ba = Audio.to_ba b 0 len in
       if Array.length inputs = Array.length outputs then (
         let chans = Array.length b in
         (* The simple case: number of channels does not get changed. *)
@@ -134,54 +134,55 @@ class lilv (source : source) plugin inputs outputs params =
           Plugin.Instance.connect_port_float inst outputs.(c) ba.(c)
         done;
         Plugin.Instance.run inst len;
-        Audio.copy_from_ba ba b offset len)
+        Audio.copy_from_ba ba b 0 len)
       else (
         (* We have to change channels. *)
-        let d = AFrame.pcm buf in
-        let dba = Audio.to_ba d offset len in
+        let dba = Audio.to_ba b 0 len in
         for c = 0 to Array.length b - 1 do
           Plugin.Instance.connect_port_float inst inputs.(c) ba.(c)
         done;
-        let output_chans = Array.length d in
+        let output_chans = Array.length b in
         for c = 0 to output_chans - 1 do
           Plugin.Instance.connect_port_float inst outputs.(c) dba.(c)
         done;
         Plugin.Instance.run inst len;
-        Audio.copy_from_ba dba d offset len)
+        Audio.copy_from_ba dba b 0 len);
+      source#set_frame_data Frame.Fields.audio Content.Audio.lift_data b
   end
 
 (** An LV2 plugin without audio input. *)
 class lilv_nosource plugin outputs params =
-  object
+  object (self)
     inherit base_nosource
-    method self_sync = (`Static, false)
+    method self_sync = (`Static, None)
 
     val inst =
       Plugin.instantiate plugin (float_of_int (Lazy.force Frame.audio_rate))
 
     initializer Plugin.Instance.activate inst
 
-    method private get_frame buf =
+    method private generate_frame =
       if must_fail then (
-        AFrame.add_break buf (AFrame.position buf);
-        must_fail <- false)
+        must_fail <- false;
+        self#end_of_track)
       else (
-        let offset = AFrame.position buf in
-        let b = AFrame.pcm buf in
+        let length = Lazy.force Frame.size in
+        let buf = Frame.create ~length self#content_type in
+        let b = Content.Audio.get_data (Frame.get buf Frame.Fields.audio) in
         let chans = Array.length b in
-        let position = AFrame.size () in
-        let len = position - offset in
-        let ba = Audio.to_ba b offset len in
+        let alen = Frame.audio_of_main length in
+        let ba = Audio.to_ba b 0 alen in
         List.iter
           (fun (p, v) ->
-            Plugin.Instance.connect_port_float inst p (constant_data len (v ())))
+            Plugin.Instance.connect_port_float inst p
+              (constant_data alen (v ())))
           params;
         for c = 0 to chans - 1 do
           Plugin.Instance.connect_port_float inst outputs.(c) ba.(c)
         done;
-        Plugin.Instance.run inst len;
-        Audio.copy_from_ba ba b offset len;
-        AFrame.add_break buf position)
+        Plugin.Instance.run inst alen;
+        Audio.copy_from_ba ba b 0 alen;
+        Frame.set_data buf Frame.Fields.audio Content.Audio.lift_data b)
   end
 
 (** An LV2 plugin without audio output (e.g. to observe the stream). The input
@@ -195,21 +196,21 @@ class lilv_noout source plugin inputs params =
 
     initializer Plugin.Instance.activate inst
 
-    method private get_frame buf =
-      let offset = AFrame.position buf in
-      let b = AFrame.pcm buf in
+    method private generate_frame =
+      let buf = source#get_frame in
+      let b = Content.Audio.get_data (Frame.get buf Frame.Fields.audio) in
       let chans = Array.length b in
-      let position = AFrame.size () in
-      let len = position - offset in
-      let ba = Audio.to_ba b offset len in
+      let alen = source#frame_audio_position in
+      let ba = Audio.to_ba b 0 alen in
       List.iter
         (fun (p, v) ->
-          Plugin.Instance.connect_port_float inst p (constant_data len (v ())))
+          Plugin.Instance.connect_port_float inst p (constant_data alen (v ())))
         params;
       for c = 0 to chans - 1 do
         Plugin.Instance.connect_port_float inst inputs.(c) ba.(c)
       done;
-      Plugin.Instance.run inst len
+      Plugin.Instance.run inst alen;
+      buf
   end
 
 (* List the indexes of control ports. *)
@@ -222,61 +223,40 @@ let get_control_ports p =
   done;
   List.rev !ans
 
-(* TODO: handle types *)
-let port_type _ = `Float
+type port_type = Float
 
-(* Make a parameter for each control port. Returns the liquidsoap parameters
-   and the parameters for the plugin. *)
-let params_of_plugin plugin =
+(* TODO: handle types *)
+let port_type _ = Float
+
+type port = {
+  port_index : int;
+  port_symbol : string;
+  port_name : string;
+  port_type : port_type;
+  port_default_float : float option;
+  port_min_float : float option;
+  port_max_float : float option;
+}
+
+let get_control_ports plugin =
   let control_ports = get_control_ports plugin in
-  let liq_params =
-    List.map
-      (fun p ->
-        let p = Plugin.port_by_index plugin p in
-        let t = port_type p in
-        ( Port.symbol p,
-          (match t with `Float -> Lang.getter_t Lang.float_t),
-          (match Port.default_float p with
-            | Some f -> Some (match t with `Float -> Lang.float f)
-            | None -> None),
-          let bounds =
-            let min = Port.min_float p in
-            let max = Port.max_float p in
-            if (min, max) = (None, None) then ""
-            else (
-              let bounds = ref " (" in
-              begin
-                match min with
-                  | Some f -> (
-                      match t with
-                        | `Float ->
-                            bounds := Printf.sprintf "%s%.6g <= " !bounds f)
-                  | None -> ()
-              end;
-              bounds := !bounds ^ "`" ^ Port.symbol p ^ "`";
-              begin
-                match max with
-                  | Some f -> (
-                      match t with
-                        | `Float ->
-                            bounds := Printf.sprintf "%s <= %.6g" !bounds f)
-                  | None -> ()
-              end;
-              !bounds ^ ")")
-          in
-          Some (Port.name p ^ bounds ^ ".") ))
-      control_ports
+  (* TODO: handle other types *)
+  let control_ports =
+    List.filter (fun p -> port_type p = Float) control_ports
   in
-  let params p =
-    let f v = List.assoc v p in
-    List.map
-      (fun p ->
-        ( p,
-          let v = f (Port.symbol (Plugin.port_by_index plugin p)) in
-          match port_type p with `Float -> Lang.to_float_getter v ))
-      control_ports
-  in
-  (liq_params, params)
+  List.map
+    (fun i ->
+      let p = Plugin.port_by_index plugin i in
+      {
+        port_index = i;
+        port_symbol = Port.symbol p;
+        port_name = Port.name p;
+        port_type = port_type p;
+        port_default_float = Port.default_float p;
+        port_min_float = Port.min_float p;
+        port_max_float = Port.max_float p;
+      })
+    control_ports
 
 (** Get input and output ports. *)
 let get_audio_ports p =
@@ -290,76 +270,155 @@ let get_audio_ports p =
   done;
   (Array.of_list (List.rev !i), Array.of_list (List.rev !o))
 
-let register_plugin plugin =
-  let inputs, outputs = get_audio_ports plugin in
-  let ni = Array.length inputs in
-  let no = Array.length outputs in
-  (* Ensure that we support the number of channels. *)
-  ignore (Audio_converter.Channel_layout.layout_of_channels ni);
-  ignore (Audio_converter.Channel_layout.layout_of_channels no);
-  let liq_params, params = params_of_plugin plugin in
-  let mono = ni = 1 && no = 1 in
-  let input_t =
-    Lang.frame_t Lang.unit_t
-      (Frame.Fields.make ~audio:(Format_type.audio_n ni) ())
-  in
-  let liq_params =
-    liq_params
-    @ if ni = 0 then [] else [("", Lang.source_t input_t, None, None)]
-  in
+type plugin = {
+  plugin_uri : string;
+  plugin_name : string;
+  plugin_inputs : int array;
+  plugin_outputs : int array;
+  plugin_controls : port list;  (** control ports *)
+  plugin_maker : string;
+  plugin_class_label : string;
+}
+
+let load_plugin plugin =
+  let plugin_inputs, plugin_outputs = get_audio_ports plugin in
+  let plugin_controls = get_control_ports plugin in
   let maker = Plugin.author_name plugin in
   let maker_homepage = Plugin.author_homepage plugin in
   let maker =
     if maker_homepage = "" then maker
     else Printf.sprintf "[%s](%s)" maker maker_homepage
   in
-  let maker = if maker = "" then "" else " by " ^ maker in
-  let descr = Plugin.name plugin ^ maker ^ "." in
-  let descr =
-    descr ^ " This is in class "
-    ^ Plugin.Class.label (Plugin.plugin_class plugin)
-    ^ "."
+  let plugin_maker = if maker = "" then "" else " by " ^ maker in
+  let plugin_class_label = Plugin.Class.label (Plugin.plugin_class plugin) in
+  {
+    plugin_uri = Plugin.uri plugin;
+    plugin_name = Plugin.name plugin;
+    plugin_inputs;
+    plugin_outputs;
+    plugin_controls;
+    plugin_maker;
+    plugin_class_label;
+  }
+
+(* Make a parameter for each control port. Returns the liquidsoap parameters
+   and the parameters for the plugin. *)
+let params_of_controls control_ports =
+  let liq_params =
+    List.map
+      (fun p ->
+        let t = p.port_type in
+        ( p.port_symbol,
+          (match t with Float -> Lang.getter_t Lang.float_t),
+          (match p.port_default_float with
+            | Some f -> Some (match t with Float -> Lang.float f)
+            | None -> None),
+          let bounds =
+            let min = p.port_min_float in
+            let max = p.port_max_float in
+            if (min, max) = (None, None) then ""
+            else (
+              let bounds = ref " (" in
+              begin
+                match min with
+                  | Some f -> (
+                      match t with
+                        | Float ->
+                            bounds := Printf.sprintf "%s%.6g <= " !bounds f)
+                  | None -> ()
+              end;
+              bounds := !bounds ^ "`" ^ p.port_symbol ^ "`";
+              begin
+                match max with
+                  | Some f -> (
+                      match t with
+                        | Float ->
+                            bounds := Printf.sprintf "%s <= %.6g" !bounds f)
+                  | None -> ()
+              end;
+              !bounds ^ ")")
+          in
+          Some (p.port_name ^ bounds ^ ".") ))
+      control_ports
   in
-  let descr = descr ^ " See <" ^ Plugin.uri plugin ^ ">." in
+  let params l =
+    List.map
+      (fun p ->
+        ( p.port_index,
+          let v = List.assoc p.port_symbol l in
+          match port_type p with Float -> Lang.to_float_getter v ))
+      control_ports
+  in
+  (liq_params, params)
+
+let register_plugin plugin p =
+  let ni = Array.length p.plugin_inputs in
+  let no = Array.length p.plugin_outputs in
+  (* Ensure that we support the number of channels. *)
+  ignore (Audio_converter.Channel_layout.layout_of_channels ni);
+  ignore (Audio_converter.Channel_layout.layout_of_channels no);
+  let mono = ni = 1 && no = 1 in
+  let input_t =
+    Lang.frame_t Lang.unit_t
+      (Frame.Fields.make ~audio:(Format_type.audio_n ni) ())
+  in
+  let liq_params, params = params_of_controls p.plugin_controls in
+  let liq_params =
+    liq_params
+    @ if ni = 0 then [] else [("", Lang.source_t input_t, None, None)]
+  in
+
+  let descr = p.plugin_name ^ p.plugin_maker ^ "." in
+  let descr = descr ^ " This is in class " ^ p.plugin_class_label ^ "." in
+  let descr = descr ^ " See <" ^ p.plugin_uri ^ ">." in
   let return_t =
     Lang.frame_t Lang.unit_t
       (Frame.Fields.make ~audio:(Format_type.audio_n no) ())
   in
   ignore
     (Lang.add_operator ~base:lv2
-       (Utils.normalize_parameter_string (Plugin.name plugin))
-       liq_params ~return_t ~category:`Audio ~flags:[`Extra] ~descr
-       (fun p ->
-         let f v = List.assoc v p in
+       (Utils.normalize_parameter_string p.plugin_name)
+       liq_params ~return_t ~category:`Audio ~flags:[`Extra] ~descr (fun l ->
+         let f v = List.assoc v l in
          let source =
            try Some (Lang.to_source (f "")) with Not_found -> None
          in
-         let params = params p in
-         if ni = 0 then new lilv_nosource plugin outputs params
+         let params = params l in
+         if ni = 0 then new lilv_nosource plugin p.plugin_outputs params
          else if no = 0 then
            (* TODO: can we really use such a type? *)
-           (new lilv_noout (Option.get source) plugin inputs params
+           (new lilv_noout (Option.get source) plugin p.plugin_inputs params
              :> Source.source)
          else if mono then
            (new lilv_mono
-              (Option.get source) plugin inputs.(0) outputs.(0) params
+              (Option.get source) plugin p.plugin_inputs.(0)
+              p.plugin_outputs.(0) params
              :> Source.source)
          else
-           (new lilv (Option.get source) plugin inputs outputs params
+           (new lilv
+              (Option.get source) plugin p.plugin_inputs p.plugin_outputs params
              :> Source.source)))
 
-let register_plugin plugin =
-  try register_plugin plugin
+let register_plugin cache plugin =
+  (* Only the uri computation is fast. Try to retrieve other parameters from the cache. *)
+  let uri = Plugin.uri plugin in
+  let p = Cache.Table.get cache uri (fun () -> load_plugin plugin) in
+  try register_plugin plugin p
   with Audio_converter.Channel_layout.Unsupported ->
     log#info "Could not register Lilv plugin %s: unhandled number of channels."
-      (Plugin.name plugin)
+      p.plugin_name
 
 let register_plugins () =
+  let cache =
+    (Cache.Table.load ~dirtype:`System ~name:"lilv plugins" "lilv-plugins"
+      : plugin Cache.Table.t)
+  in
   let world = World.create () in
   World.load_all world;
-  Plugins.iter register_plugin (World.plugins world)
+  Plugins.iter (register_plugin cache) (World.plugins world);
+  Cache.Table.store ~dirtype:`System cache
 
 let () =
   Lifecycle.on_load ~name:"lilv plugin registration" (fun () ->
-      if lilv_enabled then
+      if !Startup.register_external_plugins && lilv_enabled then
         Startup.time "Lilv plugins registration" register_plugins)
