@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable stream generator.
-  Copyright 2003-2023 Savonet team
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,106 +28,101 @@ open Source
   * while looping on the last source -- this behavior would not be suited
   * to the current use of [sequence] in transitions. *)
 class sequence ?(merge = false) ?(single_track = true) sources =
-  let self_sync_type = Utils.self_sync_type sources in
+  let self_sync_type = Clock_base.self_sync_type sources in
+  let seq_sources = Atomic.make sources in
   object (self)
     inherit operator ~name:"sequence" sources
-    val mutable seq_sources = sources
+
+    inherit
+      generate_from_multiple_sources
+        ~merge:(fun () -> merge && List.length (Atomic.get seq_sources) <> 1)
+        ~track_sensitive:(fun () -> true)
+        ()
 
     method self_sync =
       ( Lazy.force self_sync_type,
-        match sources with hd :: _ -> snd hd#self_sync | [] -> false )
+        match sources with hd :: _ -> snd hd#self_sync | [] -> None )
 
-    method stype =
-      match List.rev sources with hd :: _ -> hd#stype | [] -> `Fallible
+    method fallible =
+      match List.rev sources with hd :: _ -> hd#fallible | [] -> true
 
-    method! private wake_up activation =
-      List.iter
-        (fun s -> (s :> source)#get_ready ((self :> source) :: activation))
-        sources
+    (* We have to wait until at least one source is ready. *)
+    val mutable has_started = false
+    method queue = Atomic.get seq_sources
 
-    method! private sleep =
-      List.iter (fun s -> (s :> source)#leave (self :> source)) sources
+    method private has_started =
+      match has_started with
+        | true -> true
+        | false ->
+            has_started <- List.exists (fun s -> s#is_ready) self#queue;
+            has_started
 
-    (** When head_ready is true, it must be that:
-    *  - (List.hd seq_sources)#is_ready
-    *  - or we have started playing a track of (List.hd sources)
-    *    and that track has not ended yet.
-    * In case the head source becomes unavailable before its end of track,
-    * head_ready keeps the sequence operator available, so that its #get_frame
-    * can be called to properly end the track and cleanup the source if needed.
-    * If instead the operator had become unavailable then source#get would have
-    * inserted an end of track automatically instead of calling #get_frame. *)
-    val mutable head_ready = false
+    method private get_stateful_source ?(source_skipped = false) ~reselect () =
+      match (self#has_started, self#queue) with
+        | _, [] -> None
+        | true, s :: [] ->
+            if
+              self#can_reselect
+                ~reselect:(match reselect with `Force -> `Ok | _ -> reselect)
+                s
+            then Some s
+            else None
+        | true, s :: rest ->
+            if
+              self#can_reselect
+                ~reselect:
+                  (match reselect with
+                    | `After_position _
+                      when (not source_skipped) && single_track ->
+                        `Force
+                    | v -> v)
+                s
+            then Some s
+            else (
+              self#log#info "Finished with %s" s#id;
+              Atomic.set seq_sources rest;
+              self#get_stateful_source ~source_skipped:true
+                ~reselect:(match reselect with `Force -> `Ok | v -> v)
+                ())
+        | _ -> None
 
-    method private _is_ready ?frame () =
-      head_ready || List.exists (fun s -> s#is_ready ?frame ()) seq_sources
+    method private get_source ~reselect () =
+      self#get_stateful_source ~reselect ()
 
     method remaining =
       if merge then (
         let ( + ) a b = if a < 0 || b < 0 then -1 else a + b in
-        List.fold_left ( + ) 0 (List.map (fun s -> s#remaining) seq_sources))
-      else (List.hd seq_sources)#remaining
+        List.fold_left ( + ) 0 (List.map (fun s -> s#remaining) self#queue))
+      else (List.hd self#queue)#remaining
 
     method seek_source =
-      match seq_sources with
+      match self#queue with
         | s :: _ -> s#seek_source
         | _ -> (self :> Source.source)
 
     method abort_track =
-      if single_track && merge then (
-        match List.rev seq_sources with
+      if merge then (
+        match List.rev self#queue with
           | [] -> assert false
-          | hd :: _ -> seq_sources <- [hd]);
-      match seq_sources with hd :: _ -> hd#abort_track | _ -> ()
-
-    method private get_frame buf =
-      if head_ready then (
-        let hd = List.hd seq_sources in
-        hd#get buf;
-        if Frame.is_partial buf then (
-          head_ready <- false;
-          if List.length seq_sources > 1 then (
-            if single_track || not (hd#is_ready ~frame:buf ()) then
-              seq_sources <- List.tl seq_sources;
-            if ((not single_track) || merge) && self#is_ready ~frame:buf () then (
-              let pos = Frame.position buf in
-              self#get_frame buf;
-              Frame.set_breaks buf
-                (Utils.remove_one (( = ) pos) (Frame.breaks buf))))))
-      else (
-        match seq_sources with
-          | a :: (_ :: _ as tl) ->
-              if a#is_ready ~frame:buf () then head_ready <- true
-              else seq_sources <- tl;
-              self#get_frame buf
-          | [a] ->
-              assert (a#is_ready ~frame:buf ());
-
-              (* Our #is_ready ensures that. *)
-              head_ready <- true;
-              self#get_frame buf
-          | [] -> assert false)
+          | hd :: _ -> Atomic.set seq_sources [hd]);
+      match self#queue with hd :: _ -> hd#abort_track | _ -> ()
   end
 
 class merge_tracks source =
-  object (self)
+  object
     inherit operator ~name:"sequence" [source]
-    method stype = source#stype
-    method private _is_ready = source#is_ready
+    method fallible = source#fallible
+    method private can_generate_frame = source#is_ready
     method abort_track = source#abort_track
     method remaining = -1
     method self_sync = source#self_sync
     method seek_source = source#seek_source
 
-    method private get_frame buf =
-      source#get buf;
-      if Frame.is_partial buf && source#is_ready ~frame:buf () then (
-        self#log#info "End of track: merging.";
-        self#get_frame buf;
-        Frame.set_breaks buf
-          (match Frame.breaks buf with
-            | b :: _ :: l -> b :: l
-            | _ -> assert false))
+    method private generate_frame =
+      let buf = source#get_frame in
+      Frame.set buf Frame.Fields.track_marks
+        (Content.make ~length:(Frame.position buf)
+           Content_timed.Track_marks.format)
   end
 
 let _ =
@@ -154,6 +149,15 @@ let _ =
       "Play a sequence of sources. By default, play one track per source, \
        except for the last one which is played as much as available."
     ~return_t:frame_t
+    ~meth:
+      [
+        ( "queue",
+          ([], Lang.fun_t [] (Lang.list_t (Lang.source_t frame_t))),
+          "Return the current sequence of source",
+          fun s ->
+            Lang.val_fun [] (fun _ -> Lang.list (List.map Lang.source s#queue))
+        );
+      ]
     (fun p ->
       new sequence
         ~merge:(Lang.to_bool (List.assoc "merge" p))
