@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 
 let ( let* ) = Duppy.Monad.bind
 
+module Http = Liq_http
+
 (** Output to an harbor server. *)
 module type T = sig
   include Harbor.Transport_t
@@ -193,9 +195,11 @@ type client = {
 
 let add_meta c data =
   let mk_icy_meta meta =
-    let f x = try Some (Hashtbl.find meta x) with _ -> None in
     let meta_info =
-      match (f "artist", f "title") with
+      match
+        ( Frame.Metadata.find_opt "artist" meta,
+          Frame.Metadata.find_opt "title" meta )
+      with
         | Some a, Some t -> Some (Printf.sprintf "%s - %s" a t)
         | Some s, None | None, Some s -> Some s
         | None, None -> None
@@ -230,7 +234,7 @@ let add_meta c data =
   in
   let get_meta () =
     let meta =
-      Tutils.mutexify c.meta.metadata_m
+      Mutex_utils.mutexify c.meta.metadata_m
         (fun () ->
           let meta = c.meta.metadata in
           c.meta.metadata <- None;
@@ -258,7 +262,7 @@ let add_meta c data =
 let rec client_task c =
   let* data =
     Duppy.Monad.Io.exec ~priority:`Maybe_blocking c.handler
-      (Tutils.mutexify c.mutex
+      (Mutex_utils.mutexify c.mutex
          (fun () ->
            let buflen = Strings.Mutable.length c.buffer in
            let data =
@@ -280,13 +284,13 @@ let rec client_task c =
   in
   let* state =
     Duppy.Monad.Io.exec ~priority:`Maybe_blocking c.handler
-      (let ret = Tutils.mutexify c.mutex (fun () -> c.state) () in
+      (let ret = Mutex_utils.mutexify c.mutex (fun () -> c.state) () in
        Duppy.Monad.return ret)
   in
   if state <> Done then client_task c else Duppy.Monad.return ()
 
 let client_task c =
-  Tutils.mutexify c.mutex
+  Mutex_utils.mutexify c.mutex
     (fun () ->
       assert (c.state = Hello);
       c.state <- Sending)
@@ -322,9 +326,9 @@ class output p =
       match encoding with "" -> Charset.utf8 | s -> Charset.of_string s
     in
     let f = Charset.convert ~target:out_enc in
-    let meta = Hashtbl.create (Hashtbl.length m) in
-    Hashtbl.iter (fun a b -> Hashtbl.add meta a (f b)) m;
-    meta
+    Frame.Metadata.fold
+      (fun a b m -> Frame.Metadata.add a (f b) m)
+      Frame.Metadata.empty m
   in
   let timeout = Lang.to_float (List.assoc "timeout" p) in
   let buflen = Lang.to_int (List.assoc "buffer" p) in
@@ -347,7 +351,11 @@ class output p =
   let uri =
     match mount.[0] with '/' -> mount | _ -> Printf.sprintf "%c%s" '/' mount
   in
-  let uri = Lang.Regexp.regexp [%string {|^%{uri}$|}] in
+  let uri =
+    let regexp = [%string {|^%{uri}$|}] in
+    Liquidsoap_lang.Builtins_regexp.
+      { descr = regexp; flags = []; regexp = Re.Pcre.regexp regexp }
+  in
   let autostart = Lang.to_bool (List.assoc "start" p) in
   let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
   let register_telnet = Lang.to_bool (List.assoc "register_telnet" p) in
@@ -409,7 +417,7 @@ class output p =
   object (self)
     (** File descriptor where to dump. *)
     inherit
-      Output.encoded
+      [Strings.t] Output.encoded
         ~output_kind:"output.harbor" ~infallible ~register_telnet ~autostart
           ~export_cover_metadata:false ~on_start ~on_stop ~name:mount source
 
@@ -422,18 +430,16 @@ class output p =
     val mutable chunk_len = 0
     val burst_data = Strings.Mutable.empty ()
     val metadata = { metadata = None; metadata_m = Mutex.create () }
-
-    method encode frame ofs len =
-      (Option.get encoder).Encoder.encode frame ofs len
+    method encode frame = (Option.get encoder).Encoder.encode frame
 
     method insert_metadata m =
-      let m = Export_metadata.to_metadata m in
+      let m = Frame.Metadata.Export.to_metadata m in
       let m = recode m in
-      Tutils.mutexify metadata.metadata_m
+      Mutex_utils.mutexify metadata.metadata_m
         (fun () -> metadata.metadata <- Some m)
         ();
       (Option.get encoder).Encoder.insert_metadata
-        (Export_metadata.metadata ~cover:false m)
+        (Frame.Metadata.Export.from_metadata ~cover:false m)
 
     method add_client ~protocol ~headers ~uri ~query s =
       let ip =
@@ -498,7 +504,7 @@ class output p =
               in
               Utils.log_exception ~log:self#log ~bt msg;
               self#log#info "Client %s disconnected" ip;
-              Tutils.mutexify client.mutex
+              Mutex_utils.mutexify client.mutex
                 (fun () ->
                   client.state <- Done;
                   ignore (Strings.Mutable.flush client.buffer))
@@ -529,10 +535,12 @@ class output p =
       Duppy.Monad.Io.exec ~priority:`Maybe_blocking handler
         (Harbor.relayed reply (fun () ->
              self#log#info "Client %s connected" ip;
-             Tutils.mutexify clients_m (fun () -> Queue.push client clients) ();
-             let h_headers = Hashtbl.create (List.length headers) in
-             List.iter (fun (x, y) -> Hashtbl.add h_headers x y) headers;
-             on_connect ~protocol ~uri ~headers:h_headers ip))
+             Mutex_utils.mutexify clients_m
+               (fun () -> Queue.push client clients)
+               ();
+             on_connect ~protocol ~uri
+               ~headers:(Frame.Metadata.from_list headers)
+               ip))
 
     method send b =
       let slen = Strings.length b in
@@ -550,12 +558,12 @@ class output p =
         (match dump with
           | Some s -> Strings.iter (output_substring s) b
           | None -> ());
-        Tutils.mutexify clients_m
+        Mutex_utils.mutexify clients_m
           (fun () ->
             Queue.iter
               (fun c ->
                 let start =
-                  Tutils.mutexify c.mutex
+                  Mutex_utils.mutexify c.mutex
                     (fun () ->
                       match c.state with
                         | Hello ->
@@ -589,7 +597,7 @@ class output p =
     method start =
       assert (encoder = None);
       let enc = data.factory self#id in
-      encoder <- Some (enc Export_metadata.empty_metadata);
+      encoder <- Some (enc Frame.Metadata.Export.empty);
       let handler ~protocol ~meth:_ ~data:_ ~headers ~query ~socket uri =
         self#add_client ~protocol ~headers ~uri ~query socket
       in
@@ -601,11 +609,11 @@ class output p =
       encoder <- None;
       Harbor.remove_http_handler ~port ~verb:`Get ~uri ();
       let new_clients = Queue.create () in
-      Tutils.mutexify clients_m
+      Mutex_utils.mutexify clients_m
         (fun () ->
           Queue.iter
             (fun c ->
-              Tutils.mutexify c.mutex
+              Mutex_utils.mutexify c.mutex
                 (fun () ->
                   c.state <- Done;
                   Duppy.Monad.run
