@@ -24,6 +24,16 @@
 
 module Http = Liq_http
 
+let conf_icecast =
+  Dtools.Conf.void ~p:(Configure.conf#plug "icecast") "Icecast configuration"
+
+let conf_prefer_address =
+  Dtools.Conf.string
+    ~p:(conf_icecast#plug "prefer_address")
+    ~d:"system"
+    "Set preference for resolving addresses. One of: `\"system\"`, `\"ipv4\"` \
+     or `\"ipv6\"`."
+
 let error_translator = function
   | Cry.Error _ as e -> Some (Cry.string_of_error e)
   | _ -> None
@@ -219,8 +229,9 @@ let proto frame_t =
         Lang.nullable_t Lang.string_t,
         Some Lang.null,
         Some
-          "Preferred address type when resolving hostnames. One of: `\"ipv4\"` \
-           or `\"ipv6\"`. Defaults to system default when `null`." );
+          "Preferred address type when resolving hostnames. One of: \
+           `\"system\"`, `\"ipv4\"` or `\"ipv6\"`. Defaults to \
+           `settings.icecast.prefer_address` when `null`." );
       ( "transport",
         Lang.http_transport_base_t,
         Some (Lang.base_http_transport Http.unix_transport),
@@ -234,6 +245,12 @@ let proto frame_t =
         Some
           "Timeout for establishing network connections (disabled is negative)."
       );
+      ( "send_last_metadata_on_connect",
+        Lang.bool_t,
+        Some (Lang.bool true),
+        Some
+          "Send the source's last metadata when connecting to the remote \
+           icecast server." );
       ( "timeout",
         Lang.float_t,
         Some (Lang.float 30.),
@@ -341,6 +358,9 @@ class output p =
     let msg = Printexc.to_string error in
     Lang.to_float (Lang.apply on_error [("", Lang.string msg)])
   in
+  let send_last_metadata_on_connect =
+    e Lang.to_bool "send_last_metadata_on_connect"
+  in
   let data = encoder_data p in
   let chunked = Lang.to_bool (List.assoc "chunked" p) in
   let protocol =
@@ -426,12 +446,17 @@ class output p =
   let transport = e Lang.to_http_transport "transport" in
   let prefer_address =
     let v = List.assoc "prefer_address" p in
-    match Lang.to_valued_option Lang.to_string v with
-      | None -> `System_default
-      | Some "ipv4" -> `Ipv4
-      | Some "ipv6" -> `Ipv6
-      | Some _ ->
-          raise (Error.Invalid_value (v, "Valid values are: 'ipv4' or 'ipv6'."))
+    match
+      Option.value ~default:conf_prefer_address#get
+        (Lang.to_valued_option Lang.to_string v)
+    with
+      | "system" -> `System_default
+      | "ipv4" -> `Ipv4
+      | "ipv6" -> `Ipv6
+      | _ ->
+          raise
+            (Error.Invalid_value
+               (v, "Valid values are: `\"system\"`, `\"ipv4\"` or `\"ipv6\"`."))
   in
   let transport = (transport :> Cry.transport) in
   let transport =
@@ -603,7 +628,7 @@ class output p =
         try List.assoc "User-Agent" headers
         with Not_found -> Printf.sprintf "liquidsoap %s" Configure.version
       in
-      let source =
+      let handler =
         Cry.connection ~host ~port ~user ~password ?genre ?url ?description
           ~name ~public ~protocol ~mount ~chunked ~audio_info ~user_agent
           ~content_type:data.format ()
@@ -611,12 +636,20 @@ class output p =
       List.iter
         (fun (x, y) ->
           (* User-Agent has already been passed to Cry.. *)
-          if x <> "User-Agent" then Hashtbl.replace source.Cry.headers x y)
+          if x <> "User-Agent" then Hashtbl.replace handler.Cry.headers x y)
         headers;
       try
-        Cry.connect connection source;
+        Cry.connect connection handler;
         self#log#important "Connection setup was successful.";
 
+        (match (Lang.to_source source)#last_metadata with
+          | Some m when send_last_metadata_on_connect -> (
+              try
+                self#insert_metadata
+                  (Frame.Metadata.Export.from_metadata ~cover:false m)
+              with _ -> ())
+          | _ -> ());
+
         (* Execute on_connect hook. *)
         on_connect ()
       with
