@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,10 +20,12 @@
 
  *****************************************************************************)
 
-(** Abstract classes for easy creation of output nodes. *)
+(** Custom classes for easy creation of output nodes. *)
 
 open Source
 
+let fallibility_check = ref true
+
 let proto =
   Start_stop.output_proto
   @ [
@@ -41,28 +43,51 @@ let proto =
 
 let meth = Start_stop.meth ()
 
+module Queue = Liquidsoap_lang.Queues.Queue
+
 (** Given abstract start stop and send methods, creates an output.  Takes care
     of pulling the data out of the source, type checkings, maintains a queue of
     last ten metadata and setups standard Server commands, including
     start/stop. *)
-class virtual output ~output_kind ?(name = "") ~infallible ~register_telnet
-  ~(on_start : unit -> unit) ~(on_stop : unit -> unit) val_source autostart =
+class virtual output ~output_kind ?clock ?(name = "") ~infallible
+  ~register_telnet ~(on_start : unit -> unit) ~(on_stop : unit -> unit)
+  val_source autostart =
   let source = Lang.to_source val_source in
   object (self)
     initializer
       (* This should be done before the active_operator initializer attaches us
          to a clock. *)
-      if infallible && source#stype <> `Infallible then
-        raise (Error.Invalid_value (val_source, "That source is fallible"))
+      if !fallibility_check && infallible && source#fallible then
+        raise (Error.Invalid_value (val_source, "That source is fallible."))
 
     initializer Typing.(source#frame_type <: self#frame_type)
-    inherit active_operator ~name:output_kind [source]
+    inherit active_operator ?clock ~name:output_kind [source]
     inherit Start_stop.base ~on_start ~on_stop as start_stop
     method virtual private start : unit
     method virtual private stop : unit
     method virtual private send_frame : Frame.t -> unit
     method self_sync = source#self_sync
-    method stype = if infallible then `Infallible else `Fallible
+    method fallible = not infallible
+    method! source_type : source_type = `Output (self :> Source.active)
+
+    method private add_on_air m =
+      let d = Unix.gettimeofday () in
+      let m =
+        Frame.Metadata.add "on_air" (Request.pretty_date (Unix.localtime d)) m
+      in
+      Frame.Metadata.add "on_air_timestamp" (Printf.sprintf "%.02f" d) m
+
+    (* Metadata stuff: keep track of what was streamed. *)
+    val q_length = 10
+    val metadata_queue = Queue.create ()
+
+    method private add_metadata m =
+      let m = Frame.Metadata.Export.from_metadata ~cover:false m in
+      Queue.push metadata_queue m;
+      if Queue.length metadata_queue > q_length then
+        ignore (Queue.pop metadata_queue)
+
+    initializer self#on_metadata self#add_metadata
 
     (* Registration of Telnet commands must be delayed because some operators
        change their id at initialization time. *)
@@ -79,20 +104,18 @@ class virtual output ~output_kind ?(name = "") ~infallible ~register_telnet
             "Done")
           ~descr:"Skip current song.";
         Server.add ~ns "metadata" ~descr:"Print current metadata." (fun _ ->
-            let q = self#metadata_queue in
             fst
-              (Queue.fold
-                 (fun (s, i) m ->
+              (Queue.fold metadata_queue
+                 (fun m (s, i) ->
                    let s =
                      s
                      ^ (if s = "" then "--- " else "\n--- ")
                      ^ string_of_int i ^ " ---\n"
-                     ^ Request.string_of_metadata
-                         (Export_metadata.to_metadata m)
+                     ^ Frame.Metadata.to_string
+                         (Frame.Metadata.Export.to_metadata m)
                    in
                    (s, i - 1))
-                 ("", Queue.length q)
-                 q));
+                 ("", Queue.length metadata_queue)));
         Server.add ~ns "remaining" ~descr:"Display estimated remaining time."
           (fun _ ->
             let r = source#remaining in
@@ -108,107 +131,73 @@ class virtual output ~output_kind ?(name = "") ~infallible ~register_telnet
           ["skip"; "metadata"; "remaining"];
       registered_telnet <- false
 
-    method private _is_ready ?frame () =
-      if infallible then (
-        assert (source#is_ready ?frame ());
-        true)
-      else source#is_ready ?frame ()
-
+    method private can_generate_frame = source#is_ready
     method remaining = source#remaining
     method abort_track = source#abort_track
     method seek_source = source#seek_source
 
     (* Operator startup *)
-    method! private wake_up activation =
-      (* We prefer [name] as an ID over the default, but do not overwrite
-         user-defined ID. Our ID will be used for the server interface. *)
-      if name <> "" then self#set_id ~definitive:false name;
+    initializer
+      self#on_wake_up (fun () ->
+          (* We prefer [name] as an ID over the default, but do not overwrite
+             user-defined ID. Our ID will be used for the server interface. *)
+          if name <> "" then self#set_id ~definitive:false name;
 
-      self#log#debug "Clock is %s."
-        (Source.Clock_variables.to_string self#clock);
-      self#log#important "Content type is %s."
-        (Frame.string_of_content_type self#content_type);
+          self#log#debug "Clock is %s." (Clock.id self#clock);
+          self#log#important "Content type is %s."
+            (Frame.string_of_content_type self#content_type);
 
-      if Frame.Fields.is_empty self#content_type then
-        failwith
-          (Printf.sprintf
-             "Empty content-type detected for output %s. You might want to use \
-              an expliciy type annotation!"
-             self#id);
+          if Frame.Fields.is_empty self#content_type then
+            failwith
+              (Printf.sprintf
+                 "Empty content-type detected for output %s. You might want to \
+                  use an expliciy type annotation!"
+                 self#id);
 
-      (* Get our source ready. This can take a while (preparing playlists,
-         etc). *)
-      source#get_ready ((self :> operator) :: activation);
-      if infallible then
-        while not (source#is_ready ()) do
-          self#log#important "Waiting for %S to be ready..." source#id;
-          Thread.delay 1.
-        done;
+          if not autostart then start_stop#transition_to `Stopped;
 
-      if source#stype = `Infallible then
-        start_stop#transition_to (if autostart then `Started else `Stopped);
-
-      self#register_telnet
-
-    method! private sleep =
-      start_stop#transition_to `Stopped;
-      source#leave (self :> operator)
-
-    (* Metadata stuff: keep track of what was streamed. *)
-    val q_length = 10
-    val metadata_q = Queue.create ()
-
-    method private add_metadata m =
-      Queue.add m metadata_q;
-      if Queue.length metadata_q > q_length then ignore (Queue.take metadata_q)
-
-    method private metadata_queue = Queue.copy metadata_q
+          self#register_telnet);
+      self#on_sleep (fun () ->
+          self#cleanup_telnet;
+          start_stop#transition_to `Stopped)
 
     (* The output process *)
     val mutable skip = false
     method private skip = skip <- true
-    method private get_frame buf = if Frame.is_partial buf then source#get buf
 
-    method private output =
-      self#has_ticked;
-      if self#is_ready ~frame:self#memo () && state <> `Stopped then
-        start_stop#transition_to `Started;
+    method private generate_frame =
+      Frame.map_metadata source#get_frame (fun (pos, m) ->
+          Some (pos, self#add_on_air m))
+
+    method output =
+      if self#is_ready && state = `Idle then start_stop#transition_to `Started;
       if start_stop#state = `Started then (
-        (* Complete filling of the frame *)
-        let get_count = ref 0 in
-        while Frame.is_partial self#memo && self#is_ready ~frame:self#memo () do
-          incr get_count;
-          if !get_count > Lazy.force Frame.size then
-            self#log#severe
-              "Warning: there may be an infinite sequence of empty tracks!";
-          source#get self#memo
-        done;
-        List.iter
-          (fun (_, m) ->
-            self#add_metadata (Export_metadata.metadata ~cover:false m))
-          (Frame.get_all_metadata self#memo);
+        let data =
+          if self#is_ready then self#get_frame else self#end_of_track
+        in
 
         (* Output that frame if it has some data *)
-        if Frame.position self#memo > 0 then self#send_frame self#memo;
-        if Frame.is_partial self#memo then (
-          self#log#important "Source failed (no more tracks) stopping output...";
-          self#transition_to `Idle))
+        if Frame.position data > 0 then self#send_frame data;
+        if Frame.is_partial data then (
+          if not self#fallible then (
+            self#log#critical "Infallible source produced a partial frame!";
+            assert false);
+          self#log#info "Source ended (no more tracks) stopping output...";
+          self#transition_to `Idle);
 
-    initializer
-      self#on_after_output (fun () ->
-          (* Perform skip if needed *)
-          if skip then (
-            self#log#important "Performing user-requested skip";
-            skip <- false;
-            self#abort_track))
+        if skip then (
+          self#log#important "Performing user-requested skip";
+          skip <- false;
+          self#abort_track))
   end
 
-class dummy ~infallible ~on_start ~on_stop ~autostart ~register_telnet source =
+class dummy ?clock ~infallible ~on_start ~on_stop ~autostart ~register_telnet
+  source =
   object
     inherit
       output
-        source autostart ~name:"dummy" ~output_kind:"output.dummy" ~infallible
-          ~on_start ~on_stop ~register_telnet
+        source autostart ?clock ~name:"dummy" ~output_kind:"output.dummy"
+          ~infallible ~on_start ~on_stop ~register_telnet
 
     method! private reset = ()
     method private start = ()
@@ -237,28 +226,29 @@ let _ =
 
 (** More concrete abstract-class, which takes care of the #send_frame method for
     outputs based on encoders. *)
-class virtual encoded ~output_kind ~name ~infallible ~on_start ~on_stop
-  ~register_telnet ~autostart ~export_cover_metadata source =
+class virtual ['a] encoded ~output_kind ?clock ~name ~infallible ~on_start
+  ~on_stop ~register_telnet ~autostart ~export_cover_metadata source =
   object (self)
     inherit
       output
-        ~infallible ~on_start ~on_stop ~output_kind ~name ~register_telnet
-          source autostart
+        ~infallible ~on_start ~on_stop ~output_kind ?clock ~name
+          ~register_telnet source autostart
 
-    method virtual private insert_metadata : Export_metadata.metadata -> unit
-    method virtual private encode : Frame.t -> int -> int -> 'a
+    method virtual private insert_metadata : Frame.Metadata.Export.t -> unit
+    method virtual private encode : Frame.t -> 'a
     method virtual private send : 'a -> unit
 
     method private send_frame frame =
       let rec output_chunks frame =
         let f start stop =
-          let data = self#encode frame start (stop - start) in
+          let data = self#encode (Frame.sub frame start (stop - start)) in
           self#send data;
           match Frame.get_metadata frame start with
             | None -> ()
             | Some m ->
                 self#insert_metadata
-                  (Export_metadata.metadata ~cover:export_cover_metadata m)
+                  (Frame.Metadata.Export.from_metadata
+                     ~cover:export_cover_metadata m)
         in
         function
         | [] -> assert false
@@ -270,6 +260,6 @@ class virtual encoded ~output_kind ~name ~infallible ~on_start ~on_stop
       output_chunks frame
         (0
         :: List.sort compare
-             (List.map fst (Frame.get_all_metadata frame) @ Frame.breaks frame)
-        )
+             (List.map fst (Frame.get_all_metadata frame)
+             @ [Frame.position frame]))
   end
