@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,17 +16,20 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 
  *****************************************************************************)
 
-(** Abstract classes for easy creation of output nodes. *)
+(** Custom classes for easy creation of output nodes. *)
+
+val fallibility_check : bool ref
 
 (** Parameters needed to instantiate an output. *)
 val proto : (string * Lang.t * Lang.value option * string option) list
 
 class virtual output :
   output_kind:string
+  -> ?clock:Clock.t
   -> ?name:string
   -> infallible:bool
   -> register_telnet:bool
@@ -36,16 +39,16 @@ class virtual output :
   -> bool
   -> object
        inherit Source.active_source
-       method stype : [ `Fallible | `Infallible ]
-       method self_sync : Source.self_sync
+       method fallible : bool
+       method self_sync : Clock.self_sync
        method remaining : int
-       method output : unit
-       method private get_frame : Frame.t -> unit
        method abort_track : unit
-       method private _is_ready : ?frame:Frame.t -> unit -> bool
+       method private can_generate_frame : bool
+       method private generate_frame : Frame.t
        method state : Start_stop.state
        method transition_to : Start_stop.state -> unit
        method seek_source : Source.source
+       method output : unit
        method private video_dimensions : int * int
        method private reset : unit
        method virtual private send_frame : Frame.t -> unit
@@ -56,8 +59,9 @@ class virtual output :
 (** Default methods on output values. *)
 val meth : (string * Lang.scheme * string * (output -> Lang.value)) list
 
-class virtual encoded :
+class virtual ['a] encoded :
   output_kind:string
+  -> ?clock:Clock.t
   -> name:string
   -> infallible:bool
   -> on_start:(unit -> unit)
@@ -69,8 +73,8 @@ class virtual encoded :
   -> object
        inherit output
        method private send_frame : Frame.t -> unit
-       method virtual private encode : Frame.t -> int -> int -> 'a
-       method virtual private insert_metadata : Export_metadata.metadata -> unit
+       method virtual private encode : Frame.t -> 'a
+       method virtual private insert_metadata : Frame.Metadata.Export.t -> unit
        method virtual private send : 'a -> unit
        method private reset : unit
        method virtual private start : unit
@@ -78,7 +82,8 @@ class virtual encoded :
      end
 
 class dummy :
-  infallible:bool
+  ?clock:Clock.t
+  -> infallible:bool
   -> on_start:(unit -> unit)
   -> on_stop:(unit -> unit)
   -> autostart:bool
