@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@ let encoder_factory ?format format_val =
   let format =
     match format with Some f -> f | None -> Lang.to_format format_val
   in
-  try (Encoder.get_factory format) ~hls:false ~pos:format_val.Value.pos
+  try (Encoder.get_factory format) ~hls:false ~pos:(Value.pos format_val)
   with Not_found ->
     raise (Error.Invalid_value (format_val, "Unsupported encoding format"))
 
@@ -39,7 +39,7 @@ let base_proto =
     Some "Export cover metadata." )
   :: Output.proto
 
-class virtual base ~source ~name p =
+class virtual base ?clock ~source ~name p =
   let e f v = f (List.assoc v p) in
   (* Output settings *)
   let autostart = e Lang.to_bool "start" in
@@ -58,8 +58,8 @@ class virtual base ~source ~name p =
   in
   object (self)
     inherit
-      Output.encoded
-        ~infallible ~register_telnet ~on_start ~on_stop ~autostart
+      [Strings.t] Output.encoded
+        ?clock ~infallible ~register_telnet ~on_start ~on_stop ~autostart
           ~export_cover_metadata ~output_kind:"output.file" ~name source
 
     val mutable encoder = None
@@ -68,14 +68,14 @@ class virtual base ~source ~name p =
     method virtual stop : unit
 
     method virtual private encoder_factory
-        : string -> Export_metadata.metadata -> Encoder.encoder
+        : string -> Frame.Metadata.Export.t -> Encoder.encoder
 
     method create_encoder =
       let enc = self#encoder_factory self#id in
       let meta =
         match current_metadata with
           | Some m -> m
-          | None -> Export_metadata.empty_metadata
+          | None -> Frame.Metadata.Export.empty
       in
       encoder <- Some (enc meta)
 
@@ -91,10 +91,10 @@ class virtual base ~source ~name p =
 
     method! reset = ()
 
-    method encode frame ofs len =
+    method encode frame =
       match encoder with
         | None -> Strings.empty
-        | Some encoder -> encoder.Encoder.encode frame ofs len
+        | Some encoder -> encoder.Encoder.encode frame
 
     method virtual write_pipe : string -> int -> int -> unit
     method send b = Strings.iter self#write_pipe b
@@ -195,21 +195,21 @@ class url_output p =
     method start = self#connect
     method stop = ignore self#close_encoder
 
-    method! encode frame ofs len =
+    method! encode frame =
       try
         match encoder with
           | None when self#can_connect ->
               self#connect;
-              base#encode frame ofs len
+              base#encode frame
           | None -> Strings.empty
-          | Some _ -> base#encode frame ofs len
+          | Some _ -> base#encode frame
       with exn ->
         let bt = Printexc.get_raw_backtrace () in
         self#on_error ~bt exn;
         Strings.empty
 
     method write_pipe _ _ _ = ()
-    method! self_sync = (`Static, self_sync)
+    method! self_sync = (`Static, self#source_sync self_sync)
   end
 
 let _ =
@@ -226,16 +226,13 @@ let _ =
     takes care of the various reload mechanisms. *)
 
 let default_reopen_on_error =
-  Liquidsoap_lang.Runtime.eval ~ignored:false ~ty:(Lang.univ_t ())
-    "fun (_) -> null()"
+  Lang.eval ~cache:false ~stdlib:`Disabled ~typecheck:false "fun (_) -> null()"
 
 let default_reopen_on_metadata =
-  Liquidsoap_lang.Runtime.eval ~ignored:false ~ty:(Lang.univ_t ())
-    "fun (_) -> false"
+  Lang.eval ~cache:false ~stdlib:`Disabled ~typecheck:false "fun (_) -> false"
 
 let default_reopen_when =
-  Liquidsoap_lang.Runtime.eval ~ignored:false ~ty:(Lang.univ_t ())
-    "fun () -> false"
+  Lang.eval ~cache:false ~stdlib:`Disabled ~typecheck:false "fun () -> false"
 
 let pipe_proto frame_t arg_doc =
   base_proto
@@ -292,7 +289,7 @@ let pipe_meth =
           Lang.unit) )
   :: meth
 
-class virtual piped_output ~name p =
+class virtual piped_output ?clock ~name p =
   let source = Lang.assoc "" 3 p in
   let reopen_on_error = List.assoc "reopen_on_error" p in
   let reopen_on_error error =
@@ -315,7 +312,7 @@ class virtual piped_output ~name p =
   let on_reopen = List.assoc "on_reopen" p in
   let on_reopen () = ignore (Lang.apply on_reopen []) in
   object (self)
-    inherit base ~source ~name p as base
+    inherit base ?clock ~source ~name p as base
     val mutable open_date = 0.
     val need_reopen = Atomic.make false
     method need_reopen = Atomic.set need_reopen true
@@ -327,7 +324,8 @@ class virtual piped_output ~name p =
       let current_metadata =
         match current_metadata with
           | Some m ->
-              fun x -> subst (Hashtbl.find (Export_metadata.to_metadata m) x)
+              let m = Frame.Metadata.Export.to_metadata m in
+              fun x -> subst (Frame.Metadata.find x m)
           | None -> fun _ -> raise Not_found
       in
       Utils.interpolate current_metadata s
@@ -372,7 +370,7 @@ class virtual piped_output ~name p =
 
     method! send b = if self#is_open then base#send b
 
-    method! encode frame ofs len =
+    method! encode frame =
       (match self#is_open with
         | false when open_date <= Unix.gettimeofday () -> self#prepare_pipe
         | true when Atomic.get need_reopen ->
@@ -382,10 +380,10 @@ class virtual piped_output ~name p =
                && reopen_when () ->
             self#reopen
         | _ -> ());
-      base#encode frame ofs len
+      base#encode frame
 
     method! insert_metadata metadata =
-      if reopen_on_metadata (Export_metadata.to_metadata metadata) then
+      if reopen_on_metadata (Frame.Metadata.Export.to_metadata metadata) then
         self#reopen;
       base#insert_metadata metadata
   end
@@ -447,7 +445,7 @@ class virtual ['a] file_output_base p =
       let filename = filename () in
       let filename = Lang_string.home_unrelate filename in
       (* Avoid / in metas for filename.. *)
-      let subst m = Pcre.substitute ~pat:"/" ~subst:(fun _ -> "-") m in
+      let subst m = String.concat "-" (String.split_on_char '/' m) in
       self#interpolate ~subst filename
 
     method virtual open_out_gen : open_flag list -> int -> string -> 'a
@@ -482,8 +480,9 @@ class virtual ['a] file_output_base p =
     method close_chan fd =
       try
         self#close_out fd;
-        (try self#on_close (Option.get (Atomic.get current_filename))
-         with _ -> ());
+        (match Atomic.get current_filename with
+          | Some f -> self#on_close f
+          | None -> ());
         Atomic.set current_filename None
       with Sys_error _ as exn ->
         let bt = Printexc.get_raw_backtrace () in
@@ -492,9 +491,9 @@ class virtual ['a] file_output_base p =
     method private on_close = on_close
   end
 
-class file_output ~format_val p =
+class file_output ?clock ~format_val p =
   object
-    inherit piped_output ~name:"output.file" p
+    inherit piped_output ?clock ~name:"output.file" p
     inherit [out_channel] chan_output p
     inherit [out_channel] file_output_base p
     method encoder_factory = encoder_factory format_val
@@ -509,14 +508,14 @@ class file_output ~format_val p =
     method close_out = close_out
   end
 
-class file_output_using_encoder ~format_val p =
+class file_output_using_encoder ?clock ~format_val p =
   let format = Lang.to_format format_val in
   let append = Lang.to_bool (List.assoc "append" p) in
   let on_close = List.assoc "on_close" p in
   let on_close s = Lang.to_unit (Lang.apply on_close [("", Lang.string s)]) in
   let p = ("append", Lang.bool true) :: List.remove_assoc "append" p in
   object (self)
-    inherit piped_output ~name:"output.file" p as base
+    inherit piped_output ?clock ~name:"output.file" p as base
     inherit [unit] chan_output p
     inherit [unit] file_output_base p
 
@@ -575,12 +574,12 @@ let file_proto frame_t =
   ]
   @ chan_proto frame_t "Filename where to output the stream."
 
-let new_file_output p =
+let new_file_output ?clock p =
   let format_val = Lang.assoc "" 1 p in
   let format = Lang.to_format format_val in
   if Encoder.file_output format then
-    (new file_output_using_encoder ~format_val p :> piped_output)
-  else (new file_output ~format_val p :> piped_output)
+    (new file_output_using_encoder ?clock ~format_val p :> piped_output)
+  else (new file_output ?clock ~format_val p :> piped_output)
 
 let output_file =
   let return_t = Lang.univ_t () in
@@ -590,15 +589,15 @@ let output_file =
 
 (** External output *)
 
-class external_output p =
+class external_output ?clock p =
   let format_val = Lang.assoc "" 1 p in
   let process = Lang.to_string_getter (Lang.assoc "" 2 p) in
   let self_sync = Lang.to_bool (List.assoc "self_sync" p) in
   object (self)
-    inherit piped_output ~name:"output.external" p
+    inherit piped_output ?clock ~name:"output.external" p
     inherit [out_channel] chan_output p
     method encoder_factory = encoder_factory format_val
-    method! self_sync = (`Static, self_sync)
+    method! self_sync = (`Static, self#source_sync self_sync)
 
     method open_chan =
       let process = process () in
