@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 
 (** Plug for playlist parsing, in which [src/playlists] plugins come. *)
 
+module Http = Liq_http
+
 let log = Log.make ["playlist parser"]
 
 let conf_playlist =
