@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 
 let ( let* ) = Duppy.Monad.bind
 
+module Http = Liq_http
+
 exception Bind_error of string
 
 let () =
@@ -141,7 +143,7 @@ let prefix_ns cmd ns = to_string (ns @ [cmd])
 let add ~ns ?usage ~descr cmd handler =
   let usage = match usage with None -> cmd | Some u -> u in
   let usage = prefix_ns usage ns in
-  Tutils.mutexify lock
+  Mutex_utils.mutexify lock
     (fun () ->
       let name = prefix_ns cmd ns in
       if Hashtbl.mem commands name then
@@ -154,7 +156,9 @@ let add ~ns ?usage ~descr cmd handler =
 
 (* ... and maybe remove them. *)
 let remove ~ns cmd =
-  Tutils.mutexify lock (fun () -> Hashtbl.remove commands (prefix_ns cmd ns)) ()
+  Mutex_utils.mutexify lock
+    (fun () -> Hashtbl.remove commands (prefix_ns cmd ns))
+    ()
 
 (* That's if you want to have your command wait. *)
 type condition = {
@@ -215,7 +219,7 @@ let read ~after payload = raise (Read { payload; after })
 (* The usage string sums up all the commands... *)
 let usage () =
   let l =
-    Tutils.mutexify lock
+    Mutex_utils.mutexify lock
       (fun () -> Hashtbl.fold (fun k v l -> (k, v) :: l) commands [])
       ()
   in
@@ -233,8 +237,9 @@ let () =
   add "help" ~usage:"help [<command>]"
     ~descr:"Get information on available commands." (fun args ->
       try
-        let args = Pcre.substitute ~pat:"\\s*" ~subst:(fun _ -> "") args in
-        let _, us, d = Tutils.mutexify lock (Hashtbl.find commands) args in
+        let args = String.trim args in
+        let _, us, d = Mutex_utils.mutexify lock (Hashtbl.find commands) args in
+        Printf.printf "Done\n%!";
         Printf.sprintf "Usage: %s\r\n  %s" us d
       with Not_found ->
         (if args <> "" then "No such command: " ^ args ^ "\r\n" else "")
@@ -250,7 +255,7 @@ let exec s =
     with Not_found -> (s, "")
   in
   try
-    let command, _, _ = Tutils.mutexify lock (Hashtbl.find commands) s in
+    let command, _, _ = Mutex_utils.mutexify lock (Hashtbl.find commands) s in
     command args
   with
     | Server_wait opts -> raise (Server_wait opts)
