@@ -36,6 +36,7 @@ type lexer_let_decoration =
   | `Eval
   | `Json_parse
   | `Yaml_parse
+  | `Xml_parse
   | `Sqlite_row
   | `Sqlite_query ]
 
@@ -103,76 +104,6 @@ let attach_comments term =
     !pending_comments;
   pending_comments := []
 
-let mk_source_ty ?pos name args =
-  let fn = !Hooks.mk_source_ty in
-  fn ?pos name args
-
-let mk_clock_ty ?pos () =
-  let fn = !Hooks.mk_clock_ty in
-  fn ?pos ()
-
-let mk_named_ty ?pos = function
-  | "_" -> Type.var ?pos:(Option.map Pos.of_lexing_pos pos) ()
-  | "unit" -> Type.make ?pos:(Option.map Pos.of_lexing_pos pos) Type.unit
-  | "never" -> Type.make ?pos:(Option.map Pos.of_lexing_pos pos) Type.Never
-  | "bool" -> Type.make ?pos:(Option.map Pos.of_lexing_pos pos) Type.Bool
-  | "int" -> Type.make ?pos:(Option.map Pos.of_lexing_pos pos) Type.Int
-  | "float" -> Type.make ?pos:(Option.map Pos.of_lexing_pos pos) Type.Float
-  | "string" -> Type.make ?pos:(Option.map Pos.of_lexing_pos pos) Type.String
-  | "ref" -> Type.reference (Type.var ())
-  | "clock" -> mk_clock_ty ?pos ()
-  | "source" -> mk_source_ty ?pos "source" { extensible = true; tracks = [] }
-  | "source_methods" -> !Hooks.source_methods_t ()
-  | name -> (
-      match Type.find_opt_typ name with
-        | Some c -> c ()
-        | None ->
-            let pos =
-              Option.value ~default:(Lexing.dummy_pos, Lexing.dummy_pos) pos
-            in
-            raise
-              (Term_base.Parse_error
-                 (pos, "Unknown type constructor: " ^ name ^ ".")))
-
-let rec mk_ty ?pos = function
-  | `Named s -> mk_named_ty ?pos s
-  | `Nullable t -> Type.(make (Nullable (mk_ty ?pos t)))
-  | `List t -> Type.(make (List { t = mk_ty ?pos t; json_repr = `Tuple }))
-  | `Json_object t ->
-      Type.(
-        make
-          (List
-             {
-               t = mk_ty ?pos (`Tuple [`Named "string"; t]);
-               json_repr = `Object;
-             }))
-  | `Tuple l -> Type.(make (Tuple (List.map (mk_ty ?pos) l)))
-  | `Arrow (args, t) ->
-      Type.(
-        make
-          (Arrow
-             ( List.map
-                 (fun (optional, name, t) -> (optional, name, mk_ty ?pos t))
-                 args,
-               mk_ty ?pos t )))
-  | `Record l -> List.fold_left (mk_meth_ty ?pos) Type.(make (Tuple [])) l
-  | `Method (t, l) -> List.fold_left (mk_meth_ty ?pos) (mk_ty ?pos t) l
-  | `Invoke (t, s) -> snd (Type.invoke (mk_ty ?pos t) s)
-  | `Source (s, p) -> mk_source_ty ?pos s p
-
-and mk_meth_ty ?pos base { Term.name; optional; typ; json_name } =
-  Type.(
-    make
-      (Meth
-         ( {
-             meth = name;
-             optional;
-             scheme = ([], mk_ty ?pos typ);
-             doc = "";
-             json_name;
-           },
-           base )))
-
 let let_args ~decoration ~pat ?arglist ~def ?cast () =
   { decoration; pat; arglist; def; cast }
 
@@ -185,6 +116,7 @@ type let_opt_el = string * Term.t
 let let_decoration_of_lexer_let_decoration = function
   | `Json_parse -> `Json_parse []
   | `Yaml_parse -> `Yaml_parse
+  | `Xml_parse -> `Xml_parse
   | `Sqlite_query -> `Sqlite_query
   | `Sqlite_row -> `Sqlite_row
   | `Eval -> `Eval
