@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,25 +31,29 @@ let global_evar_names = ref false
 open Type_base
 include R
 
+type t = Type_base.constr R.t
+
 (** Given a position, find the relevant excerpt. *)
-let excerpt (start, stop) =
+let excerpt pos =
+  let { Pos.fname; lstart; lstop; cstart; cstop } = Pos.unpack pos in
   try
-    if start.Lexing.pos_fname <> stop.Lexing.pos_fname then raise Exit;
-    let fname = start.Lexing.pos_fname in
-    let l1 = start.Lexing.pos_lnum in
-    let l2 = stop.Lexing.pos_lnum in
-    let ic = open_in fname in
-    let n = ref 1 in
-    while !n < l1 do
-      ignore (input_line ic);
-      incr n
-    done;
-    let lines = ref [] in
-    while !n <= l2 do
-      lines := input_line ic :: !lines;
-      incr n
-    done;
-    close_in ic;
+    let lines =
+      let ic = open_in fname in
+      Fun.protect
+        ~finally:(fun () -> close_in ic)
+        (fun () ->
+          let n = ref 1 in
+          while !n < lstart do
+            ignore (input_line ic);
+            incr n
+          done;
+          let lines = ref [] in
+          while !n <= lstop do
+            lines := input_line ic :: !lines;
+            incr n
+          done;
+          lines)
+    in
     let lines = Array.of_list (List.rev !lines) in
     let lines =
       let n = Array.length lines in
@@ -64,14 +68,8 @@ let excerpt (start, stop) =
     in
     (* The order is important here because both lines might be the same. *)
     lines.(Array.length lines - 1) <-
-      insert_at (Console.stop_color ())
-        (stop.Lexing.pos_cnum - stop.Lexing.pos_bol)
-        lines.(Array.length lines - 1);
-    lines.(0) <-
-      insert_at
-        (Console.start_color [`red])
-        (start.Lexing.pos_cnum - start.Lexing.pos_bol)
-        lines.(0);
+      insert_at (Console.stop_color ()) cstop lines.(Array.length lines - 1);
+    lines.(0) <- insert_at (Console.start_color [`red]) cstart lines.(0);
     let lines = Array.to_list lines in
     let s = String.concat "\n" lines ^ "\n" in
     Some s
@@ -103,7 +101,7 @@ let evar_global_name =
     with Not_found ->
       incr n;
       let name = String.uppercase_ascii (name !n) in
-      Hashtbl.add evars i name;
+      Hashtbl.replace evars i name;
       name
 
 (** Compute the structure that a term represents, given the list of universally
@@ -157,7 +155,7 @@ let make ?(filter_out = fun _ -> false) ?(generalized = []) t : t =
         try Hashtbl.find evars var.name
         with Not_found ->
           let name = String.uppercase_ascii (name (counter ())) in
-          Hashtbl.add evars var.name name;
+          Hashtbl.replace evars var.name name;
           name
       in
       `EVar (Printf.sprintf "'%s%s" constr_symbols s, Constraints.of_list c))
@@ -166,6 +164,11 @@ let make ?(filter_out = fun _ -> false) ?(generalized = []) t : t =
     if filter_out t then `Ellipsis
     else (
       match t.descr with
+        | Int -> `Constr ("int", [])
+        | Float -> `Constr ("float", [])
+        | String -> `Constr ("string", [])
+        | Bool -> `Constr ("bool", [])
+        | Never -> `Constr ("never", [])
         | Custom c -> c.repr repr g c.typ
         | Getter t -> `Getter (repr g t)
         | List { t; json_repr } -> `List (repr g t, json_repr)
@@ -197,8 +200,7 @@ let make ?(filter_out = fun _ -> false) ?(generalized = []) t : t =
         | Var { contents = Link (`Covariant, t) } when !debug || !debug_variance
           ->
             `Debug ("[>", repr g t, "]")
-        | Var { contents = Link (_, t) } -> repr g t
-        | _ -> raise NotImplemented)
+        | Var { contents = Link (_, t) } -> repr g t)
   in
   repr generalized t
 
@@ -487,7 +489,9 @@ let print_scheme f (generalized, t) =
   if !debug then
     List.iter
       (fun v ->
-        print f (make ~generalized (Type_base.make (Var (ref (Free v)))));
+        print f
+          (make ~generalized
+             (Type_base.make (Var { id = 0; contents = Free v })));
         Format.fprintf f ".")
       generalized;
   print f (make ~generalized t)
