@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,9 +22,11 @@
 
 let messages = Atomic.make []
 
+(* Add a startup message. *)
 let message fmt =
   Printf.ksprintf (fun s -> Atomic.set messages (s :: Atomic.get messages)) fmt
 
+(* Time a startup function. *)
 let time name f =
   let t = Sys.time () in
   let ans = f () in
@@ -32,3 +34,6 @@ let time name f =
   ans
 
 let messages () = Atomic.get messages |> List.rev
+
+(* Should we register external plugins? *)
+let register_external_plugins = ref true
