@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,11 +22,24 @@
 
 include Type_base
 include Ref_type
-module Ground = Ground_type
+module Custom = Type_custom
+
+let record_constr =
+  {
+    constr_descr = "a record type";
+    univ_descr = None;
+    satisfied =
+      (fun ~subtype:_ ~satisfies b ->
+        let m, b = split_meths b in
+        match b.descr with
+          | Var _ -> satisfies b
+          | Tuple [] when m = [] -> raise Unsatisfied_constraint
+          | Tuple [] -> ()
+          | _ -> raise Unsatisfied_constraint);
+  }
 
 let num_constr =
   {
-    t = Num;
     constr_descr = "a number type";
     univ_descr = None;
     satisfied =
@@ -34,16 +47,12 @@ let num_constr =
         let b = demeth b in
         match b.descr with
           | Var _ -> satisfies b
-          | Custom { typ = Ground.Never.Type }
-          | Custom { typ = Ground.Int.Type }
-          | Custom { typ = Ground.Float.Type } ->
-              ()
+          | Never | Int | Float -> ()
           | _ -> raise Unsatisfied_constraint);
   }
 
 let ord_constr =
   {
-    t = Ord;
     constr_descr = "an orderable type";
     univ_descr = None;
     satisfied =
@@ -51,7 +60,8 @@ let ord_constr =
         let m, b = split_meths b in
         match b.descr with
           | Var _ -> satisfies b
-          | Custom c when Ground_type.is_ground c.Type_base.typ -> ()
+          | Custom _ | Int | Float | String | Bool | Never -> ()
+          | Constr c -> List.iter (fun (_, t) -> satisfies t) c.params
           | Tuple [] ->
               (* For records, we want to ensure that all fields are ordered. *)
               List.iter
