@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -36,25 +36,6 @@ let forget_arguments = true
 
 type env = (string * scheme) list
 
-let rec hide_meth l a =
-  match (deref a).descr with
-    | Meth ({ meth = l' }, u) when l' = l -> hide_meth l u
-    | Meth (m, u) -> Type.make ?pos:a.pos (Meth (m, hide_meth l u))
-    | _ -> a
-
-let rec opt_meth l a =
-  match (deref a).descr with
-    | Meth (({ meth = l' } as m), u) when l' = l ->
-        Type.make ?pos:a.pos (Meth ({ m with optional = true }, u))
-    | Meth (m, u) -> Type.make ?pos:a.pos (Meth (m, opt_meth l u))
-    | _ -> a
-
-let rec get_meth l a =
-  match (deref a).descr with
-    | Meth (({ meth = l' } as meth), _) when l = l' -> meth
-    | Meth (_, a) -> get_meth l a
-    | _ -> assert false
-
 (** {1 Type generalization and instantiation}
 
     We don't have type schemes per se, but we compute generalizable variables
@@ -78,6 +59,7 @@ let filter_vars f t =
   let rec aux l t =
     let t = deref t in
     match t.descr with
+      | Int | Float | String | Bool | Never -> l
       | Custom c -> c.filter_vars aux l c.typ
       | Getter t -> aux l t
       | List { t } | Nullable t -> aux l t
@@ -90,7 +72,6 @@ let filter_vars f t =
       | Var { contents = Free var } ->
           if f var && not (List.exists (Var.eq var) l) then var :: l else l
       | Var { contents = Link _ } -> assert false
-      | _ -> raise NotImplemented
   in
   aux [] t
 
@@ -102,52 +83,16 @@ let generalizable ~level t = filter_vars (fun v -> v.level > level) t
 
 let generalize ~level t : scheme = (generalizable ~level t, t)
 
-(** Copy a term, substituting some EVars as indicated by a list of
-    associations. Other EVars are not copied, so sharing is preserved. *)
-let copy_with (subst : Subst.t) t =
-  let rec aux t =
-    let descr =
-      match t.descr with
-        | Var { contents = Free v } as var -> (
-            try (Subst.value subst v).descr with Not_found -> var)
-        | Constr c ->
-            let params = List.map (fun (v, t) -> (v, aux t)) c.params in
-            Constr { c with params }
-        | Custom c -> Custom { c with typ = c.copy_with aux c.typ }
-        | Getter t -> Getter (aux t)
-        | List { t; json_repr } -> List { t = aux t; json_repr }
-        | Nullable t -> Nullable (aux t)
-        | Tuple l -> Tuple (List.map aux l)
-        | Meth (({ scheme = g, t } as m), u) ->
-            (* We assume that we don't substitute generalized variables. *)
-            if !debug then
-              assert (Subst.M.for_all (fun v _ -> not (List.mem v g)) subst);
-            Meth ({ m with scheme = (g, aux t) }, aux u)
-        | Arrow (p, t) ->
-            Arrow (List.map (fun (o, l, t) -> (o, l, aux t)) p, aux t)
-        | Var { contents = Link (_, t) } ->
-            (* TODO: we remove the link here, it would be too difficult to preserve
-               sharing. We could at least keep it when no variable is changed. *)
-            (aux t).descr
-        | _ -> raise NotImplemented
-    in
-    Type.make ?pos:t.pos descr
-  in
-  if Subst.is_identity subst then t else aux t
-
 (** Instantiate a type scheme, given as a type together with a list of
-    generalized variables. Fresh variables are created with the given (current)
-    level, and attached to the appropriate constraints. This erases position
+    generalized variables. This erases position
     information, since they usually become irrelevant. *)
 let instantiate ~level ((generalized, t) : scheme) =
-  let subst =
-    List.map
-      (fun v ->
-        (v, var ~level ~constraints:(Constraints.elements v.constraints) ()))
-      generalized
-  in
-  let subst = Subst.of_list subst in
-  copy_with subst t
+  if generalized = [] then t
+  else (
+    let mapper =
+      Type.Fresh.init ~selector:(fun v -> List.memq v generalized) ~level ()
+    in
+    Type.Fresh.make mapper t)
 
 (** {1 Assignation} *)
 
@@ -158,6 +103,12 @@ exception Occur_check of var * t
     prepare the instantiation [a<-b] by adjusting the levels. *)
 let occur_check (a : var) =
   let rec occur_check = function
+    | { descr = Int }
+    | { descr = Float }
+    | { descr = String }
+    | { descr = Bool }
+    | { descr = Never } ->
+        ()
     | { descr = Constr c } -> List.iter (fun (_, x) -> occur_check x) c.params
     | { descr = Tuple l } -> List.iter occur_check l
     | { descr = Getter t } -> occur_check t
@@ -178,7 +129,6 @@ let occur_check (a : var) =
         if Type.Var.eq a x then raise (Occur_check (a, b));
         x.level <- min a.level x.level
     | { descr = Var { contents = Link (_, b) } } -> occur_check b
-    | _ -> raise NotImplemented
   in
   occur_check
 
@@ -232,9 +182,12 @@ let rec sup ~pos a b =
           with Incompatible -> sup a b)
       | None -> mk (Meth ({ m with optional = true }, sup a b))
   in
+  let a = deref a in
+  let b = deref b in
   if a == b then a
   else (
-    match ((deref a).descr, (deref b).descr) with
+    match (a.descr, b.descr) with
+      | v, v' when v == v' -> a
       | Var { contents = Free _ }, _ -> b
       | _, Var { contents = Free _ } -> a
       | Nullable a, Nullable b -> mk (Nullable (sup a b))
@@ -325,7 +278,7 @@ and bind ?(variance = `Invariant) a b =
   (* update_level a.level b; *)
   satisfies_constraints b (Constraints.elements a.constraints);
   let b = if b.pos = None then Type.make ?pos:a0.pos b.Type.descr else b in
-  v := Link (variance, b)
+  v.contents <- Link (variance, b)
 
 (** Ensure that the type for the method [l] in [a] is a subtype of the one for the same method in [b]. *)
 and unify_meth a b l =
@@ -355,7 +308,7 @@ and unify_meth a b l =
        - {foo?:int} <: {foo:int?} *)
    let s1 =
      match (optional1, optional2, (deref (snd s1)).descr) with
-       | true, true, t when Ground_type.Never.is_descr t -> s2
+       | true, true, Never -> s2
        | _, true, Nullable t -> (fst s1, t)
        | true, false, _ -> raise (Error (Repr.make a, Repr.make b))
        | _ -> s1
@@ -419,6 +372,7 @@ and ( <: ) a b =
     Printf.printf "\n%s <: %s\n%!" (Type.to_string a) (Type.to_string b);
   if a != b then (
     match (a.descr, b.descr) with
+      | a, b when a == b -> ()
       | Var { contents = Free v }, Var { contents = Free v' } when Var.eq v v'
         ->
           ()
@@ -434,10 +388,10 @@ and ( <: ) a b =
              failwith
                (Printf.sprintf "invalid sup: %s !< %s (%s)" (Type.to_string b')
                   (Type.to_string b'') (Printexc.to_string e)));
-          if b'' != b' then var := Link (`Covariant, b'');
+          if b'' != b' then var.contents <- Link (`Covariant, b'');
           a <: b''
       | Var ({ contents = Link (`Covariant, a') } as var), _ ->
-          var := Link (`Invariant, a');
+          var.contents <- Link (`Invariant, a');
           a <: b
       | _, Var { contents = Link (_, b) } -> a <: b
       | Var { contents = Link (_, a) }, _ -> a <: b
@@ -566,6 +520,8 @@ and ( <: ) a b =
       | Arrow ([], t1), Getter t2 -> (
           try t1 <: t2
           with Error (a, b) -> raise (Error (`Arrow ([], a), `Getter b)))
+      | Never, Var { contents = Free _ } | Var { contents = Free _ }, Never ->
+          raise (Error (Repr.make a, Repr.make b))
       | Var { contents = Free _ }, _ -> (
           try bind a b
           with Occur_check _ | Unsatisfied_constraint ->
@@ -602,8 +558,7 @@ and ( <: ) a b =
                           },
                           var () ));
                 a <: b
-            | _ when optional || Ground_type.Never.is_descr (deref t2).descr ->
-                a <: hide_meth l c
+            | _ when optional || (deref t2).descr = Never -> a <: hide_meth l c
             | _ ->
                 raise
                   (Error
