@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,9 +21,9 @@
  *****************************************************************************)
 
 module Runtime = Liquidsoap_lang.Runtime
-module Doc = Liquidsoap_lang.Doc
 module Environment = Liquidsoap_lang.Environment
 module Profiler = Liquidsoap_lang.Profiler
+module Queue = Liquidsoap_lang.Queues.Queue
 
 let usage =
   {|Usage : liquidsoap [OPTION, SCRIPT or EXPR]...
@@ -54,7 +54,8 @@ let force_start =
 let allow_root =
   Dtools.Conf.bool
     ~p:(Configure.conf_init#plug "allow_root")
-    ~d:false "Allow liquidsoap to run as root"
+    ~d:(Lazy.force Utils.is_docker)
+    "Allow liquidsoap to run as root"
     ~comments:
       [
         "This should be reserved for advanced dynamic uses of liquidsoap ";
@@ -68,14 +69,21 @@ let root_error () =
     | false, _, 0 -> Some "root guid (group)"
     | _ -> None
 
-(* Do not run, don't even check the scripts. *)
-let parse_only = ref false
-
-(* Should we load the stdlib? *)
-let stdlib = ref true
+let eval_mode : [ `Parse_only | `Parse_and_type | `Eval | `Eval_toplevel ] ref =
+  ref `Eval
 
 (* Should we error if stdlib is not found? *)
-let error_on_no_stdlib = not (Filename.is_relative Sys.argv.(0))
+let is_relative = Filename.is_relative Sys.argv.(0)
+
+(* Should we load the stdlib? *)
+let stdlib : Liquidsoap_lang.Lang_eval.stdlib ref =
+  ref (if is_relative then `If_present else `Force)
+
+(* Shall we use the cache *)
+let cache = ref true
+
+(* Display cache key. *)
+let show_cache_key = ref false
 
 (* Should we load the deprecated wrapper? *)
 let deprecated = ref true
@@ -83,128 +91,110 @@ let deprecated = ref true
 (* Shall we start an interactive interpreter (REPL) *)
 let interactive = ref false
 let log = Log.make ["main"]
+let to_load = Queue.create ()
 
-(** [load_libs] should be called before loading a script or looking up the
-    documentation, to make sure that pervasive libraries have been loaded,
-    unless the user explicitly opposed to it. *)
-let load_libs =
+let eval_script expr =
+  let open Liquidsoap_lang in
+  match !eval_mode with
+    | `Parse_only -> ignore (Runtime.parse expr)
+    | `Parse_and_type ->
+        let parsed_term, term = Runtime.parse expr in
+        ignore
+          (Lang.type_term ~name:"main script" ~parsed_term ~stdlib:!stdlib
+             ~trim:true ~deprecated:!deprecated term);
+        if !show_cache_key then
+          Printf.printf "Term cached with key %s\n"
+            (Parsed_term.hash parsed_term)
+    | (`Eval as v) | (`Eval_toplevel as v) ->
+        let toplevel = v = `Eval_toplevel in
+        let stdlib = !stdlib in
+        ignore
+          (Lang.eval ~toplevel ~cache:!cache ~stdlib ~deprecated:!deprecated
+             ~name:"main script" expr);
+        if not (Lang_eval.effective_toplevel ~stdlib toplevel) then
+          Environment.clear_toplevel_environments ()
+
+(** Evaluate the user script. *)
+let eval () =
   Lifecycle.load ();
   (* Register settings module. Needs to be done last to make sure every
      dependent OCaml module has been linked. *)
-  Lazy.force Builtins_settings.settings_module;
-  let loaded = ref false in
-  fun () ->
-    if !stdlib && not !loaded then (
-      try
-        let t = Sys.time () in
-        Runtime.load_libs ~error_on_no_stdlib ~deprecated:!deprecated
-          ~parse_only:!parse_only ();
-        log#important "Standard library loaded in %.02f seconds."
-          (Sys.time () -. t);
-        loaded := true
-      with Liquidsoap_lang.Runtime.Error ->
-        Dtools.Init.exec Dtools.Log.stop;
-        flush_all ();
-        exit 1)
-
-let last_item_lib = ref false
-
-(** Evaluate a script or expression.
-
-    This used to be done immediately, which made it possible to write things
-    like "liquidsoap myscript.liq -h myop" and get some doc on an operator.
-
-    Now, we delay each evaluation because the last item has to be treated as a
-    non-library, ie. all defined variables should be used. By default the last
-    item is (the only one) not treated as a library, for better diagnosis, but
-    this can be disabled (which is useful when --checking a lib).
-*)
-let do_eval, eval =
-  let delayed = ref None in
-  let eval src ~lib =
-    try
-      load_libs ();
-      match src with
-        | `StdIn -> Runtime.from_in_channel ~lib ~parse_only:!parse_only stdin
-        | `Expr_or_File expr when not (Sys.file_exists expr) ->
-            Runtime.from_string ~lib ~parse_only:!parse_only expr
-        | `Expr_or_File f ->
-            let basename = Filename.basename f in
-            let basename =
-              try Filename.chop_extension basename with _ -> basename
-            in
-            Utils.var_script := basename;
-            Startup.time ("Loaded " ^ f) (fun () ->
-                Runtime.from_file ~lib ~parse_only:!parse_only f)
-    with Liquidsoap_lang.Runtime.Error ->
-      Dtools.Init.exec Dtools.Log.stop;
-      flush_all ();
-      exit 1
+  Stdlib.Lazy.force Builtins_settings.settings_module;
+  let scripts = Queue.flush_elements to_load in
+  let script =
+    String.concat "\n"
+      (List.map
+         (function
+           | `Stdin -> "%include \"-\""
+           | `Expr_or_file expr when not (Sys.file_exists expr) ->
+               Printf.sprintf "%s" expr
+           | `Expr_or_file f -> Printf.sprintf "%%include %S" f)
+         scripts)
   in
-  let force ~lib =
-    match !delayed with
-      | Some f ->
-          f ~lib;
-          delayed := None
-      | None -> ()
-  in
-  ( force,
-    fun src ->
-      force ~lib:true;
-      delayed := Some (eval src) )
+  let t = Sys.time () in
+  try
+    eval_script script;
+    log#important "User script loaded in %.02f seconds." (Sys.time () -. t)
+  with Liquidsoap_lang.Runtime.Error ->
+    Dtools.Init.exec Dtools.Log.stop;
+    flush_all ();
+    exit 1
 
-let load_libs () =
-  do_eval ~lib:true;
-  load_libs ()
+let with_toplevel =
+  let do_run_streams = run_streams in
+  let do_exit = exit in
+  fun ?(exit = true) ?(run_streams = false) fn ->
+    do_run_streams := run_streams;
+    eval_mode := `Eval_toplevel;
+    eval ();
+    fn ();
+    if exit then do_exit 0
 
 let lang_doc name =
-  run_streams := false;
-  load_libs ();
-  try Lang_string.kprint_string ~pager:true (Doc.Value.print name)
-  with Not_found -> Printf.printf "Plugin not found!\n%!"
+  with_toplevel (fun () ->
+      try Lang_string.kprint_string ~pager:true (Doc.Value.print name)
+      with Not_found -> Printf.printf "Plugin not found!\n%!")
 
 let process_request s =
-  load_libs ();
-  run_streams := false;
-  let req = Request.create ~cue_in_metadata:None ~cue_out_metadata:None s in
-  match Request.resolve ~ctype:None req 20. with
-    | Request.Failed ->
-        Printf.eprintf "Request resolution failed.\n";
-        Request.destroy req;
-        flush_all ();
-        exit 2
-    | Request.Timeout ->
-        Printf.eprintf "Request resolution timeout.\n";
-        Request.destroy req;
-        flush_all ();
-        exit 1
-    | Request.Resolved ->
-        Request.read_metadata req;
-        let metadata =
-          Request.get_all_metadata req
-          |> Hashtbl.to_seq
-          |> Seq.map (fun (k, v) ->
-                 (k, if String.length v > 1024 then "<redacted>" else v))
-          |> Seq.map (fun (k, v) -> k ^ " = " ^ v)
-          |> List.of_seq |> String.concat "\n"
-        in
-        Printf.printf "%s\n" metadata;
-        Printf.printf "duration = %!";
-        begin
-          try
-            Printf.printf "%.2f s\n"
-              (Request.duration
-                 ~metadata:(Request.get_all_metadata req)
-                 (Option.get (Request.get_filename req)))
-          with Not_found -> Printf.printf "failed\n"
-        end;
-        Request.destroy req
+  with_toplevel (fun () ->
+      let req = Request.create ~cue_in_metadata:None ~cue_out_metadata:None s in
+      match Request.resolve req with
+        | `Failed ->
+            Printf.eprintf "Request resolution failed.\n";
+            Request.destroy req;
+            flush_all ();
+            exit 2
+        | `Timeout ->
+            Printf.eprintf "Request resolution timeout.\n";
+            Request.destroy req;
+            flush_all ();
+            exit 1
+        | `Resolved ->
+            let metadata =
+              Request.metadata req |> Frame.Metadata.to_list |> List.to_seq
+              |> Seq.map (fun (k, v) ->
+                     (k, if String.length v > 1024 then "<redacted>" else v))
+              |> Seq.map (fun (k, v) -> k ^ " = " ^ v)
+              |> List.of_seq |> String.concat "\n"
+            in
+            Printf.printf "%s\n" metadata;
+            Printf.printf "duration = %!";
+            begin
+              match
+                Request.duration ~metadata:(Request.metadata req)
+                  (Option.get (Request.get_filename req))
+              with
+                | Some f -> Printf.printf "%.2f s\n" f
+                | None -> Printf.printf "n/a\n"
+                | exception Not_found -> Printf.printf "failed\n"
+            end;
+            Request.destroy req)
 
 let format_doc s =
   let prefix = "\t  " in
   let indent = 8 + 2 in
   let max_width = 80 in
-  let s = Pcre.split ~pat:" " s in
+  let s = String.split_on_char ' ' s in
   let s =
     let rec join line width = function
       | [] -> [line]
@@ -224,7 +214,7 @@ let options =
   ref
     ([
        ( ["-"],
-         Arg.Unit (fun () -> eval `StdIn),
+         Arg.Unit (fun () -> Queue.push to_load `Stdin),
          "Read script from standard input." );
        ( ["-r"; "--request"],
          Arg.String process_request,
@@ -234,22 +224,29 @@ let options =
          "Get help about a scripting value: source, operator, builtin or \
           library function, etc." );
        ( ["-c"; "--check"],
-         Arg.Unit (fun () -> run_streams := false),
-         "Check and evaluate scripts but do not perform any streaming." );
-       ( ["-cl"; "--check-lib"],
-         Arg.Unit
-           (fun () ->
-             last_item_lib := true;
-             run_streams := false),
-         "Like --check but treats all scripts and expressions as libraries, so \
-          that unused toplevel variables are not reported." );
-       ( ["-p"; "--parse-only"],
          Arg.Unit
            (fun () ->
              run_streams := false;
-             deprecated := false;
-             parse_only := true),
-         "Parse scripts but do not type-check and run them." );
+             eval_mode := `Parse_and_type),
+         "Parse, type-check but do not evaluate the script." );
+       ( ["-p"; "--parse-only"],
+         Arg.Unit (fun () -> eval_mode := `Parse_only),
+         "Parse script but do not type-check and run them." );
+       ( ["--cache-stdlib"],
+         Arg.Unit (fun () -> with_toplevel (fun () -> ())),
+         "Generate the standard library cache." );
+       ( ["--cache-only"],
+         Arg.Unit
+           (fun () ->
+             run_streams := false;
+             eval_mode := `Parse_and_type;
+             show_cache_key := true;
+             cache := true),
+         "Parse, type-check and save script's cache but do no run it." );
+       (["--no-cache"], Arg.Clear cache, "Disable cache");
+       ( ["--no-external-plugins"],
+         Arg.Clear Startup.register_external_plugins,
+         "Disable external plugins." );
        ( ["-q"; "--quiet"],
          Arg.Unit (fun () -> Dtools.Log.conf_stdout#set false),
          "Do not print log messages on standard output." );
@@ -304,72 +301,68 @@ let options =
         ( ["--list-plugins"],
           Arg.Unit
             (fun () ->
-              run_streams := false;
-              load_libs ();
-              Lang_string.kprint_string ~pager:true Doc.Plug.print_string),
+              with_toplevel (fun () ->
+                  Lang_string.kprint_string ~pager:true Doc.Plug.print_string)),
           Printf.sprintf
             "List all plugins (builtin scripting values, supported formats and \
              protocols)." );
         ( ["--list-functions"],
           Arg.Unit
             (fun () ->
-              run_streams := false;
-              load_libs ();
-              Lang_string.kprint_string ~pager:true Doc.Value.print_functions),
+              with_toplevel (fun () ->
+                  Lang_string.kprint_string ~pager:true
+                    Doc.Value.print_functions)),
           Printf.sprintf "List all functions." );
         ( ["--list-functions-by-category"],
           Arg.Unit
             (fun () ->
-              run_streams := false;
-              load_libs ();
-              Lang_string.kprint_string ~pager:true
-                Doc.Value.print_functions_by_category),
+              with_toplevel (fun () ->
+                  Lang_string.kprint_string ~pager:true
+                    Doc.Value.print_functions_by_category)),
           Printf.sprintf "List all functions, sorted by category." );
         ( ["--list-functions-md"],
           Arg.Unit
             (fun () ->
-              run_streams := false;
-              load_libs ();
-              Lang_string.kprint_string ~pager:true
-                (Doc.Value.print_functions_md ~extra:false)),
+              with_toplevel (fun () ->
+                  Lang_string.kprint_string ~pager:true
+                    (Doc.Value.print_functions_md ~extra:false))),
           Printf.sprintf "Documentation of all functions in markdown format." );
         ( ["--list-functions-json"],
           Arg.Unit
             (fun () ->
-              run_streams := false;
-              load_libs ();
-              Lang_string.print_string ~pager:true
-                (Json.to_string ~compact:false (Doc.Value.to_json ()))),
+              with_toplevel (fun () ->
+                  Lang_string.print_string ~pager:true
+                    (Json.to_string ~compact:false (Doc.Value.to_json ())))),
           Printf.sprintf "Documentation of all functions in JSON format." );
         ( ["--list-extra-functions-md"],
           Arg.Unit
             (fun () ->
-              run_streams := false;
-              load_libs ();
-              Lang_string.kprint_string ~pager:true
-                (Doc.Value.print_functions_md ~extra:true)),
+              with_toplevel (fun () ->
+                  Lang_string.kprint_string ~pager:true
+                    (Doc.Value.print_functions_md ~extra:true))),
           Printf.sprintf "Documentation of all extra functions in markdown." );
         ( ["--list-deprecated-functions-md"],
           Arg.Unit
             (fun () ->
-              run_streams := false;
               deprecated := true;
-              load_libs ();
-              Lang_string.kprint_string ~pager:true
-                (Doc.Value.print_functions_md ~deprecated:true)),
+              with_toplevel (fun () ->
+                  Lang_string.kprint_string ~pager:true
+                    (Doc.Value.print_functions_md ~deprecated:true))),
           Printf.sprintf
             "Documentation of all deprecated functions in markdown." );
         ( ["--list-protocols-md"],
           Arg.Unit
             (fun () ->
-              run_streams := false;
-              load_libs ();
-              Lang_string.kprint_string ~pager:true Doc.Protocol.print_md),
+              with_toplevel (fun () ->
+                  Lang_string.kprint_string ~pager:true Doc.Protocol.print_md)),
           Printf.sprintf "Documentation of all protocols in markdown." );
         ( ["--no-stdlib"],
-          Arg.Clear stdlib,
+          Arg.Unit (fun () -> stdlib := `Disabled),
           Printf.sprintf "Do not load stdlib script libraries (i.e., %s/*.liq)."
             (Configure.liq_libs_dir ()) );
+        ( ["--stdlib"],
+          Arg.String (fun s -> stdlib := `Override s),
+          "Override the location of the standard library." );
         ( ["--no-deprecated"],
           Arg.Clear deprecated,
           "Do not load wrappers for deprecated operators." );
@@ -381,7 +374,7 @@ let options =
             (fun () ->
               Printf.printf
                 {|Liquidsoap %s
-Copyright (c) 2003-2023 Savonet team
+Copyright (c) 2003-2024 Savonet team
 Liquidsoap is open-source software, released under GNU General Public License.
 See <http://liquidsoap.info> for more information.
 |}
@@ -401,7 +394,10 @@ See <http://liquidsoap.info> for more information.
               exit 0),
           "Print out opam's liquidsoap.config, for internal use." );
         ( ["--interactive"],
-          Arg.Set interactive,
+          Arg.Unit
+            (fun () ->
+              interactive := true;
+              eval_mode := `Eval_toplevel),
           "Start an interactive interpreter." );
         ( ["--"],
           Arg.Unit (fun () -> Arg.current := Array.length Shebang.argv - 1),
@@ -410,10 +406,9 @@ See <http://liquidsoap.info> for more information.
         ( ["--list-settings"],
           Arg.Unit
             (fun () ->
-              load_libs ();
-              Lang_string.print_string ~pager:true
-                (Builtins_settings.print_settings ());
-              exit 0),
+              with_toplevel (fun () ->
+                  Lang_string.print_string ~pager:true
+                    (Builtins_settings.print_settings ()))),
           "Display configuration keys in markdown format." );
         ( ["--unsafe"],
           Arg.Unit (fun () -> Typing.do_occur_check := false),
@@ -421,6 +416,9 @@ See <http://liquidsoap.info> for more information.
         ( ["--safe"],
           Arg.Unit (fun () -> Typing.do_occur_check := true),
           "Disable the effects of --unsafe." );
+        ( ["--no-fallible-check"],
+          Arg.Unit (fun () -> Output.fallibility_check := false),
+          "Ignore fallible sources." );
       ])
 
 let expand_options options =
@@ -446,6 +444,12 @@ let parse argv l f msg =
         Lang_string.print_string ~pager:true msg;
         exit 0
 
+let parse_options () =
+  (* Parse command-line, and notably load scripts. *)
+  parse Shebang.argv (expand_options !options)
+    (fun s -> Queue.push to_load (`Expr_or_file s))
+    usage
+
 let absolute s =
   if String.length s > 0 && s.[0] <> '/' then Sys.getcwd () ^ "/" ^ s else s
 
@@ -487,10 +491,7 @@ We hope you enjoy this snapshot build of Liquidsoap!
          |> String.split_on_char '\n'));
 
   Lifecycle.on_script_parse ~name:"main application script parse" (fun () ->
-      (* Parse command-line, and notably load scripts. *)
-      parse Shebang.argv (expand_options !options)
-        (fun s -> eval (`Expr_or_File s))
-        usage;
+      parse_options ();
 
       if !interactive then (
         let default_log =
@@ -507,17 +508,16 @@ We hope you enjoy this snapshot build of Liquidsoap!
       (* Allow frame settings to be evaluated here: *)
       Frame_settings.lazy_config_eval := true;
 
-      do_eval ~lib:!last_item_lib)
+      eval ())
 
 let initial_cleanup () =
   if !Term.profile then
     log#important "Profiler stats:\n\n%s" (Profiler.stats ());
-  Clock.stop ();
   Tutils.cleanup ()
 
 let final_cleanup () =
   log#important "Cleaning downloaded files...";
-  Request.clean ();
+  Request.cleanup ();
   log#important "Freeing memory...";
   Dtools.Init.exec Dtools.Log.stop;
   flush_all ();
@@ -656,7 +656,7 @@ let () =
 
       if
         (not !interactive)
-        && (not (Source.has_outputs ()))
+        && List.length (Clock.clocks ()) = 0
         && not force_start#get
       then (
         final_cleanup ();
@@ -711,18 +711,15 @@ let () =
          an exception and dtools catches it so we don't get a backtrace (by
          default at least). *)
       Server.start ();
-      Clock.start ();
       Tutils.start ();
 
-      if !interactive then (
-        load_libs ();
+      if !interactive then
         ignore
           (Thread.create
              (fun () ->
                Runtime.interactive ();
                Tutils.shutdown 0)
-             ())));
-
+             ()));
   Lifecycle.on_main_loop ~name:"main application main loop" Tutils.main
 
 (* Here we go! *)
