@@ -152,11 +152,11 @@ and json_of_type_fun_arg (b, s, t) =
     ~extra:[("optional", `Bool b); ("label", `String s)]
     (json_of_type_annotation t)
 
-and json_of_meth_annotation { optional; name; typ; json_name } =
+and json_of_meth_annotation { optional_meth; name; typ; json_name } =
   type_node ~typ:"method_annotation"
     ~extra:
       [
-        ("optional", `Bool optional);
+        ("optional", `Bool optional_meth);
         ("name", `String name);
         ("json_name", match json_name with None -> `Null | Some n -> `String n);
       ]
@@ -302,6 +302,7 @@ let json_of_let_decoration ~to_json : Parsed_term.let_decoration -> Json.t =
       `Assoc (ast_node ~typ:"var" [("value", `String "sqlite.row")])
   | `Yaml_parse ->
       `Assoc (ast_node ~typ:"var" [("value", `String "yaml.parse")])
+  | `Xml_parse -> `Assoc (ast_node ~typ:"var" [("value", `String "xml.parse")])
   | `Json_parse [] ->
       `Assoc (ast_node ~typ:"var" [("value", `String "json.parse")])
   | `Json_parse args ->
