@@ -7,7 +7,7 @@ let send_called = ref false
 class encoded_test =
   object (self)
     inherit
-      Output.encoded
+      [unit] Output.encoded
         ~output_kind:"foo" ~name:"encoded_test" ~infallible:false
           ~register_telnet:false
         ~on_start:(fun _ -> ())
@@ -19,7 +19,7 @@ class encoded_test =
       assert !send_called;
       insert_metadata_called := true
 
-    method encode _ _ _ =
+    method encode _ =
       encode_called := true;
       ()
 
@@ -35,10 +35,11 @@ class encoded_test =
 let () =
   Frame_settings.lazy_config_eval := true;
   let encoded_test = new encoded_test in
-  let frame = Frame.dummy () in
-  Frame.add_break frame (Lazy.force Frame.size);
-  let m = Hashtbl.create 1 in
-  Hashtbl.add m "foo" "bla";
-  Frame.set_metadata frame 0 m;
+  encoded_test#content_type_computation_allowed;
+  let frame =
+    Frame.create ~length:(Lazy.force Frame.size) encoded_test#content_type
+  in
+  let m = Frame.Metadata.from_list [("foo", "bla")] in
+  let frame = Frame.add_metadata frame 0 m in
   encoded_test#test_send_frame frame;
   assert !insert_metadata_called
